/* A recursive-descent parser generated by greg 0.4.3 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
struct _GREG;
#define YYRULECOUNT 268

/* PEG Markdown Highlight
 * Copyright 2011-2016 Ali Rantakari -- http://hasseg.org
 * Licensed under the GPL2+ and MIT licenses (see LICENSE for more info).
 * 
 * pmh_grammar.leg
 * 
 * This is a slightly adapted version of the PEG grammar from John MacFarlane's
 * peg-markdown compiler.
 */

/* PEG Markdown Highlight
 * Copyright 2011-2016 Ali Rantakari -- http://hasseg.org
 * Licensed under the GPL2+ and MIT licenses (see LICENSE for more info).
 * 
 * pmh_parser_head.c
 * 
 * Code to be inserted into the beginning of the parser code generated
 * from the PEG grammar.
 */

#include "pmh_parser.h"

#ifndef pmh_DEBUG_OUTPUT
#define pmh_DEBUG_OUTPUT 0
#endif

#if pmh_DEBUG_OUTPUT
#define pmh_IF(x)           if (x)
#define pmh_PRINTF(x, ...)  fprintf(stderr, x, ##__VA_ARGS__)
#define pmh_PUTCHAR(x)      putchar(x)
#else
#define pmh_IF(x)
#define pmh_PRINTF(x, ...)
#define pmh_PUTCHAR(x)
#endif



// Alias strdup to _strdup on MSVC:
#ifdef _MSC_VER
#define strdup _strdup
#endif


char *strdup_or_null(char *s)
{
    return (s == NULL) ? NULL : strdup(s);
}


// Internal language element occurrence structure, containing
// both public and private members:
struct pmh_RealElement
{
    // "Public" members:
    // (these must match what's defined in pmh_definitions.h)
    // -----------------------------------------------
    pmh_element_type type;
    unsigned long pos;
    unsigned long end;
    struct pmh_RealElement *next;
    char *label;
    char *address;
    
    // "Private" members for use by the parser itself:
    // -----------------------------------------------
    
    // next element in list of all elements:
    struct pmh_RealElement *all_elems_next;
    
    // offset to text (for elements of type pmh_EXTRA_TEXT, used when the
    // parser reads the value of 'text'):
    int text_offset;
    
    // text content (for elements of type pmh_EXTRA_TEXT):
    char *text;
    
    // children of element (for elements of type pmh_RAW_LIST)
    struct pmh_RealElement *children;
};
typedef struct pmh_RealElement pmh_realelement;




// Parser state data:
typedef struct
{
    /* The original, unmodified UTF-8 input: */
    char *original_input;
    
    /* The offsets of the bytes we have stripped from original_input: */
    unsigned long *strip_positions;
    size_t strip_positions_len;
    
    /* Buffer of characters to be parsed: */
    char *charbuf;
    
    /* Linked list of {start, end} offset pairs determining which parts */
    /* of charbuf to actually parse: */
    pmh_realelement *current_elem;
    pmh_realelement *elem_head;
    
    /* Current parsing offset within charbuf: */
    unsigned long offset;
    
    /* The extensions to use for parsing (bitfield */
    /* of enum pmh_extensions): */
    int extensions;
    
    /* Array of parsing result elements, indexed by type: */
    pmh_realelement **head_elems;
    
    /* Whether we are parsing only references: */
    bool parsing_only_references;
    
    /* List of reference elements: */
    pmh_realelement *references;
} parser_data;

static parser_data *mk_parser_data(char *original_input,
                                   unsigned long *strip_positions,
                                   size_t strip_positions_len,
                                   char *charbuf,
                                   pmh_realelement *parsing_elems,
                                   unsigned long offset,
                                   int extensions,
                                   pmh_realelement **head_elems,
                                   pmh_realelement *references)
{
    parser_data *p_data = (parser_data *)malloc(sizeof(parser_data));
    p_data->extensions = extensions;
    p_data->original_input = original_input;
    p_data->strip_positions = strip_positions;
    p_data->strip_positions_len = strip_positions_len;
    p_data->charbuf = charbuf;
    p_data->offset = offset;
    p_data->elem_head = p_data->current_elem = parsing_elems;
    p_data->references = references;
    p_data->parsing_only_references = false;
    if (head_elems != NULL)
        p_data->head_elems = head_elems;
    else {
        p_data->head_elems = (pmh_realelement **)
                             malloc(sizeof(pmh_realelement *) * pmh_NUM_TYPES);
        int i;
        for (i = 0; i < pmh_NUM_TYPES; i++)
            p_data->head_elems[i] = NULL;
    }
    return p_data;
}


// Forward declarations
static void parse_markdown(parser_data *p_data);
static void parse_references(parser_data *p_data);





static char **get_element_type_names()
{
    static char **elem_type_names = NULL;
    if (elem_type_names == NULL)
    {
        elem_type_names = (char **)malloc(sizeof(char*) * pmh_NUM_LANG_TYPES);
        int i;
        for (i = 0; i < pmh_NUM_LANG_TYPES; i++)
            elem_type_names[i] = NULL;
        elem_type_names[pmh_LINK] = "LINK";
        elem_type_names[pmh_AUTO_LINK_URL] = "AUTO_LINK_URL";
        elem_type_names[pmh_AUTO_LINK_EMAIL] = "AUTO_LINK_EMAIL";
        elem_type_names[pmh_IMAGE] = "IMAGE";
        elem_type_names[pmh_CODE] = "CODE";
        elem_type_names[pmh_HTML] = "HTML";
        elem_type_names[pmh_HTML_ENTITY] = "HTML_ENTITY";
        elem_type_names[pmh_EMPH] = "EMPH";
        elem_type_names[pmh_STRONG] = "STRONG";
        elem_type_names[pmh_LIST_BULLET] = "LIST_BULLET";
        elem_type_names[pmh_LIST_ENUMERATOR] = "LIST_ENUMERATOR";
        elem_type_names[pmh_COMMENT] = "COMMENT";
        elem_type_names[pmh_H1] = "H1";
        elem_type_names[pmh_H2] = "H2";
        elem_type_names[pmh_H3] = "H3";
        elem_type_names[pmh_H4] = "H4";
        elem_type_names[pmh_H5] = "H5";
        elem_type_names[pmh_H6] = "H6";
        elem_type_names[pmh_BLOCKQUOTE] = "BLOCKQUOTE";
        elem_type_names[pmh_VERBATIM] = "VERBATIM";
        elem_type_names[pmh_HTMLBLOCK] = "HTMLBLOCK";
        elem_type_names[pmh_HRULE] = "HRULE";
        elem_type_names[pmh_REFERENCE] = "REFERENCE";
        elem_type_names[pmh_FENCEDCODEBLOCK] = "FENCEDCODEBLOCK";
        elem_type_names[pmh_NOTE] = "NOTE";
        elem_type_names[pmh_STRIKE] = "STRIKE";
        elem_type_names[pmh_FRONTMATTER] = "FRONTMATTER";
        elem_type_names[pmh_DISPLAYFORMULA] = "DISPLAYFORMULA";
        elem_type_names[pmh_INLINEEQUATION] = "INLINEEQUATION";
        elem_type_names[pmh_MARK] = "MARK";
        elem_type_names[pmh_TABLE] = "TABLE";
        elem_type_names[pmh_TABLEHEADER] = "TABLEHEADER";
        elem_type_names[pmh_TABLEBORDER] = "TABLEBORDER";
    }
    return elem_type_names;
}

pmh_element_type pmh_element_type_from_name(char *name)
{
    char **elem_type_names = get_element_type_names();
    
    int i;
    for (i = 0; i < pmh_NUM_LANG_TYPES; i++)
    {
        char *i_name = elem_type_names[i];
        if (i_name == NULL)
            continue;
        if (strcmp(i_name, name) == 0)
            return i;
    }
    
    return pmh_NO_TYPE;
}

char *pmh_element_name_from_type(pmh_element_type type)
{
    char **elem_type_names = get_element_type_names();
    char* ret = elem_type_names[type];
    if (ret == NULL)
        return "unknown type";
    return ret;
}




/*
Remove pmh_RAW elements with zero length; return pointer
to new head.
*/
static pmh_realelement *remove_zero_length_raw_spans(pmh_realelement *elem)
{
    pmh_realelement *head = elem;
    pmh_realelement *parent = NULL;
    pmh_realelement *c = head;
    while (c != NULL)
    {
        if (c->type == pmh_RAW && c->pos >= c->end)
        {
            if (parent != NULL)
                parent->next = c->next;
            else
                head = c->next;
            parent = c;
            c = c->next;
            continue;
        }
        parent = c;
        c = c->next;
    }
    return head;
}

#if pmh_DEBUG_OUTPUT
/*
Print null-terminated string s.t. some characters are
represented by their corresponding espace sequences
*/
static void print_str_literal_escapes(char *str)
{
    char *c = str;
    pmh_PRINTF("'");
    while (*c != '\0')
    {
        if (*c == '\n')         pmh_PRINTF("\\n");
        else if (*c == '\t')    pmh_PRINTF("\\t");
        else putchar(*c);
        c++;
    }
    pmh_PRINTF("'");
}
#endif

#if pmh_DEBUG_OUTPUT
/*
Print elements in a linked list of
pmh_RAW, pmh_SEPARATOR, pmh_EXTRA_TEXT elements
*/
static void print_raw_spans_inline(pmh_realelement *elem)
{
    pmh_realelement *cur = elem;
    while (cur != NULL)
    {
        if (cur->type == pmh_SEPARATOR)
            pmh_PRINTF("<pmh_SEP %ld> ", cur->pos);
        else if (cur->type == pmh_EXTRA_TEXT) {
            pmh_PRINTF("{pmh_ETEXT ");
            print_str_literal_escapes(cur->text);
            pmh_PRINTF("}");
        }
        else
            pmh_PRINTF("(%ld-%ld) ", cur->pos, cur->end);
        cur = cur->next;
    }
}
#endif

/*
Perform postprocessing parsing runs for pmh_RAW_LIST elements in `elem`,
iteratively until no such elements exist.
*/
static void process_raw_blocks(parser_data *p_data)
{
    pmh_PRINTF("--------process_raw_blocks---------\n");
    while (p_data->head_elems[pmh_RAW_LIST] != NULL)
    {
        pmh_PRINTF("new iteration.\n");
        pmh_realelement *cursor = p_data->head_elems[pmh_RAW_LIST];
        p_data->head_elems[pmh_RAW_LIST] = NULL;
        while (cursor != NULL)
        {
            pmh_realelement *span_list = (pmh_realelement*)cursor->children;
            
            span_list = remove_zero_length_raw_spans(span_list);
            
            #if pmh_DEBUG_OUTPUT
            pmh_PRINTF("  process: ");
            print_raw_spans_inline(span_list);
            pmh_PRINTF("\n");
            #endif
            
            while (span_list != NULL)
            {
                pmh_PRINTF("next: span_list: %ld-%ld\n",
                           span_list->pos, span_list->end);
                
                // Skip separators in the beginning, as well as
                // separators after another separator:
                if (span_list->type == pmh_SEPARATOR) {
                    span_list = span_list->next;
                    continue;
                }
                
                // Store list of spans until next separator in subspan_list:
                pmh_realelement *subspan_list = span_list;
                pmh_realelement *previous = NULL;
                while (span_list != NULL && span_list->type != pmh_SEPARATOR) {
                    previous = span_list;
                    span_list = span_list->next;
                }
                if (span_list != NULL && span_list->type == pmh_SEPARATOR) {
                    span_list = span_list->next;
                    previous->next = NULL;
                }
                
                #if pmh_DEBUG_OUTPUT
                pmh_PRINTF("    subspan process: ");
                print_raw_spans_inline(subspan_list);
                pmh_PRINTF("\n");
                #endif
                
                // Process subspan_list:
                parser_data *raw_p_data = mk_parser_data(
                    p_data->original_input,
                    p_data->strip_positions,
                    p_data->strip_positions_len,
                    p_data->charbuf,
                    subspan_list,
                    subspan_list->pos,
                    p_data->extensions,
                    p_data->head_elems,
                    p_data->references
                );
                parse_markdown(raw_p_data);
                free(raw_p_data);
                
                pmh_PRINTF("parse over\n");
            }
            
            cursor = cursor->next;
        }
    }
}


#if pmh_DEBUG_OUTPUT
static void print_raw_blocks(char *text, pmh_realelement *elem[])
{
    pmh_PRINTF("--------print_raw_blocks---------\n");
    pmh_PRINTF("block:\n");
    pmh_realelement *cursor = elem[pmh_RAW_LIST];
    while (cursor != NULL)
    {
        print_raw_spans_inline(cursor->children);
        cursor = cursor->next;
    }
}
#endif




/* Free all elements created while parsing */
void pmh_free_elements(pmh_element **elems)
{
    pmh_realelement *cursor = (pmh_realelement*)elems[pmh_ALL];
    while (cursor != NULL) {
        pmh_realelement *tofree = cursor;
        cursor = cursor->all_elems_next;
        if (tofree->text != NULL)
            free(tofree->text);
        if (tofree->label != NULL)
            free(tofree->label);
        if (tofree->address != NULL)
            free(tofree->address);
        free(tofree);
    }
    elems[pmh_ALL] = NULL;
    
    free(elems);
}






#define IS_CONTINUATION_BYTE(x) ((x & 0xC0) == 0x80)
#define HAS_UTF8_BOM(x)         ( ((*x & 0xFF) == 0xEF)\
                                  && ((*(x+1) & 0xFF) == 0xBB)\
                                  && ((*(x+2) & 0xFF) == 0xBF) )
#define ADD_STRIP_POS(x) \
    /* reallocate more space for the array, if needed: */ \
    if (strip_positions_size <= strip_positions_pos) { \
        size_t new_size = strip_positions_size * 2; \
        unsigned long *new_arr = (unsigned long *) \
                                 calloc(new_size, \
                                        sizeof(unsigned long)); \
        memcpy(new_arr, strip_positions, \
               (sizeof(unsigned long) * strip_positions_size)); \
        strip_positions_size = new_size; \
        free(strip_positions); \
        strip_positions = new_arr; \
    } \
    strip_positions[strip_positions_pos] = x; \
    strip_positions_pos++;

/*
Copy `str` to `out`, while doing the following:
  - remove UTF-8 continuation bytes
  - remove possible UTF-8 BOM (byte order mark)
  - append two newlines to the end (like peg-markdown does)
  - keep track of which bytes we have stripped (in strip_positions)
*/
static int strcpy_preformat(char *str, char **out,
                            unsigned long **out_strip_positions,
                            size_t *out_strip_positions_len)
{
    size_t strip_positions_size = 1024;
    size_t strip_positions_pos = 0;
    unsigned long *strip_positions = (unsigned long *)
                                     calloc(strip_positions_size,
                                            sizeof(unsigned long));
    
    
    // +2 in the following is due to the "\n\n" suffix:
    char *new_str = (char *)malloc(sizeof(char) * strlen(str) + 1 + 2);
    char *c = str;
    int i = 0;
    
    if (HAS_UTF8_BOM(c)) {
        c += 3;
        ADD_STRIP_POS(0);
        ADD_STRIP_POS(1);
        ADD_STRIP_POS(2);
    }
    
    while (*c != '\0')
    {
        if (!IS_CONTINUATION_BYTE(*c)) {
            *(new_str+i) = *c, i++;
        } else {
            ADD_STRIP_POS((int)(c-str));
        }
        c++;
    }
    
    *(new_str+(i++)) = '\n';
    *(new_str+(i++)) = '\n';
    *(new_str+i) = '\0';
    
    *out = new_str;
    *out_strip_positions = strip_positions;
    *out_strip_positions_len = strip_positions_pos;
    return i;
}



void pmh_markdown_to_elements(char *text, int extensions,
                              pmh_element **out_result[])
{
    char *text_copy = NULL;
    unsigned long *strip_positions = NULL;
    size_t strip_positions_len = 0;
    int text_copy_len = strcpy_preformat(text, &text_copy, &strip_positions,
                                         &strip_positions_len);
    
    pmh_realelement *parsing_elem = (pmh_realelement *)
                                    malloc(sizeof(pmh_realelement));
    parsing_elem->type = pmh_RAW;
    parsing_elem->pos = 0;
    parsing_elem->end = text_copy_len;
    parsing_elem->next = NULL;
    
    parser_data *p_data = mk_parser_data(
        text,
        strip_positions,
        strip_positions_len,
        text_copy,
        parsing_elem,
        0,
        extensions,
        NULL,
        NULL
    );
    pmh_realelement **result = p_data->head_elems;
    
    if (*text_copy != '\0')
    {
        // Get reference definitions into p_data->references
        parse_references(p_data);
        
        // Reset parser state to beginning of input
        p_data->offset = 0;
        p_data->current_elem = p_data->elem_head;
        
        // Parse whole document
        parse_markdown(p_data);
        
        #if pmh_DEBUG_OUTPUT
        print_raw_blocks(text_copy, result);
        #endif
        
        process_raw_blocks(p_data);
    }
    
    free(strip_positions);
    free(p_data);
    free(parsing_elem);
    free(text_copy);
    
    *out_result = (pmh_element**)result;
}



/*
Mergesort linked list of elements (using comparison function `compare`),
return new head. (Adapted slightly from Simon Tatham's algorithm.)
*/
static pmh_element *ll_mergesort(pmh_element *list,
                                 int (*compare)(const pmh_element*,
                                                const pmh_element*))
{
    if (!list)
        return NULL;
    
    pmh_element *out_head = list;
    
    /* Merge widths of doubling size until done */
    int merge_width = 1;
    while (1)
    {
        pmh_element *l, *r; /* left & right segment pointers */
        pmh_element *tail = NULL; /* tail of sorted section */
        
        l = out_head;
        out_head = NULL;
        
        int merge_count = 0;
        
        while (l)
        {
            merge_count++;
            
            /* Position r, determine lsize & rsize */
            r = l;
            int lsize = 0;
            int i;
            for (i = 0; i < merge_width; i++) {
                lsize++;
                r = r->next;
                if (!r)
                    break;
            }
            int rsize = merge_width;
            
            /* Merge l & r */
            while (lsize > 0 || (rsize > 0 && r))
            {
                bool get_from_left = false;
                if (lsize == 0)             get_from_left = false;
                else if (rsize == 0 || !r)  get_from_left = true;
                else if (compare(l,r) <= 0) get_from_left = true;
                
                pmh_element *e;
                if (get_from_left) {
                    e = l; l = l->next; lsize--;
                } else {
                    e = r; r = r->next; rsize--;
                }
                
                /* add the next pmh_element to the merged list */
                if (tail)
                    tail->next = e;
                else
                    out_head = e;
                tail = e;
            }
            
            l = r;
        }
        tail->next = NULL;
        
        if (merge_count <= 1)
            return out_head;
        
        merge_width *= 2;
    }
}

static int elem_compare_by_pos(const pmh_element *a, const pmh_element *b)
{
    return a->pos - b->pos;
}

void pmh_sort_elements_by_pos(pmh_element *element_lists[])
{
    int i;
    for (i = 0; i < pmh_NUM_LANG_TYPES; i++)
        element_lists[i] = ll_mergesort(element_lists[i], &elem_compare_by_pos);
}








/* return true if extension is selected */
static bool extension(parser_data *p_data, int ext)
{
    return ((p_data->extensions & ext) != 0);
}

/* return reference pmh_realelement for a given label */
static pmh_realelement *get_reference(parser_data *p_data, char *label)
{
    if (!label)
        return NULL;
    
    pmh_realelement *cursor = p_data->references;
    while (cursor != NULL)
    {
        if (cursor->label && strcmp(label, cursor->label) == 0)
            return cursor;
        cursor = cursor->next;
    }
    return NULL;
}


/* cons an element/list onto a list, returning pointer to new head */
static pmh_realelement *cons(pmh_realelement *elem, pmh_realelement *list)
{
    assert(elem != NULL);
    
    pmh_realelement *cur = elem;
    while (cur->next != NULL) {
        cur = cur->next;
    }
    cur->next = list;
    
    return elem;
}


/* reverse a list, returning pointer to new list */
static pmh_realelement *reverse(pmh_realelement *list)
{
    pmh_realelement *new_head = NULL;
    pmh_realelement *next = NULL;
    while (list != NULL) {
        next = list->next;
        list->next = new_head;
        new_head = list;
        list = next;
    }
    return new_head;
}



/* construct pmh_realelement */
static pmh_realelement *mk_element(parser_data *p_data, pmh_element_type type,
                                   long pos, long end)
{
    pmh_realelement *result = (pmh_realelement *)malloc(sizeof(pmh_realelement));
    memset(result, 0, sizeof(*result));
    result->type = type;
    result->pos = pos;
    result->end = end;
    
    pmh_realelement *old_all_elements_head = p_data->head_elems[pmh_ALL];
    p_data->head_elems[pmh_ALL] = result;
    result->all_elems_next = old_all_elements_head;
    
    //pmh_PRINTF("  mk_element: %s [%ld - %ld]\n", pmh_element_name_from_type(type), pos, end);
    
    return result;
}

static pmh_realelement *copy_element(parser_data *p_data, pmh_realelement *elem)
{
    pmh_realelement *result = mk_element(p_data, elem->type, elem->pos, elem->end);
    result->label = strdup_or_null(elem->label);
    result->text = strdup_or_null(elem->text);
    result->address = strdup_or_null(elem->address);
    return result;
}

/* construct pmh_EXTRA_TEXT pmh_realelement */
static pmh_realelement *mk_etext(parser_data *p_data, char *string)
{
    pmh_realelement *result;
    assert(string != NULL);
    result = mk_element(p_data, pmh_EXTRA_TEXT, 0,0);
    result->text = strdup_or_null(string);
    return result;
}


/*
Given an element where the offsets {pos, end} represent
locations in the *parsed text* (defined by the linked list of pmh_RAW and
pmh_EXTRA_TEXT elements in p_data->current_elem), fix these offsets to represent
corresponding offsets in the parse buffer (p_data->charbuf). Also split
the given pmh_realelement into multiple parts if its offsets span multiple
p_data->current_elem elements. Return the (list of) elements with real offsets.
*/
static pmh_realelement *fix_offsets(parser_data *p_data, pmh_realelement *elem)
{
    if (elem->type == pmh_EXTRA_TEXT)
        return mk_etext(p_data, elem->text);
    
    pmh_realelement *new_head = copy_element(p_data, elem);
    
    pmh_realelement *tail = new_head;
    pmh_realelement *prev = NULL;
    
    bool found_start = false;
    bool found_end = false;
    bool tail_needs_pos = false;
    unsigned long previous_end = 0;
    unsigned long c = 0;
    
    pmh_realelement *cursor = p_data->elem_head;
    while (cursor != NULL)
    {
        int thislen = (cursor->type == pmh_EXTRA_TEXT)
                        ? strlen(cursor->text)
                        : cursor->end - cursor->pos;
        
        if (tail_needs_pos && cursor->type != pmh_EXTRA_TEXT) {
            tail->pos = cursor->pos;
            tail_needs_pos = false;
        }
        
        unsigned int this_pos = cursor->pos;
        
        if (!found_start && (c <= elem->pos && elem->pos <= c+thislen)) {
            tail->pos = (cursor->type == pmh_EXTRA_TEXT)
                        ? previous_end
                        : cursor->pos + (elem->pos - c);
            this_pos = tail->pos;
            found_start = true;
        }
        
        if (!found_end && (c <= elem->end && elem->end <= c+thislen)) {
            tail->end = (cursor->type == pmh_EXTRA_TEXT)
                        ? previous_end
                        : cursor->pos + (elem->end - c);
            found_end = true;
        }
        
        if (found_start && found_end)
            break;
        
        if (cursor->type != pmh_EXTRA_TEXT)
            previous_end = cursor->end;
        
        if (found_start) {
            pmh_realelement *new_elem = copy_element(p_data, tail);
            new_elem->pos = this_pos;
            new_elem->end = cursor->end;
            new_elem->next = tail;
            if (prev != NULL)
                prev->next = new_elem;
            if (new_head == tail)
                new_head = new_elem;
            prev = new_elem;
            tail_needs_pos = true;
        }
        
        c += thislen;
        cursor = cursor->next;
    }
    
    return new_head;
}



/* Add an element to p_data->head_elems. */
static void add(parser_data *p_data, pmh_realelement *elem)
{
    if (elem->type != pmh_RAW_LIST)
    {
        pmh_PRINTF("  add: %s [%ld - %ld]\n",
                   pmh_element_name_from_type(elem->type), elem->pos, elem->end);
        elem = fix_offsets(p_data, elem);
        pmh_PRINTF("     > %s [%ld - %ld]\n",
                   pmh_element_name_from_type(elem->type), elem->pos, elem->end);
    }
    else
    {
        pmh_PRINTF("  add: pmh_RAW_LIST ");
        pmh_realelement *cursor = elem->children;
        pmh_realelement *previous = NULL;
        while (cursor != NULL)
        {
            pmh_realelement *next = cursor->next;
            pmh_PRINTF("(%ld-%ld)>", cursor->pos, cursor->end);
            pmh_realelement *new_cursor = fix_offsets(p_data, cursor);
            if (previous != NULL)
                previous->next = new_cursor;
            else
                elem->children = new_cursor;
            pmh_PRINTF("(%ld-%ld)", new_cursor->pos, new_cursor->end);
            while (new_cursor->next != NULL) {
                new_cursor = new_cursor->next;
                pmh_PRINTF("(%ld-%ld)", new_cursor->pos, new_cursor->end);
            }
            pmh_PRINTF(" ");
            if (next != NULL)
                new_cursor->next = next;
            previous = new_cursor;
            cursor = next;
        }
        pmh_PRINTF("\n");
    }
    
    if (p_data->head_elems[elem->type] == NULL)
        p_data->head_elems[elem->type] = elem;
    else
    {
        pmh_realelement *last = elem;
        while (last->next != NULL)
            last = last->next;
        last->next = p_data->head_elems[elem->type];
        p_data->head_elems[elem->type] = elem;
    }
}


// Given a range in the list of spans we use for parsing (pos, end), return
// a copy of the corresponding section in the original input, with all of
// the UTF-8 bytes intact:
static char *copy_input_span(parser_data *p_data,
                             unsigned long pos, unsigned long end)
{
    if (end <= pos)
        return NULL;
    
    char *ret = NULL;
    
    // Adjust (pos,end) to match actual indexes in charbuf:
    pmh_realelement *dummy = mk_element(p_data, pmh_NO_TYPE, pos, end);
    pmh_realelement *fixed_dummies = fix_offsets(p_data, dummy);
    pmh_realelement *cursor = fixed_dummies;
    while (cursor != NULL)
    {
        if (cursor->end <= cursor->pos)
        {
            cursor = cursor->next;
            continue;
        }
        
        // Adjust cursor's span to take bytes stripped from the original
        // input into account (i.e. match the corresponding span in
        // p_data->original_input):
        unsigned long adjusted_pos = cursor->pos;
        unsigned long adjusted_end = cursor->end;
        size_t i;
        for (i = 0; i < p_data->strip_positions_len; i++)
        {
            unsigned long strip_position = p_data->strip_positions[i];
            if (strip_position <= adjusted_pos)
                adjusted_pos++;
            if (strip_position <= adjusted_end)
                adjusted_end++;
            else
                break;
        }
        
        // Copy span from original input:
        size_t adjusted_len = adjusted_end - adjusted_pos;
        char *str = (char *)malloc(sizeof(char)*adjusted_len + 1);
        *str = '\0';
        strncat(str, (p_data->original_input + adjusted_pos), adjusted_len);
        
        if (ret == NULL)
            ret = str;
        else
        {
            // append str to ret:
            char *new_ret = (char *)malloc(sizeof(char)
                                           *(strlen(str) + strlen(ret)) + 1);
            *new_ret = '\0';
            strcat(new_ret, ret);
            strcat(new_ret, str);
            free(ret);
            free(str);
            ret = new_ret;
        }
        
        cursor = cursor->next;
    }
    
    return ret;
}





# define YYSTYPE pmh_realelement *
#ifdef __DEBUG__
# define YY_DEBUG 1
#endif

#define YY_INPUT(buf, result, max_size)\
        yy_input_func(buf, &result, max_size, (parser_data *)G->data)

static void yy_input_func(char *buf, int *result, int max_size,
                          parser_data *p_data)
{
    if (p_data->current_elem == NULL)
    {
        (*result) = 0;
        return;
    }
    
    if (p_data->current_elem->type == pmh_EXTRA_TEXT)
    {
        int yyc;
        bool moreToRead = (p_data->current_elem->text
                           && *(p_data->current_elem->text
                                + p_data->current_elem->text_offset) != '\0');
        if (moreToRead)
        {
            yyc = *(p_data->current_elem->text + p_data->current_elem->text_offset++);
            pmh_PRINTF("\e[47;30m"); pmh_PUTCHAR(yyc); pmh_PRINTF("\e[0m");
            pmh_IF(yyc == '\n') pmh_PRINTF("\e[47m \e[0m");
        }
        else
        {
            yyc = EOF;
            p_data->current_elem = p_data->current_elem->next;
            pmh_PRINTF("\e[41m \e[0m");
            if (p_data->current_elem != NULL)
                p_data->offset = p_data->current_elem->pos;
        }
        (*result) = (EOF == yyc) ? 0 : (*(buf) = yyc, 1);
        return;
    }
    
    *(buf) = *(p_data->charbuf + p_data->offset);
    (*result) = (*buf != '\0');
    p_data->offset++;
    
    pmh_PRINTF("\e[43;30m"); pmh_PUTCHAR(*buf); pmh_PRINTF("\e[0m");
    pmh_IF(*buf == '\n') pmh_PRINTF("\e[42m \e[0m");
    
    if (p_data->offset >= p_data->current_elem->end)
    {
        p_data->current_elem = p_data->current_elem->next;
        pmh_PRINTF("\e[41m \e[0m");
        if (p_data->current_elem != NULL)
            p_data->offset = p_data->current_elem->pos;
    }
}



#define elem(x)     mk_element((parser_data *)G->data, x, thunk->begin, thunk->end)
#define elem_s(x)   mk_element((parser_data *)G->data, x, s->pos, thunk->end)
#define mk_sep      mk_element((parser_data *)G->data, pmh_SEPARATOR, 0,0)
#define mk_notype   mk_element((parser_data *)G->data, pmh_NO_TYPE, 0,0)
#define etext(x)    mk_etext((parser_data *)G->data, x)
#define ADD(x)      add((parser_data *)G->data, x)
#define EXT(x)      extension((parser_data *)G->data, x)
#define REF_EXISTS(x) reference_exists((parser_data *)G->data, x)
#define GET_REF(x)  get_reference((parser_data *)G->data, x)
#define PARSING_REFERENCES ((parser_data *)G->data)->parsing_only_references
#define FREE_LABEL(l) { free(l->label); l->label = NULL; }
#define FREE_ADDRESS(l) { free(l->address); l->address = NULL; }

// This gives us the text matched with < > as it appears in the original input:
#define COPY_YYTEXT_ORIG() copy_input_span((parser_data *)G->data, thunk->begin, thunk->end)

// pos is the postition of next char to parse after matching one $
// Do not allow $, digit, and \ before opening $
bool inline_equation_predict_pre(char *buf, int pos)
{
    if (pos < 2) {
        return true;
    }

    unsigned char ch = buf[pos - 2];
    return ch != '$' && (ch < '0' || ch > '9') && ch != '\\';
}

// Do not allow \, space before ending $
bool inline_equation_predict_post(char *buf, int pos)
{
    unsigned char ch = buf[pos - 2];
    return ch != '\\' && ch != ' ' && ch != '\t';
}

#define IEP_PRE         inline_equation_predict_pre(G->buf, G->pos)
#define IEP_POST        inline_equation_predict_post(G->buf, G->pos)

// Do not allow whitespace before ending ~~
bool strike_predict_post(char *buf, int pos)
{
    unsigned char ch = buf[pos - 1];
    return ch != '\n' && ch != '\r' && ch != ' ' && ch != '\t';
}

#define STRIKE_POST     strike_predict_post(G->buf, G->pos)

// Number of display formula raw opening
unsigned char nr_dfr = 0;

bool start_dfr()
{
    nr_dfr = 1;
    return true;
}

bool inc_dfr()
{
    ++nr_dfr;
    return true;
}

bool dec_dfr()
{
    --nr_dfr;
    return true;
}

bool nested_dfr()
{
    return nr_dfr > 1;
}


#ifndef YY_ALLOC
#define YY_ALLOC(N, D) malloc(N)
#endif
#ifndef YY_CALLOC
#define YY_CALLOC(N, S, D) calloc(N, S)
#endif
#ifndef YY_REALLOC
#define YY_REALLOC(B, N, D) realloc(B, N)
#endif
#ifndef YY_FREE
#define YY_FREE free
#endif
#ifndef YY_LOCAL
#define YY_LOCAL(T)     static T
#endif
#ifndef YY_ACTION
#define YY_ACTION(T)    static T
#endif
#ifndef YY_RULE
#define YY_RULE(T)      static T
#endif
#ifndef YY_PARSE
#define YY_PARSE(T)     T
#endif
#ifndef YY_NAME
#define YY_NAME(N) yy##N
#endif
#ifndef YY_INPUT
#define YY_INPUT(buf, result, max_size)                 \
  {                                                     \
    int yyc= getchar();                                 \
    result= (EOF == yyc) ? 0 : (*(buf)= yyc, 1);        \
    yyprintf((stderr, "<%c>", yyc));                  \
  }
#endif
#ifndef YY_BEGIN
#define YY_BEGIN        ( G->begin= G->pos, 1)
#endif
#ifndef YY_END
#define YY_END          ( G->end= G->pos, 1)
#endif
#ifdef YY_DEBUG
# define yyprintf(args) fprintf args
#else
# define yyprintf(args)
#endif
#ifndef YYSTYPE
#define YYSTYPE int
#endif
#ifndef YY_XTYPE
#define YY_XTYPE void *
#endif
#ifndef YY_XVAR
#define YY_XVAR yyxvar
#endif

#ifndef YY_STACK_SIZE
#define YY_STACK_SIZE 128
#endif

#ifndef YY_BUFFER_START_SIZE
#define YY_BUFFER_START_SIZE 1024
#endif

#ifndef YY_PART
#define yydata G->data
#define yy G->ss

struct _yythunk; // forward declaration
typedef void (*yyaction)(struct _GREG *G, char *yytext, int yyleng, struct _yythunk *thunkpos, YY_XTYPE YY_XVAR);
typedef struct _yythunk { int begin, end;  yyaction  action;  struct _yythunk *next; } yythunk;

typedef struct _GREG {
  char *buf;
  int buflen;
  int   offset;
  int   pos;
  int   limit;
  char *text;
  int   textlen;
  int   begin;
  int   end;
  yythunk *thunks;
  int   thunkslen;
  int thunkpos;
  YYSTYPE ss;
  YYSTYPE *val;
  YYSTYPE *vals;
  int valslen;
  YY_XTYPE data;
} GREG;

YY_LOCAL(int) yyrefill(GREG *G)
{
  int yyn;
  while (G->buflen - G->pos < 512)
    {
      G->buflen *= 2;
      G->buf= (char *)YY_REALLOC(G->buf, G->buflen, G->data);
    }
  YY_INPUT((G->buf + G->pos), yyn, (G->buflen - G->pos));
  if (!yyn) return 0;
  G->limit += yyn;
  return 1;
}

YY_LOCAL(int) yymatchDot(GREG *G)
{
  if (G->pos >= G->limit && !yyrefill(G)) return 0;
  ++G->pos;
  return 1;
}

YY_LOCAL(int) yymatchChar(GREG *G, int c)
{
  if (G->pos >= G->limit && !yyrefill(G)) return 0;
  if ((unsigned char)G->buf[G->pos] == c)
    {
      ++G->pos;
      yyprintf((stderr, "  ok   yymatchChar(%c) @ %s\n", c, G->buf+G->pos));
      return 1;
    }
  yyprintf((stderr, "  fail yymatchChar(%c) @ %s\n", c, G->buf+G->pos));
  return 0;
}

YY_LOCAL(int) yymatchString(GREG *G, char *s)
{
  int yysav= G->pos;
  while (*s)
    {
      if (G->pos >= G->limit && !yyrefill(G)) return 0;
      if (G->buf[G->pos] != *s)
        {
          G->pos= yysav;
          return 0;
        }
      ++s;
      ++G->pos;
    }
  return 1;
}

YY_LOCAL(int) yymatchClass(GREG *G, unsigned char *bits)
{
  int c;
  if (G->pos >= G->limit && !yyrefill(G)) return 0;
  c= (unsigned char)G->buf[G->pos];
  if (bits[c >> 3] & (1 << (c & 7)))
    {
      ++G->pos;
      yyprintf((stderr, "  ok   yymatchClass @ %s\n", G->buf+G->pos));
      return 1;
    }
  yyprintf((stderr, "  fail yymatchClass @ %s\n", G->buf+G->pos));
  return 0;
}

YY_LOCAL(void) yyDo(GREG *G, yyaction action, int begin, int end)
{
  while (G->thunkpos >= G->thunkslen)
    {
      G->thunkslen *= 2;
      G->thunks= (yythunk *)YY_REALLOC(G->thunks, sizeof(yythunk) * G->thunkslen, G->data);
    }
  G->thunks[G->thunkpos].begin=  begin;
  G->thunks[G->thunkpos].end=    end;
  G->thunks[G->thunkpos].action= action;
  ++G->thunkpos;
}

YY_LOCAL(int) yyText(GREG *G, int begin, int end)
{
  int yyleng= end - begin;
  if (yyleng <= 0)
    yyleng= 0;
  else
    {
      while (G->textlen < (yyleng + 1))
        {
          G->textlen *= 2;
          G->text= (char *)YY_REALLOC(G->text, G->textlen, G->data);
        }
      memcpy(G->text, G->buf + begin, yyleng);
    }
  G->text[yyleng]= '\0';
  return yyleng;
}

YY_LOCAL(void) yyDone(GREG *G)
{
  int pos;
  for (pos= 0; pos < G->thunkpos; ++pos)
    {
      yythunk *thunk= &G->thunks[pos];
      int yyleng= thunk->end ? yyText(G, thunk->begin, thunk->end) : thunk->begin;
      yyprintf((stderr, "DO [%d] %p %s\n", pos, thunk->action, G->text));
      thunk->action(G, G->text, yyleng, thunk, G->data);
    }
  G->thunkpos= 0;
}

YY_LOCAL(void) yyCommit(GREG *G)
{
  if ((G->limit -= G->pos))
    {
      memmove(G->buf, G->buf + G->pos, G->limit);
    }
  G->offset += G->pos;
  G->begin -= G->pos;
  G->end -= G->pos;
  G->pos= G->thunkpos= 0;
}

YY_LOCAL(int) yyAccept(GREG *G, int tp0)
{
  if (tp0)
    {
      fprintf(stderr, "accept denied at %d\n", tp0);
      return 0;
    }
  else
    {
      yyDone(G);
      yyCommit(G);
    }
  return 1;
}

YY_LOCAL(void) yyPush(GREG *G, char *text, int count, yythunk *thunk, YY_XTYPE YY_XVAR) { G->val += count; }
YY_LOCAL(void) yyPop(GREG *G, char *text, int count, yythunk *thunk, YY_XTYPE YY_XVAR)  { G->val -= count; }
YY_LOCAL(void) yySet(GREG *G, char *text, int count, yythunk *thunk, YY_XTYPE YY_XVAR)  { G->val[count]= G->ss; }

#endif /* YY_PART */

#define YYACCEPT        yyAccept(G, yythunkpos0)

YY_RULE(int) yy_RawNoteBlock(GREG *G); /* 268 */
YY_RULE(int) yy_RawNoteReference(GREG *G); /* 267 */
YY_RULE(int) yy_AlphanumericAscii(GREG *G); /* 266 */
YY_RULE(int) yy_Quoted(GREG *G); /* 265 */
YY_RULE(int) yy_HtmlTag(GREG *G); /* 264 */
YY_RULE(int) yy_MarkTagClose(GREG *G); /* 263 */
YY_RULE(int) yy_MarkTagText(GREG *G); /* 262 */
YY_RULE(int) yy_MarkTagOpen(GREG *G); /* 261 */
YY_RULE(int) yy_Ticks5(GREG *G); /* 260 */
YY_RULE(int) yy_Ticks4(GREG *G); /* 259 */
YY_RULE(int) yy_Ticks3(GREG *G); /* 258 */
YY_RULE(int) yy_Ticks2(GREG *G); /* 257 */
YY_RULE(int) yy_Ticks1(GREG *G); /* 256 */
YY_RULE(int) yy_SkipBlock(GREG *G); /* 255 */
YY_RULE(int) yy_References(GREG *G); /* 254 */
YY_RULE(int) yy_EmptyTitle(GREG *G); /* 253 */
YY_RULE(int) yy_RefTitleParens(GREG *G); /* 252 */
YY_RULE(int) yy_RefTitleDouble(GREG *G); /* 251 */
YY_RULE(int) yy_RefTitleSingle(GREG *G); /* 250 */
YY_RULE(int) yy_LabelInline(GREG *G); /* 249 */
YY_RULE(int) yy_RefTitle(GREG *G); /* 248 */
YY_RULE(int) yy_RefSrc(GREG *G); /* 247 */
YY_RULE(int) yy_AutoLinkEmail(GREG *G); /* 246 */
YY_RULE(int) yy_AutoLinkUrl(GREG *G); /* 245 */
YY_RULE(int) yy_ImageSizeHeight(GREG *G); /* 244 */
YY_RULE(int) yy_ImageSizeWidth(GREG *G); /* 243 */
YY_RULE(int) yy_ImageSizeComplete(GREG *G); /* 242 */
YY_RULE(int) yy_TitleDoubleExt(GREG *G); /* 241 */
YY_RULE(int) yy_TitleSingleExt(GREG *G); /* 240 */
YY_RULE(int) yy_TitleDouble(GREG *G); /* 239 */
YY_RULE(int) yy_TitleSingle(GREG *G); /* 238 */
YY_RULE(int) yy_SourceContents(GREG *G); /* 237 */
YY_RULE(int) yy_ImageSize(GREG *G); /* 236 */
YY_RULE(int) yy_TitleExt(GREG *G); /* 235 */
YY_RULE(int) yy_Title(GREG *G); /* 234 */
YY_RULE(int) yy_Source(GREG *G); /* 233 */
YY_RULE(int) yy_Label(GREG *G); /* 232 */
YY_RULE(int) yy_ReferenceLinkSingle(GREG *G); /* 231 */
YY_RULE(int) yy_ReferenceLinkDouble(GREG *G); /* 230 */
YY_RULE(int) yy_AutoLink(GREG *G); /* 229 */
YY_RULE(int) yy_ReferenceLink(GREG *G); /* 228 */
YY_RULE(int) yy_ExplicitLinkSize(GREG *G); /* 227 */
YY_RULE(int) yy_ExplicitLink(GREG *G); /* 226 */
YY_RULE(int) yy_StrongUl(GREG *G); /* 225 */
YY_RULE(int) yy_StrongStar(GREG *G); /* 224 */
YY_RULE(int) yy_Whitespace(GREG *G); /* 223 */
YY_RULE(int) yy_EmphUl(GREG *G); /* 222 */
YY_RULE(int) yy_EmphStar(GREG *G); /* 221 */
YY_RULE(int) yy_StarLine(GREG *G); /* 220 */
YY_RULE(int) yy_UlLine(GREG *G); /* 219 */
YY_RULE(int) yy_SpecialChar(GREG *G); /* 218 */
YY_RULE(int) yy_Eof(GREG *G); /* 217 */
YY_RULE(int) yy_NormalEndline(GREG *G); /* 216 */
YY_RULE(int) yy_TerminalEndline(GREG *G); /* 215 */
YY_RULE(int) yy_LineBreak(GREG *G); /* 214 */
YY_RULE(int) yy_CharEntity(GREG *G); /* 213 */
YY_RULE(int) yy_DecEntity(GREG *G); /* 212 */
YY_RULE(int) yy_HexEntity(GREG *G); /* 211 */
YY_RULE(int) yy_ExtendedSpecialChar(GREG *G); /* 210 */
YY_RULE(int) yy_Alphanumeric(GREG *G); /* 209 */
YY_RULE(int) yy_NormalChar(GREG *G); /* 208 */
YY_RULE(int) yy_Symbol(GREG *G); /* 207 */
YY_RULE(int) yy_EscapedChar(GREG *G); /* 206 */
YY_RULE(int) yy_Entity(GREG *G); /* 205 */
YY_RULE(int) yy_RawHtml(GREG *G); /* 204 */
YY_RULE(int) yy_Mark(GREG *G); /* 203 */
YY_RULE(int) yy_Code(GREG *G); /* 202 */
YY_RULE(int) yy_InlineNote(GREG *G); /* 201 */
YY_RULE(int) yy_NoteReference(GREG *G); /* 200 */
YY_RULE(int) yy_Link(GREG *G); /* 199 */
YY_RULE(int) yy_Image(GREG *G); /* 198 */
YY_RULE(int) yy_Strike(GREG *G); /* 197 */
YY_RULE(int) yy_Emph(GREG *G); /* 196 */
YY_RULE(int) yy_Strong(GREG *G); /* 195 */
YY_RULE(int) yy_UlOrStarLine(GREG *G); /* 194 */
YY_RULE(int) yy_Str(GREG *G); /* 193 */
YY_RULE(int) yy_InStyleTags(GREG *G); /* 192 */
YY_RULE(int) yy_StyleClose(GREG *G); /* 191 */
YY_RULE(int) yy_StyleOpen(GREG *G); /* 190 */
YY_RULE(int) yy_HtmlBlockType(GREG *G); /* 189 */
YY_RULE(int) yy_HtmlBlockSelfClosing(GREG *G); /* 188 */
YY_RULE(int) yy_HtmlComment(GREG *G); /* 187 */
YY_RULE(int) yy_HtmlBlockInTags(GREG *G); /* 186 */
YY_RULE(int) yy_HtmlBlockHead(GREG *G); /* 185 */
YY_RULE(int) yy_HtmlBlockCloseHead(GREG *G); /* 184 */
YY_RULE(int) yy_HtmlBlockOpenHead(GREG *G); /* 183 */
YY_RULE(int) yy_HtmlBlockScript(GREG *G); /* 182 */
YY_RULE(int) yy_HtmlBlockCloseScript(GREG *G); /* 181 */
YY_RULE(int) yy_HtmlBlockOpenScript(GREG *G); /* 180 */
YY_RULE(int) yy_HtmlBlockTr(GREG *G); /* 179 */
YY_RULE(int) yy_HtmlBlockCloseTr(GREG *G); /* 178 */
YY_RULE(int) yy_HtmlBlockOpenTr(GREG *G); /* 177 */
YY_RULE(int) yy_HtmlBlockThead(GREG *G); /* 176 */
YY_RULE(int) yy_HtmlBlockCloseThead(GREG *G); /* 175 */
YY_RULE(int) yy_HtmlBlockOpenThead(GREG *G); /* 174 */
YY_RULE(int) yy_HtmlBlockTh(GREG *G); /* 173 */
YY_RULE(int) yy_HtmlBlockCloseTh(GREG *G); /* 172 */
YY_RULE(int) yy_HtmlBlockOpenTh(GREG *G); /* 171 */
YY_RULE(int) yy_HtmlBlockTfoot(GREG *G); /* 170 */
YY_RULE(int) yy_HtmlBlockCloseTfoot(GREG *G); /* 169 */
YY_RULE(int) yy_HtmlBlockOpenTfoot(GREG *G); /* 168 */
YY_RULE(int) yy_HtmlBlockTd(GREG *G); /* 167 */
YY_RULE(int) yy_HtmlBlockCloseTd(GREG *G); /* 166 */
YY_RULE(int) yy_HtmlBlockOpenTd(GREG *G); /* 165 */
YY_RULE(int) yy_HtmlBlockTbody(GREG *G); /* 164 */
YY_RULE(int) yy_HtmlBlockCloseTbody(GREG *G); /* 163 */
YY_RULE(int) yy_HtmlBlockOpenTbody(GREG *G); /* 162 */
YY_RULE(int) yy_HtmlBlockLi(GREG *G); /* 161 */
YY_RULE(int) yy_HtmlBlockCloseLi(GREG *G); /* 160 */
YY_RULE(int) yy_HtmlBlockOpenLi(GREG *G); /* 159 */
YY_RULE(int) yy_HtmlBlockFrameset(GREG *G); /* 158 */
YY_RULE(int) yy_HtmlBlockCloseFrameset(GREG *G); /* 157 */
YY_RULE(int) yy_HtmlBlockOpenFrameset(GREG *G); /* 156 */
YY_RULE(int) yy_HtmlBlockDt(GREG *G); /* 155 */
YY_RULE(int) yy_HtmlBlockCloseDt(GREG *G); /* 154 */
YY_RULE(int) yy_HtmlBlockOpenDt(GREG *G); /* 153 */
YY_RULE(int) yy_HtmlBlockDd(GREG *G); /* 152 */
YY_RULE(int) yy_HtmlBlockCloseDd(GREG *G); /* 151 */
YY_RULE(int) yy_HtmlBlockOpenDd(GREG *G); /* 150 */
YY_RULE(int) yy_HtmlBlockUl(GREG *G); /* 149 */
YY_RULE(int) yy_HtmlBlockCloseUl(GREG *G); /* 148 */
YY_RULE(int) yy_HtmlBlockOpenUl(GREG *G); /* 147 */
YY_RULE(int) yy_HtmlBlockTable(GREG *G); /* 146 */
YY_RULE(int) yy_HtmlBlockCloseTable(GREG *G); /* 145 */
YY_RULE(int) yy_HtmlBlockOpenTable(GREG *G); /* 144 */
YY_RULE(int) yy_HtmlBlockPre(GREG *G); /* 143 */
YY_RULE(int) yy_HtmlBlockClosePre(GREG *G); /* 142 */
YY_RULE(int) yy_HtmlBlockOpenPre(GREG *G); /* 141 */
YY_RULE(int) yy_HtmlBlockP(GREG *G); /* 140 */
YY_RULE(int) yy_HtmlBlockCloseP(GREG *G); /* 139 */
YY_RULE(int) yy_HtmlBlockOpenP(GREG *G); /* 138 */
YY_RULE(int) yy_HtmlBlockOl(GREG *G); /* 137 */
YY_RULE(int) yy_HtmlBlockCloseOl(GREG *G); /* 136 */
YY_RULE(int) yy_HtmlBlockOpenOl(GREG *G); /* 135 */
YY_RULE(int) yy_HtmlBlockNoscript(GREG *G); /* 134 */
YY_RULE(int) yy_HtmlBlockCloseNoscript(GREG *G); /* 133 */
YY_RULE(int) yy_HtmlBlockOpenNoscript(GREG *G); /* 132 */
YY_RULE(int) yy_HtmlBlockNoframes(GREG *G); /* 131 */
YY_RULE(int) yy_HtmlBlockCloseNoframes(GREG *G); /* 130 */
YY_RULE(int) yy_HtmlBlockOpenNoframes(GREG *G); /* 129 */
YY_RULE(int) yy_HtmlBlockMenu(GREG *G); /* 128 */
YY_RULE(int) yy_HtmlBlockCloseMenu(GREG *G); /* 127 */
YY_RULE(int) yy_HtmlBlockOpenMenu(GREG *G); /* 126 */
YY_RULE(int) yy_HtmlBlockH6(GREG *G); /* 125 */
YY_RULE(int) yy_HtmlBlockCloseH6(GREG *G); /* 124 */
YY_RULE(int) yy_HtmlBlockOpenH6(GREG *G); /* 123 */
YY_RULE(int) yy_HtmlBlockH5(GREG *G); /* 122 */
YY_RULE(int) yy_HtmlBlockCloseH5(GREG *G); /* 121 */
YY_RULE(int) yy_HtmlBlockOpenH5(GREG *G); /* 120 */
YY_RULE(int) yy_HtmlBlockH4(GREG *G); /* 119 */
YY_RULE(int) yy_HtmlBlockCloseH4(GREG *G); /* 118 */
YY_RULE(int) yy_HtmlBlockOpenH4(GREG *G); /* 117 */
YY_RULE(int) yy_HtmlBlockH3(GREG *G); /* 116 */
YY_RULE(int) yy_HtmlBlockCloseH3(GREG *G); /* 115 */
YY_RULE(int) yy_HtmlBlockOpenH3(GREG *G); /* 114 */
YY_RULE(int) yy_HtmlBlockH2(GREG *G); /* 113 */
YY_RULE(int) yy_HtmlBlockCloseH2(GREG *G); /* 112 */
YY_RULE(int) yy_HtmlBlockOpenH2(GREG *G); /* 111 */
YY_RULE(int) yy_HtmlBlockH1(GREG *G); /* 110 */
YY_RULE(int) yy_HtmlBlockCloseH1(GREG *G); /* 109 */
YY_RULE(int) yy_HtmlBlockOpenH1(GREG *G); /* 108 */
YY_RULE(int) yy_HtmlBlockForm(GREG *G); /* 107 */
YY_RULE(int) yy_HtmlBlockCloseForm(GREG *G); /* 106 */
YY_RULE(int) yy_HtmlBlockOpenForm(GREG *G); /* 105 */
YY_RULE(int) yy_HtmlBlockFieldset(GREG *G); /* 104 */
YY_RULE(int) yy_HtmlBlockCloseFieldset(GREG *G); /* 103 */
YY_RULE(int) yy_HtmlBlockOpenFieldset(GREG *G); /* 102 */
YY_RULE(int) yy_HtmlBlockDl(GREG *G); /* 101 */
YY_RULE(int) yy_HtmlBlockCloseDl(GREG *G); /* 100 */
YY_RULE(int) yy_HtmlBlockOpenDl(GREG *G); /* 99 */
YY_RULE(int) yy_HtmlBlockDiv(GREG *G); /* 98 */
YY_RULE(int) yy_HtmlBlockCloseDiv(GREG *G); /* 97 */
YY_RULE(int) yy_HtmlBlockOpenDiv(GREG *G); /* 96 */
YY_RULE(int) yy_HtmlBlockDir(GREG *G); /* 95 */
YY_RULE(int) yy_HtmlBlockCloseDir(GREG *G); /* 94 */
YY_RULE(int) yy_HtmlBlockOpenDir(GREG *G); /* 93 */
YY_RULE(int) yy_HtmlBlockCenter(GREG *G); /* 92 */
YY_RULE(int) yy_HtmlBlockCloseCenter(GREG *G); /* 91 */
YY_RULE(int) yy_HtmlBlockOpenCenter(GREG *G); /* 90 */
YY_RULE(int) yy_HtmlBlockBlockquote(GREG *G); /* 89 */
YY_RULE(int) yy_HtmlBlockCloseBlockquote(GREG *G); /* 88 */
YY_RULE(int) yy_HtmlBlockOpenBlockquote(GREG *G); /* 87 */
YY_RULE(int) yy_HtmlBlockAddress(GREG *G); /* 86 */
YY_RULE(int) yy_HtmlBlockCloseAddress(GREG *G); /* 85 */
YY_RULE(int) yy_HtmlAttribute(GREG *G); /* 84 */
YY_RULE(int) yy_HtmlBlockOpenAddress(GREG *G); /* 83 */
YY_RULE(int) yy_OptionallyIndentedLine(GREG *G); /* 82 */
YY_RULE(int) yy_Indent(GREG *G); /* 81 */
YY_RULE(int) yy_ListBlockLine(GREG *G); /* 80 */
YY_RULE(int) yy_ListContinuationBlock(GREG *G); /* 79 */
YY_RULE(int) yy_ListBlock(GREG *G); /* 78 */
YY_RULE(int) yy_ListItem(GREG *G); /* 77 */
YY_RULE(int) yy_Enumerator(GREG *G); /* 76 */
YY_RULE(int) yy_ListItemTight(GREG *G); /* 75 */
YY_RULE(int) yy_ListLoose(GREG *G); /* 74 */
YY_RULE(int) yy_ListTight(GREG *G); /* 73 */
YY_RULE(int) yy_Bullet(GREG *G); /* 72 */
YY_RULE(int) yy_TableCell(GREG *G); /* 71 */
YY_RULE(int) yy_TableBorder(GREG *G); /* 70 */
YY_RULE(int) yy_TableLine(GREG *G); /* 69 */
YY_RULE(int) yy_TableDelimiter(GREG *G); /* 68 */
YY_RULE(int) yy_TableHeader(GREG *G); /* 67 */
YY_RULE(int) yy_InlineEquationMultiple(GREG *G); /* 66 */
YY_RULE(int) yy_InlineEquationSingle(GREG *G); /* 65 */
YY_RULE(int) yy_InlineEquation(GREG *G); /* 64 */
YY_RULE(int) yy_Nonspacechar(GREG *G); /* 63 */
YY_RULE(int) yy_DisplayFormulaRawMark(GREG *G); /* 62 */
YY_RULE(int) yy_DisplayFormulaRawEnd(GREG *G); /* 61 */
YY_RULE(int) yy_Spnl(GREG *G); /* 60 */
YY_RULE(int) yy_DisplayFormulaRawStart(GREG *G); /* 59 */
YY_RULE(int) yy_FormulaNumber(GREG *G); /* 58 */
YY_RULE(int) yy_DisplayFormulaRaw(GREG *G); /* 57 */
YY_RULE(int) yy_DisplayFormulaDollar(GREG *G); /* 56 */
YY_RULE(int) yy_FencedCodeBlockTidle(GREG *G); /* 55 */
YY_RULE(int) yy_FencedCodeBlockTick(GREG *G); /* 54 */
YY_RULE(int) yy_FencedCodeBlockEndTidle(GREG *G); /* 53 */
YY_RULE(int) yy_FencedCodeBlockChunkTidle(GREG *G); /* 52 */
YY_RULE(int) yy_FencedCodeBlockStartTidleLine(GREG *G); /* 51 */
YY_RULE(int) yy_FencedCodeBlockStartTidle(GREG *G); /* 50 */
YY_RULE(int) yy_Spacechar(GREG *G); /* 49 */
YY_RULE(int) yy_FencedCodeBlockEndTick(GREG *G); /* 48 */
YY_RULE(int) yy_FencedCodeBlockChunkTick(GREG *G); /* 47 */
YY_RULE(int) yy_FencedCodeBlockStartTickLine(GREG *G); /* 46 */
YY_RULE(int) yy_FencedCodeBlockStartTick(GREG *G); /* 45 */
YY_RULE(int) yy_VerbatimChunk(GREG *G); /* 44 */
YY_RULE(int) yy_IndentedLine(GREG *G); /* 43 */
YY_RULE(int) yy_NonblankIndentedLine(GREG *G); /* 42 */
YY_RULE(int) yy_Line(GREG *G); /* 41 */
YY_RULE(int) yy_StartList(GREG *G); /* 40 */
YY_RULE(int) yy_BlockQuoteRaw(GREG *G); /* 39 */
YY_RULE(int) yy_Endline(GREG *G); /* 38 */
YY_RULE(int) yy_RawLine(GREG *G); /* 37 */
YY_RULE(int) yy_SetextBottom2(GREG *G); /* 36 */
YY_RULE(int) yy_SetextBottom1(GREG *G); /* 35 */
YY_RULE(int) yy_SetextHeading2(GREG *G); /* 34 */
YY_RULE(int) yy_SetextHeading1(GREG *G); /* 33 */
YY_RULE(int) yy_SetextHeading(GREG *G); /* 32 */
YY_RULE(int) yy_Space(GREG *G); /* 31 */
YY_RULE(int) yy_AtxHeading(GREG *G); /* 30 */
YY_RULE(int) yy_AtxStart(GREG *G); /* 29 */
YY_RULE(int) yy_Inline(GREG *G); /* 28 */
YY_RULE(int) yy_Sp(GREG *G); /* 27 */
YY_RULE(int) yy_AtxInline(GREG *G); /* 26 */
YY_RULE(int) yy_Inlines(GREG *G); /* 25 */
YY_RULE(int) yy_NonindentSpace(GREG *G); /* 24 */
YY_RULE(int) yy_Plain(GREG *G); /* 23 */
YY_RULE(int) yy_Para(GREG *G); /* 22 */
YY_RULE(int) yy_StyleBlock(GREG *G); /* 21 */
YY_RULE(int) yy_HtmlBlock(GREG *G); /* 20 */
YY_RULE(int) yy_Table(GREG *G); /* 19 */
YY_RULE(int) yy_BulletList(GREG *G); /* 18 */
YY_RULE(int) yy_OrderedList(GREG *G); /* 17 */
YY_RULE(int) yy_Heading(GREG *G); /* 16 */
YY_RULE(int) yy_HorizontalRule(GREG *G); /* 15 */
YY_RULE(int) yy_Reference(GREG *G); /* 14 */
YY_RULE(int) yy_Note(GREG *G); /* 13 */
YY_RULE(int) yy_DisplayFormula(GREG *G); /* 12 */
YY_RULE(int) yy_FencedCodeBlock(GREG *G); /* 11 */
YY_RULE(int) yy_Verbatim(GREG *G); /* 10 */
YY_RULE(int) yy_BlockQuote(GREG *G); /* 9 */
YY_RULE(int) yy_BlankLine(GREG *G); /* 8 */
YY_RULE(int) yy_FrontMatterEndMark(GREG *G); /* 7 */
YY_RULE(int) yy_FrontMatterBlock(GREG *G); /* 6 */
YY_RULE(int) yy_Newline(GREG *G); /* 5 */
YY_RULE(int) yy_LocMarker(GREG *G); /* 4 */
YY_RULE(int) yy_Block(GREG *G); /* 3 */
YY_RULE(int) yy_FrontMatter(GREG *G); /* 2 */
YY_RULE(int) yy_Doc(GREG *G); /* 1 */

YY_ACTION(void) yy_1_InlineNote(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_InlineNote\n"));
   ADD(elem_s(pmh_NOTE)); ;
#undef s
}
YY_ACTION(void) yy_1_Note(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_Note\n"));
   ADD(elem_s(pmh_NOTE)); ;
#undef s
}
YY_ACTION(void) yy_1_NoteReference(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_NoteReference\n"));
   ADD(elem(pmh_NOTE)); ;
}
YY_ACTION(void) yy_1_RawLine(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_RawLine\n"));
   yy = elem(pmh_RAW); ;
}
YY_ACTION(void) yy_1_Line(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Line\n"));
   yy = mk_element((parser_data *)G->data, pmh_RAW, yy->pos, yy->end); ;
}
YY_ACTION(void) yy_1_StartList(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_StartList\n"));
   yy = NULL; ;
}
YY_ACTION(void) yy_1_HtmlComment(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_HtmlComment\n"));
   ADD(elem_s(pmh_COMMENT)); ;
#undef s
}
YY_ACTION(void) yy_1_RawHtml(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_RawHtml\n"));
   ADD(elem_s(pmh_HTML)); ;
#undef s
}
YY_ACTION(void) yy_1_MarkTagClose(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_MarkTagClose\n"));
   yy = elem_s(pmh_HTML); ;
#undef s
}
YY_ACTION(void) yy_1_MarkTagOpen(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_MarkTagOpen\n"));
   yy = elem_s(pmh_HTML); ;
#undef s
}
YY_ACTION(void) yy_1_Mark(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define c G->val[-1]
#define o G->val[-2]
  yyprintf((stderr, "do yy_1_Mark\n"));
  
         ADD(o);
         ADD(mk_element((parser_data *)G->data, pmh_MARK, o->end, c->pos));
         ADD(c);
       ;
#undef c
#undef o
}
YY_ACTION(void) yy_1_Code(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_Code\n"));
   ADD(elem_s(pmh_CODE)); ;
#undef s
}
YY_ACTION(void) yy_1_Ticks5(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Ticks5\n"));
   yy = elem(pmh_NO_TYPE); ;
}
YY_ACTION(void) yy_1_Ticks4(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Ticks4\n"));
   yy = elem(pmh_NO_TYPE); ;
}
YY_ACTION(void) yy_1_Ticks3(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Ticks3\n"));
   yy = elem(pmh_NO_TYPE); ;
}
YY_ACTION(void) yy_1_Ticks2(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Ticks2\n"));
   yy = elem(pmh_NO_TYPE); ;
}
YY_ACTION(void) yy_1_Ticks1(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Ticks1\n"));
   yy = elem(pmh_NO_TYPE); ;
}
YY_ACTION(void) yy_1_RefSrc(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_RefSrc\n"));
   yy = mk_notype; yy->address = COPY_YYTEXT_ORIG(); ;
}
YY_ACTION(void) yy_2_Label(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_2_Label\n"));
  
            s->pos = s->pos;
            s->end = thunk->end;
            yy = s;
        ;
#undef s
}
YY_ACTION(void) yy_1_Label(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_Label\n"));
   s->label = COPY_YYTEXT_ORIG(); ;
#undef s
}
YY_ACTION(void) yy_1_Reference(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define r G->val[-1]
#define l G->val[-2]
#define s G->val[-3]
  yyprintf((stderr, "do yy_1_Reference\n"));
  
                pmh_realelement *el = elem_s(pmh_REFERENCE);
                el->label = strdup_or_null(l->label);
                el->address = strdup_or_null(r->address);
                ADD(el);
                FREE_LABEL(l);
                FREE_ADDRESS(r);
              ;
#undef r
#undef l
#undef s
}
YY_ACTION(void) yy_3_AutoLinkEmail(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_3_AutoLinkEmail\n"));
  
                s->end = thunk->end;
                ADD(s);
               ;
#undef s
}
YY_ACTION(void) yy_2_AutoLinkEmail(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_2_AutoLinkEmail\n"));
   s->address = COPY_YYTEXT_ORIG(); ;
#undef s
}
YY_ACTION(void) yy_1_AutoLinkEmail(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_AutoLinkEmail\n"));
   s->type = pmh_AUTO_LINK_EMAIL; ;
#undef s
}
YY_ACTION(void) yy_3_AutoLinkUrl(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_3_AutoLinkUrl\n"));
  
                s->end = thunk->end;
                ADD(s);
               ;
#undef s
}
YY_ACTION(void) yy_2_AutoLinkUrl(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_2_AutoLinkUrl\n"));
   s->address = COPY_YYTEXT_ORIG(); ;
#undef s
}
YY_ACTION(void) yy_1_AutoLinkUrl(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_AutoLinkUrl\n"));
   s->type = pmh_AUTO_LINK_URL; ;
#undef s
}
YY_ACTION(void) yy_3_Source(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_3_Source\n"));
   yy->address = COPY_YYTEXT_ORIG(); ;
}
YY_ACTION(void) yy_2_Source(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_2_Source\n"));
   yy->address = COPY_YYTEXT_ORIG(); ;
}
YY_ACTION(void) yy_1_Source(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Source\n"));
   yy = mk_notype; ;
}
YY_ACTION(void) yy_1_ExplicitLinkSize(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define l G->val[-1]
#define s G->val[-2]
  yyprintf((stderr, "do yy_1_ExplicitLinkSize\n"));
  
                        yy = elem_s(pmh_LINK);
                        if (l->address != NULL)
                            yy->address = strdup_or_null(l->address);
                        FREE_LABEL(s);
                        FREE_ADDRESS(l);
                    ;
#undef l
#undef s
}
YY_ACTION(void) yy_1_ExplicitLink(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define l G->val[-1]
#define s G->val[-2]
  yyprintf((stderr, "do yy_1_ExplicitLink\n"));
  
                    yy = elem_s(pmh_LINK);
                    if (l->address != NULL)
                        yy->address = strdup_or_null(l->address);
                    FREE_LABEL(s);
                    FREE_ADDRESS(l);
                ;
#undef l
#undef s
}
YY_ACTION(void) yy_1_ReferenceLinkSingle(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_ReferenceLinkSingle\n"));
  
                        	pmh_realelement *reference = GET_REF(s->label);
                            if (reference) {
                                yy = elem_s(pmh_LINK);
                                yy->label = strdup_or_null(s->label);
                                yy->address = strdup_or_null(reference->address);
                            } else
                                yy = NULL;
                            FREE_LABEL(s);
                        ;
#undef s
}
YY_ACTION(void) yy_1_ReferenceLinkDouble(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define l G->val[-1]
#define s G->val[-2]
  yyprintf((stderr, "do yy_1_ReferenceLinkDouble\n"));
  
                        	pmh_realelement *reference = GET_REF(l->label);
                            if (reference) {
                                yy = elem_s(pmh_LINK);
                                yy->label = strdup_or_null(l->label);
                                yy->address = strdup_or_null(reference->address);
                            } else
                                yy = NULL;
                            FREE_LABEL(s);
                            FREE_LABEL(l);
                        ;
#undef l
#undef s
}
YY_ACTION(void) yy_1_Link(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Link\n"));
   if (yy) ADD(yy); ;
}
YY_ACTION(void) yy_1_Image(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Image\n"));
  
            if (yy != NULL) {
                yy->type = pmh_IMAGE;
                yy->pos -= 1;
                ADD(yy);
            }
        ;
}
YY_ACTION(void) yy_1_Strike(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_Strike\n"));
   ADD(elem_s(pmh_STRIKE)); ;
#undef s
}
YY_ACTION(void) yy_1_StrongUl(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_StrongUl\n"));
   ADD(elem_s(pmh_STRONG)); ;
#undef s
}
YY_ACTION(void) yy_1_StrongStar(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_StrongStar\n"));
   ADD(elem_s(pmh_STRONG)); ;
#undef s
}
YY_ACTION(void) yy_1_EmphUl(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_EmphUl\n"));
   ADD(elem_s(pmh_EMPH)); ;
#undef s
}
YY_ACTION(void) yy_1_EmphStar(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_EmphStar\n"));
   ADD(elem_s(pmh_EMPH)); ;
#undef s
}
YY_ACTION(void) yy_1_Entity(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_Entity\n"));
   ADD(elem_s(pmh_HTML_ENTITY)); ;
#undef s
}
YY_ACTION(void) yy_1_StyleBlock(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_StyleBlock\n"));
   ADD(elem_s(pmh_HTMLBLOCK)); ;
#undef s
}
YY_ACTION(void) yy_1_HtmlBlock(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_HtmlBlock\n"));
   ADD(elem_s(pmh_HTMLBLOCK)); ;
#undef s
}
YY_ACTION(void) yy_1_HtmlBlockH6(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_HtmlBlockH6\n"));
   ADD(elem_s(pmh_H6)); ;
#undef s
}
YY_ACTION(void) yy_1_HtmlBlockH5(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_HtmlBlockH5\n"));
   ADD(elem_s(pmh_H5)); ;
#undef s
}
YY_ACTION(void) yy_1_HtmlBlockH4(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_HtmlBlockH4\n"));
   ADD(elem_s(pmh_H4)); ;
#undef s
}
YY_ACTION(void) yy_1_HtmlBlockH3(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_HtmlBlockH3\n"));
   ADD(elem_s(pmh_H3)); ;
#undef s
}
YY_ACTION(void) yy_1_HtmlBlockH2(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_HtmlBlockH2\n"));
   ADD(elem_s(pmh_H2)); ;
#undef s
}
YY_ACTION(void) yy_1_HtmlBlockH1(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_HtmlBlockH1\n"));
   ADD(elem_s(pmh_H1)); ;
#undef s
}
YY_ACTION(void) yy_1_Enumerator(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Enumerator\n"));
   ADD(elem(pmh_LIST_ENUMERATOR)); ;
}
YY_ACTION(void) yy_3_ListContinuationBlock(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_3_ListContinuationBlock\n"));
   yy = a; ;
#undef a
}
YY_ACTION(void) yy_2_ListContinuationBlock(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_2_ListContinuationBlock\n"));
   a = cons(yy, a); ;
#undef a
}
YY_ACTION(void) yy_1_ListContinuationBlock(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_1_ListContinuationBlock\n"));
   if (*yytext == '\0') /* if strlen(yytext) == 0 */
                                a = cons(elem(pmh_SEPARATOR), a);
                            else
                                a = cons(elem(pmh_RAW), a);
                          ;
#undef a
}
YY_ACTION(void) yy_3_ListBlock(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_3_ListBlock\n"));
   yy = a; ;
#undef a
}
YY_ACTION(void) yy_2_ListBlock(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_2_ListBlock\n"));
   a = cons(elem(pmh_RAW), a); ;
#undef a
}
YY_ACTION(void) yy_1_ListBlock(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_1_ListBlock\n"));
   a = cons(yy, a); ;
#undef a
}
YY_ACTION(void) yy_3_ListItemTight(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_3_ListItemTight\n"));
   yy = a; ;
#undef a
}
YY_ACTION(void) yy_2_ListItemTight(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_2_ListItemTight\n"));
   a = cons(yy, a); ;
#undef a
}
YY_ACTION(void) yy_1_ListItemTight(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_1_ListItemTight\n"));
   a = cons(yy, a); ;
#undef a
}
YY_ACTION(void) yy_3_ListItem(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_3_ListItem\n"));
   yy = a; ;
#undef a
}
YY_ACTION(void) yy_2_ListItem(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_2_ListItem\n"));
   a = cons(yy, a); ;
#undef a
}
YY_ACTION(void) yy_1_ListItem(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_1_ListItem\n"));
   a = cons(yy, a); ;
#undef a
}
YY_ACTION(void) yy_2_ListLoose(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define b G->val[-1]
#define a G->val[-2]
  yyprintf((stderr, "do yy_2_ListLoose\n"));
   pmh_realelement *cur = a;
              while (cur != NULL) {
                  pmh_realelement *rawlist = mk_element((parser_data *)G->data, pmh_RAW_LIST, 0,0);
                  rawlist->children = reverse(cur->children);
                  ADD(rawlist);
                  cur = cur->next;
              }
            ;
#undef b
#undef a
}
YY_ACTION(void) yy_1_ListLoose(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define b G->val[-1]
#define a G->val[-2]
  yyprintf((stderr, "do yy_1_ListLoose\n"));
   b = cons(etext("\n\n"), b); /* In loose list, \n\n added to end of each element */
                pmh_realelement *el = mk_notype;
                el->children = b;
                a = cons(el, a);
              ;
#undef b
#undef a
}
YY_ACTION(void) yy_2_ListTight(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_2_ListTight\n"));
   pmh_realelement *cur = a;
              while (cur != NULL) {
                  pmh_realelement *rawlist = mk_element((parser_data *)G->data, pmh_RAW_LIST, 0,0);
                  rawlist->children = reverse(cur->children);
                  ADD(rawlist);
                  cur = cur->next;
              }
            ;
#undef a
}
YY_ACTION(void) yy_1_ListTight(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_1_ListTight\n"));
   pmh_realelement *el = mk_notype;
                el->children = yy;
                a = cons(el, a);
              ;
#undef a
}
YY_ACTION(void) yy_1_Bullet(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_Bullet\n"));
   ADD(elem(pmh_LIST_BULLET)); ;
}
YY_ACTION(void) yy_1_HorizontalRule(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_HorizontalRule\n"));
   ADD(elem(pmh_HRULE)); ;
}
YY_ACTION(void) yy_1_TableBorder(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_TableBorder\n"));
   ADD(elem(pmh_TABLEBORDER)); ;
}
YY_ACTION(void) yy_1_TableHeader(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_TableHeader\n"));
   ADD(elem_s(pmh_TABLEHEADER)); ;
#undef s
}
YY_ACTION(void) yy_1_Table(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_Table\n"));
   ADD(elem_s(pmh_TABLE)); ;
#undef s
}
YY_ACTION(void) yy_1_InlineEquation(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_InlineEquation\n"));
   ADD(elem(pmh_INLINEEQUATION)); ;
}
YY_ACTION(void) yy_1_DisplayFormulaRaw(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_DisplayFormulaRaw\n"));
   ADD(elem(pmh_DISPLAYFORMULA)); ;
}
YY_ACTION(void) yy_1_DisplayFormulaDollar(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_DisplayFormulaDollar\n"));
   ADD(elem(pmh_DISPLAYFORMULA)); ;
}
YY_ACTION(void) yy_1_FencedCodeBlockTidle(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_FencedCodeBlockTidle\n"));
   ADD(elem(pmh_FENCEDCODEBLOCK)); ;
}
YY_ACTION(void) yy_1_FencedCodeBlockTick(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_FencedCodeBlockTick\n"));
   ADD(elem(pmh_FENCEDCODEBLOCK)); ;
}
YY_ACTION(void) yy_1_Verbatim(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_Verbatim\n"));
   ADD(elem_s(pmh_VERBATIM)); ;
#undef s
}
YY_ACTION(void) yy_5_BlockQuoteRaw(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_5_BlockQuoteRaw\n"));
   yy = a; ;
#undef a
}
YY_ACTION(void) yy_4_BlockQuoteRaw(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_4_BlockQuoteRaw\n"));
   a = cons(etext("\n"), a); ;
#undef a
}
YY_ACTION(void) yy_3_BlockQuoteRaw(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_3_BlockQuoteRaw\n"));
   a = cons(yy, a); ;
#undef a
}
YY_ACTION(void) yy_2_BlockQuoteRaw(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_2_BlockQuoteRaw\n"));
   a = cons(yy, a); ;
#undef a
}
YY_ACTION(void) yy_1_BlockQuoteRaw(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_1_BlockQuoteRaw\n"));
   ADD(elem(pmh_BLOCKQUOTE)); ;
#undef a
}
YY_ACTION(void) yy_1_BlockQuote(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define a G->val[-1]
  yyprintf((stderr, "do yy_1_BlockQuote\n"));
   pmh_realelement *rawlist = mk_element((parser_data *)G->data, pmh_RAW_LIST, 0,0);
              rawlist->children = reverse(a);
              ADD(rawlist);
            ;
#undef a
}
YY_ACTION(void) yy_1_SetextHeading2(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_SetextHeading2\n"));
   ADD(elem_s(pmh_H2)); ;
#undef s
}
YY_ACTION(void) yy_1_SetextHeading1(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_SetextHeading1\n"));
   ADD(elem_s(pmh_H1)); ;
#undef s
}
YY_ACTION(void) yy_1_AtxHeading(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_AtxHeading\n"));
   ADD(elem_s(s->type)); ;
#undef s
}
YY_ACTION(void) yy_1_AtxStart(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_AtxStart\n"));
   yy = elem((pmh_element_type)(pmh_H1 + (strlen(yytext) - 1))); ;
}
YY_ACTION(void) yy_1_FrontMatter(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
#define s G->val[-1]
  yyprintf((stderr, "do yy_1_FrontMatter\n"));
  
                    ADD(elem(pmh_FRONTMATTER));
                ;
#undef s
}
YY_ACTION(void) yy_1_LocMarker(GREG *G, char *yytext, int yyleng, yythunk *thunk, YY_XTYPE YY_XVAR)
{
  yyprintf((stderr, "do yy_1_LocMarker\n"));
   yy = elem(pmh_NO_TYPE); ;
}

YY_RULE(int) yy_RawNoteBlock(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "RawNoteBlock"));
  {  int yypos4= G->pos, yythunkpos4= G->thunkpos;  if (!yy_BlankLine(G)) { goto l4; }  goto l1;
  l4:;	  G->pos= yypos4; G->thunkpos= yythunkpos4;
  }  if (!yy_OptionallyIndentedLine(G)) { goto l1; }
  l2:;	
  {  int yypos3= G->pos, yythunkpos3= G->thunkpos;
  {  int yypos5= G->pos, yythunkpos5= G->thunkpos;  if (!yy_BlankLine(G)) { goto l5; }  goto l3;
  l5:;	  G->pos= yypos5; G->thunkpos= yythunkpos5;
  }  if (!yy_OptionallyIndentedLine(G)) { goto l3; }  goto l2;
  l3:;	  G->pos= yypos3; G->thunkpos= yythunkpos3;
  }
  l6:;	
  {  int yypos7= G->pos, yythunkpos7= G->thunkpos;  if (!yy_BlankLine(G)) { goto l7; }  goto l6;
  l7:;	  G->pos= yypos7; G->thunkpos= yythunkpos7;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "RawNoteBlock", G->buf+G->pos));
  return 1;
  l1:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "RawNoteBlock", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_RawNoteReference(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "RawNoteReference"));  if (!yymatchString(G, "[^")) goto l8;
  {  int yypos11= G->pos, yythunkpos11= G->thunkpos;  if (!yy_Newline(G)) { goto l11; }  goto l8;
  l11:;	  G->pos= yypos11; G->thunkpos= yythunkpos11;
  }
  {  int yypos12= G->pos, yythunkpos12= G->thunkpos;  if (!yymatchChar(G, ']')) goto l12;  goto l8;
  l12:;	  G->pos= yypos12; G->thunkpos= yythunkpos12;
  }  if (!yymatchDot(G)) goto l8;
  l9:;	
  {  int yypos10= G->pos, yythunkpos10= G->thunkpos;
  {  int yypos13= G->pos, yythunkpos13= G->thunkpos;  if (!yy_Newline(G)) { goto l13; }  goto l10;
  l13:;	  G->pos= yypos13; G->thunkpos= yythunkpos13;
  }
  {  int yypos14= G->pos, yythunkpos14= G->thunkpos;  if (!yymatchChar(G, ']')) goto l14;  goto l10;
  l14:;	  G->pos= yypos14; G->thunkpos= yythunkpos14;
  }  if (!yymatchDot(G)) goto l10;  goto l9;
  l10:;	  G->pos= yypos10; G->thunkpos= yythunkpos10;
  }  if (!yymatchChar(G, ']')) goto l8;
  yyprintf((stderr, "  ok   %s @ %s\n", "RawNoteReference", G->buf+G->pos));
  return 1;
  l8:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "RawNoteReference", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_AlphanumericAscii(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "AlphanumericAscii"));  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\376\377\377\007\376\377\377\007\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l15;
  yyprintf((stderr, "  ok   %s @ %s\n", "AlphanumericAscii", G->buf+G->pos));
  return 1;
  l15:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "AlphanumericAscii", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Quoted(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Quoted"));
  {  int yypos17= G->pos, yythunkpos17= G->thunkpos;  if (!yymatchChar(G, '"')) goto l18;
  l19:;	
  {  int yypos20= G->pos, yythunkpos20= G->thunkpos;
  {  int yypos21= G->pos, yythunkpos21= G->thunkpos;  if (!yymatchChar(G, '"')) goto l21;  goto l20;
  l21:;	  G->pos= yypos21; G->thunkpos= yythunkpos21;
  }  if (!yymatchDot(G)) goto l20;  goto l19;
  l20:;	  G->pos= yypos20; G->thunkpos= yythunkpos20;
  }  if (!yymatchChar(G, '"')) goto l18;  goto l17;
  l18:;	  G->pos= yypos17; G->thunkpos= yythunkpos17;  if (!yymatchChar(G, '\'')) goto l16;
  l22:;	
  {  int yypos23= G->pos, yythunkpos23= G->thunkpos;
  {  int yypos24= G->pos, yythunkpos24= G->thunkpos;  if (!yymatchChar(G, '\'')) goto l24;  goto l23;
  l24:;	  G->pos= yypos24; G->thunkpos= yythunkpos24;
  }  if (!yymatchDot(G)) goto l23;  goto l22;
  l23:;	  G->pos= yypos23; G->thunkpos= yythunkpos23;
  }  if (!yymatchChar(G, '\'')) goto l16;
  }
  l17:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Quoted", G->buf+G->pos));
  return 1;
  l16:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Quoted", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlTag(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlTag"));  if (!yymatchChar(G, '<')) goto l25;  if (!yy_Spnl(G)) { goto l25; }
  {  int yypos26= G->pos, yythunkpos26= G->thunkpos;  if (!yymatchChar(G, '/')) goto l26;  goto l27;
  l26:;	  G->pos= yypos26; G->thunkpos= yythunkpos26;
  }
  l27:;	  if (!yy_AlphanumericAscii(G)) { goto l25; }
  l28:;	
  {  int yypos29= G->pos, yythunkpos29= G->thunkpos;  if (!yy_AlphanumericAscii(G)) { goto l29; }  goto l28;
  l29:;	  G->pos= yypos29; G->thunkpos= yythunkpos29;
  }  if (!yy_Spnl(G)) { goto l25; }
  l30:;	
  {  int yypos31= G->pos, yythunkpos31= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l31; }  goto l30;
  l31:;	  G->pos= yypos31; G->thunkpos= yythunkpos31;
  }
  {  int yypos32= G->pos, yythunkpos32= G->thunkpos;  if (!yymatchChar(G, '/')) goto l32;  goto l33;
  l32:;	  G->pos= yypos32; G->thunkpos= yythunkpos32;
  }
  l33:;	  if (!yy_Spnl(G)) { goto l25; }  if (!yymatchChar(G, '>')) goto l25;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlTag", G->buf+G->pos));
  return 1;
  l25:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlTag", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_MarkTagClose(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "MarkTagClose"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l34;  if (!yy_LocMarker(G)) { goto l34; }  yyDo(G, yySet, -1, 0);  if (!yymatchChar(G, '<')) goto l34;  if (!yy_Spnl(G)) { goto l34; }  if (!yymatchChar(G, '/')) goto l34;
  {  int yypos35= G->pos, yythunkpos35= G->thunkpos;  if (!yymatchString(G, "mark")) goto l36;  goto l35;
  l36:;	  G->pos= yypos35; G->thunkpos= yythunkpos35;  if (!yymatchString(G, "MARK")) goto l34;
  }
  l35:;	  if (!yy_Spnl(G)) { goto l34; }  if (!yymatchChar(G, '>')) goto l34;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l34;  yyDo(G, yy_1_MarkTagClose, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "MarkTagClose", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l34:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "MarkTagClose", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_MarkTagText(GREG *G)
{
  yyprintf((stderr, "%s\n", "MarkTagText"));
  l38:;	
  {  int yypos39= G->pos, yythunkpos39= G->thunkpos;
  {  int yypos40= G->pos, yythunkpos40= G->thunkpos;  if (!yy_MarkTagClose(G)) { goto l40; }  goto l39;
  l40:;	  G->pos= yypos40; G->thunkpos= yythunkpos40;
  }
  {  int yypos41= G->pos, yythunkpos41= G->thunkpos;  if (!yy_RawHtml(G)) { goto l42; }  goto l41;
  l42:;	  G->pos= yypos41; G->thunkpos= yythunkpos41;  if (!yymatchDot(G)) goto l39;
  }
  l41:;	  goto l38;
  l39:;	  G->pos= yypos39; G->thunkpos= yythunkpos39;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "MarkTagText", G->buf+G->pos));
  return 1;
}
YY_RULE(int) yy_MarkTagOpen(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "MarkTagOpen"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l43;  if (!yy_LocMarker(G)) { goto l43; }  yyDo(G, yySet, -1, 0);  if (!yymatchChar(G, '<')) goto l43;  if (!yy_Spnl(G)) { goto l43; }
  {  int yypos44= G->pos, yythunkpos44= G->thunkpos;  if (!yymatchString(G, "mark")) goto l45;  goto l44;
  l45:;	  G->pos= yypos44; G->thunkpos= yythunkpos44;  if (!yymatchString(G, "MARK")) goto l43;
  }
  l44:;	  if (!yy_Spnl(G)) { goto l43; }
  l46:;	
  {  int yypos47= G->pos, yythunkpos47= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l47; }  goto l46;
  l47:;	  G->pos= yypos47; G->thunkpos= yythunkpos47;
  }  if (!yymatchChar(G, '>')) goto l43;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l43;  yyDo(G, yy_1_MarkTagOpen, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "MarkTagOpen", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l43:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "MarkTagOpen", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Ticks5(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Ticks5"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l48;  if (!yymatchString(G, "`````")) goto l48;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l48;
  {  int yypos49= G->pos, yythunkpos49= G->thunkpos;  if (!yymatchChar(G, '`')) goto l49;  goto l48;
  l49:;	  G->pos= yypos49; G->thunkpos= yythunkpos49;
  }  yyDo(G, yy_1_Ticks5, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Ticks5", G->buf+G->pos));
  return 1;
  l48:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Ticks5", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Ticks4(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Ticks4"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l50;  if (!yymatchString(G, "````")) goto l50;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l50;
  {  int yypos51= G->pos, yythunkpos51= G->thunkpos;  if (!yymatchChar(G, '`')) goto l51;  goto l50;
  l51:;	  G->pos= yypos51; G->thunkpos= yythunkpos51;
  }  yyDo(G, yy_1_Ticks4, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Ticks4", G->buf+G->pos));
  return 1;
  l50:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Ticks4", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Ticks3(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Ticks3"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l52;  if (!yymatchString(G, "```")) goto l52;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l52;
  {  int yypos53= G->pos, yythunkpos53= G->thunkpos;  if (!yymatchChar(G, '`')) goto l53;  goto l52;
  l53:;	  G->pos= yypos53; G->thunkpos= yythunkpos53;
  }  yyDo(G, yy_1_Ticks3, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Ticks3", G->buf+G->pos));
  return 1;
  l52:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Ticks3", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Ticks2(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Ticks2"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l54;  if (!yymatchString(G, "``")) goto l54;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l54;
  {  int yypos55= G->pos, yythunkpos55= G->thunkpos;  if (!yymatchChar(G, '`')) goto l55;  goto l54;
  l55:;	  G->pos= yypos55; G->thunkpos= yythunkpos55;
  }  yyDo(G, yy_1_Ticks2, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Ticks2", G->buf+G->pos));
  return 1;
  l54:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Ticks2", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Ticks1(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Ticks1"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l56;  if (!yymatchChar(G, '`')) goto l56;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l56;
  {  int yypos57= G->pos, yythunkpos57= G->thunkpos;  if (!yymatchChar(G, '`')) goto l57;  goto l56;
  l57:;	  G->pos= yypos57; G->thunkpos= yythunkpos57;
  }  yyDo(G, yy_1_Ticks1, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Ticks1", G->buf+G->pos));
  return 1;
  l56:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Ticks1", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_SkipBlock(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "SkipBlock"));
  {  int yypos59= G->pos, yythunkpos59= G->thunkpos;
  {  int yypos63= G->pos, yythunkpos63= G->thunkpos;  if (!yy_BlankLine(G)) { goto l63; }  goto l60;
  l63:;	  G->pos= yypos63; G->thunkpos= yythunkpos63;
  }  if (!yy_RawLine(G)) { goto l60; }
  l61:;	
  {  int yypos62= G->pos, yythunkpos62= G->thunkpos;
  {  int yypos64= G->pos, yythunkpos64= G->thunkpos;  if (!yy_BlankLine(G)) { goto l64; }  goto l62;
  l64:;	  G->pos= yypos64; G->thunkpos= yythunkpos64;
  }  if (!yy_RawLine(G)) { goto l62; }  goto l61;
  l62:;	  G->pos= yypos62; G->thunkpos= yythunkpos62;
  }
  l65:;	
  {  int yypos66= G->pos, yythunkpos66= G->thunkpos;  if (!yy_BlankLine(G)) { goto l66; }  goto l65;
  l66:;	  G->pos= yypos66; G->thunkpos= yythunkpos66;
  }  goto l59;
  l60:;	  G->pos= yypos59; G->thunkpos= yythunkpos59;  if (!yy_BlankLine(G)) { goto l58; }
  l67:;	
  {  int yypos68= G->pos, yythunkpos68= G->thunkpos;  if (!yy_BlankLine(G)) { goto l68; }  goto l67;
  l68:;	  G->pos= yypos68; G->thunkpos= yythunkpos68;
  }
  }
  l59:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "SkipBlock", G->buf+G->pos));
  return 1;
  l58:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "SkipBlock", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_References(GREG *G)
{
  yyprintf((stderr, "%s\n", "References"));
  l70:;	
  {  int yypos71= G->pos, yythunkpos71= G->thunkpos;
  {  int yypos72= G->pos, yythunkpos72= G->thunkpos;  if (!yy_Reference(G)) { goto l73; }  goto l72;
  l73:;	  G->pos= yypos72; G->thunkpos= yythunkpos72;  if (!yy_SkipBlock(G)) { goto l71; }
  }
  l72:;	  goto l70;
  l71:;	  G->pos= yypos71; G->thunkpos= yythunkpos71;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "References", G->buf+G->pos));
  return 1;
}
YY_RULE(int) yy_EmptyTitle(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "EmptyTitle"));  if (!yymatchString(G, "")) goto l74;
  yyprintf((stderr, "  ok   %s @ %s\n", "EmptyTitle", G->buf+G->pos));
  return 1;
  l74:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "EmptyTitle", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_RefTitleParens(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "RefTitleParens"));  if (!yy_Spnl(G)) { goto l75; }  if (!yymatchChar(G, '(')) goto l75;
  l76:;	
  {  int yypos77= G->pos, yythunkpos77= G->thunkpos;
  {  int yypos78= G->pos, yythunkpos78= G->thunkpos;
  {  int yypos79= G->pos, yythunkpos79= G->thunkpos;  if (!yymatchChar(G, ')')) goto l80;  if (!yy_Sp(G)) { goto l80; }  if (!yy_Newline(G)) { goto l80; }  goto l79;
  l80:;	  G->pos= yypos79; G->thunkpos= yythunkpos79;  if (!yy_Newline(G)) { goto l78; }
  }
  l79:;	  goto l77;
  l78:;	  G->pos= yypos78; G->thunkpos= yythunkpos78;
  }  if (!yymatchDot(G)) goto l77;  goto l76;
  l77:;	  G->pos= yypos77; G->thunkpos= yythunkpos77;
  }  if (!yymatchChar(G, ')')) goto l75;
  yyprintf((stderr, "  ok   %s @ %s\n", "RefTitleParens", G->buf+G->pos));
  return 1;
  l75:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "RefTitleParens", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_RefTitleDouble(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "RefTitleDouble"));  if (!yy_Spnl(G)) { goto l81; }  if (!yymatchChar(G, '"')) goto l81;
  l82:;	
  {  int yypos83= G->pos, yythunkpos83= G->thunkpos;
  {  int yypos84= G->pos, yythunkpos84= G->thunkpos;
  {  int yypos85= G->pos, yythunkpos85= G->thunkpos;  if (!yymatchChar(G, '"')) goto l86;  if (!yy_Sp(G)) { goto l86; }  if (!yy_Newline(G)) { goto l86; }  goto l85;
  l86:;	  G->pos= yypos85; G->thunkpos= yythunkpos85;  if (!yy_Newline(G)) { goto l84; }
  }
  l85:;	  goto l83;
  l84:;	  G->pos= yypos84; G->thunkpos= yythunkpos84;
  }  if (!yymatchDot(G)) goto l83;  goto l82;
  l83:;	  G->pos= yypos83; G->thunkpos= yythunkpos83;
  }  if (!yymatchChar(G, '"')) goto l81;
  yyprintf((stderr, "  ok   %s @ %s\n", "RefTitleDouble", G->buf+G->pos));
  return 1;
  l81:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "RefTitleDouble", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_RefTitleSingle(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "RefTitleSingle"));  if (!yy_Spnl(G)) { goto l87; }  if (!yymatchChar(G, '\'')) goto l87;
  l88:;	
  {  int yypos89= G->pos, yythunkpos89= G->thunkpos;
  {  int yypos90= G->pos, yythunkpos90= G->thunkpos;
  {  int yypos91= G->pos, yythunkpos91= G->thunkpos;  if (!yymatchChar(G, '\'')) goto l92;  if (!yy_Sp(G)) { goto l92; }  if (!yy_Newline(G)) { goto l92; }  goto l91;
  l92:;	  G->pos= yypos91; G->thunkpos= yythunkpos91;  if (!yy_Newline(G)) { goto l90; }
  }
  l91:;	  goto l89;
  l90:;	  G->pos= yypos90; G->thunkpos= yythunkpos90;
  }  if (!yymatchDot(G)) goto l89;  goto l88;
  l89:;	  G->pos= yypos89; G->thunkpos= yythunkpos89;
  }  if (!yymatchChar(G, '\'')) goto l87;
  yyprintf((stderr, "  ok   %s @ %s\n", "RefTitleSingle", G->buf+G->pos));
  return 1;
  l87:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "RefTitleSingle", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_LabelInline(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "LabelInline"));
  {  int yypos94= G->pos, yythunkpos94= G->thunkpos;  if (!yy_Str(G)) { goto l95; }  goto l94;
  l95:;	  G->pos= yypos94; G->thunkpos= yythunkpos94;  if (!yy_Endline(G)) { goto l96; }  goto l94;
  l96:;	  G->pos= yypos94; G->thunkpos= yythunkpos94;  if (!yy_Space(G)) { goto l97; }  goto l94;
  l97:;	  G->pos= yypos94; G->thunkpos= yythunkpos94;  if (!yy_RawHtml(G)) { goto l98; }  goto l94;
  l98:;	  G->pos= yypos94; G->thunkpos= yythunkpos94;  if (!yy_Entity(G)) { goto l99; }  goto l94;
  l99:;	  G->pos= yypos94; G->thunkpos= yythunkpos94;  if (!yy_EscapedChar(G)) { goto l100; }  goto l94;
  l100:;	  G->pos= yypos94; G->thunkpos= yythunkpos94;  if (!yy_Symbol(G)) { goto l93; }
  }
  l94:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "LabelInline", G->buf+G->pos));
  return 1;
  l93:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "LabelInline", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_RefTitle(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "RefTitle"));
  {  int yypos102= G->pos, yythunkpos102= G->thunkpos;  if (!yy_RefTitleSingle(G)) { goto l103; }  goto l102;
  l103:;	  G->pos= yypos102; G->thunkpos= yythunkpos102;  if (!yy_RefTitleDouble(G)) { goto l104; }  goto l102;
  l104:;	  G->pos= yypos102; G->thunkpos= yythunkpos102;  if (!yy_RefTitleParens(G)) { goto l105; }  goto l102;
  l105:;	  G->pos= yypos102; G->thunkpos= yythunkpos102;  if (!yy_EmptyTitle(G)) { goto l101; }
  }
  l102:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "RefTitle", G->buf+G->pos));
  return 1;
  l101:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "RefTitle", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_RefSrc(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "RefSrc"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l106;  if (!yy_Nonspacechar(G)) { goto l106; }
  l107:;	
  {  int yypos108= G->pos, yythunkpos108= G->thunkpos;  if (!yy_Nonspacechar(G)) { goto l108; }  goto l107;
  l108:;	  G->pos= yypos108; G->thunkpos= yythunkpos108;
  }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l106;  yyDo(G, yy_1_RefSrc, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "RefSrc", G->buf+G->pos));
  return 1;
  l106:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "RefSrc", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_AutoLinkEmail(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "AutoLinkEmail"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l109;  if (!yy_LocMarker(G)) { goto l109; }  yyDo(G, yySet, -1, 0);  yyDo(G, yy_1_AutoLinkEmail, G->begin, G->end);  if (!yymatchChar(G, '<')) goto l109;
  {  int yypos110= G->pos, yythunkpos110= G->thunkpos;  if (!yymatchString(G, "mailto:")) goto l110;  goto l111;
  l110:;	  G->pos= yypos110; G->thunkpos= yythunkpos110;
  }
  l111:;	  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l109;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\062\350\377\003\376\377\377\207\376\377\377\107\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l109;
  l112:;	
  {  int yypos113= G->pos, yythunkpos113= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\062\350\377\003\376\377\377\207\376\377\377\107\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l113;  goto l112;
  l113:;	  G->pos= yypos113; G->thunkpos= yythunkpos113;
  }  if (!yymatchChar(G, '@')) goto l109;
  {  int yypos116= G->pos, yythunkpos116= G->thunkpos;  if (!yy_Newline(G)) { goto l116; }  goto l109;
  l116:;	  G->pos= yypos116; G->thunkpos= yythunkpos116;
  }
  {  int yypos117= G->pos, yythunkpos117= G->thunkpos;  if (!yymatchChar(G, '>')) goto l117;  goto l109;
  l117:;	  G->pos= yypos117; G->thunkpos= yythunkpos117;
  }  if (!yymatchDot(G)) goto l109;
  l114:;	
  {  int yypos115= G->pos, yythunkpos115= G->thunkpos;
  {  int yypos118= G->pos, yythunkpos118= G->thunkpos;  if (!yy_Newline(G)) { goto l118; }  goto l115;
  l118:;	  G->pos= yypos118; G->thunkpos= yythunkpos118;
  }
  {  int yypos119= G->pos, yythunkpos119= G->thunkpos;  if (!yymatchChar(G, '>')) goto l119;  goto l115;
  l119:;	  G->pos= yypos119; G->thunkpos= yythunkpos119;
  }  if (!yymatchDot(G)) goto l115;  goto l114;
  l115:;	  G->pos= yypos115; G->thunkpos= yythunkpos115;
  }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l109;  yyDo(G, yy_2_AutoLinkEmail, G->begin, G->end);  if (!yymatchChar(G, '>')) goto l109;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l109;  yyDo(G, yy_3_AutoLinkEmail, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "AutoLinkEmail", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l109:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "AutoLinkEmail", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_AutoLinkUrl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "AutoLinkUrl"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l120;  if (!yy_LocMarker(G)) { goto l120; }  yyDo(G, yySet, -1, 0);  yyDo(G, yy_1_AutoLinkUrl, G->begin, G->end);  if (!yymatchChar(G, '<')) goto l120;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l120;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\000\000\376\377\377\007\376\377\377\007\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l120;
  l121:;	
  {  int yypos122= G->pos, yythunkpos122= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\000\000\376\377\377\007\376\377\377\007\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l122;  goto l121;
  l122:;	  G->pos= yypos122; G->thunkpos= yythunkpos122;
  }  if (!yymatchString(G, "://")) goto l120;
  {  int yypos125= G->pos, yythunkpos125= G->thunkpos;  if (!yy_Newline(G)) { goto l125; }  goto l120;
  l125:;	  G->pos= yypos125; G->thunkpos= yythunkpos125;
  }
  {  int yypos126= G->pos, yythunkpos126= G->thunkpos;  if (!yymatchChar(G, '>')) goto l126;  goto l120;
  l126:;	  G->pos= yypos126; G->thunkpos= yythunkpos126;
  }  if (!yymatchDot(G)) goto l120;
  l123:;	
  {  int yypos124= G->pos, yythunkpos124= G->thunkpos;
  {  int yypos127= G->pos, yythunkpos127= G->thunkpos;  if (!yy_Newline(G)) { goto l127; }  goto l124;
  l127:;	  G->pos= yypos127; G->thunkpos= yythunkpos127;
  }
  {  int yypos128= G->pos, yythunkpos128= G->thunkpos;  if (!yymatchChar(G, '>')) goto l128;  goto l124;
  l128:;	  G->pos= yypos128; G->thunkpos= yythunkpos128;
  }  if (!yymatchDot(G)) goto l124;  goto l123;
  l124:;	  G->pos= yypos124; G->thunkpos= yythunkpos124;
  }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l120;  yyDo(G, yy_2_AutoLinkUrl, G->begin, G->end);  if (!yymatchChar(G, '>')) goto l120;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l120;  yyDo(G, yy_3_AutoLinkUrl, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "AutoLinkUrl", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l120:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "AutoLinkUrl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ImageSizeHeight(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "ImageSizeHeight"));  if (!yymatchString(G, "=x")) goto l129;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l129;
  l130:;	
  {  int yypos131= G->pos, yythunkpos131= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l131;  goto l130;
  l131:;	  G->pos= yypos131; G->thunkpos= yythunkpos131;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "ImageSizeHeight", G->buf+G->pos));
  return 1;
  l129:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ImageSizeHeight", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ImageSizeWidth(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "ImageSizeWidth"));  if (!yymatchChar(G, '=')) goto l132;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l132;
  l133:;	
  {  int yypos134= G->pos, yythunkpos134= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l134;  goto l133;
  l134:;	  G->pos= yypos134; G->thunkpos= yythunkpos134;
  }  if (!yymatchChar(G, 'x')) goto l132;
  yyprintf((stderr, "  ok   %s @ %s\n", "ImageSizeWidth", G->buf+G->pos));
  return 1;
  l132:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ImageSizeWidth", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ImageSizeComplete(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "ImageSizeComplete"));  if (!yymatchChar(G, '=')) goto l135;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l135;
  l136:;	
  {  int yypos137= G->pos, yythunkpos137= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l137;  goto l136;
  l137:;	  G->pos= yypos137; G->thunkpos= yythunkpos137;
  }  if (!yymatchChar(G, 'x')) goto l135;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l135;
  l138:;	
  {  int yypos139= G->pos, yythunkpos139= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l139;  goto l138;
  l139:;	  G->pos= yypos139; G->thunkpos= yythunkpos139;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "ImageSizeComplete", G->buf+G->pos));
  return 1;
  l135:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ImageSizeComplete", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_TitleDoubleExt(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "TitleDoubleExt"));  if (!yymatchChar(G, '"')) goto l140;
  l141:;	
  {  int yypos142= G->pos, yythunkpos142= G->thunkpos;
  {  int yypos143= G->pos, yythunkpos143= G->thunkpos;
  {  int yypos144= G->pos, yythunkpos144= G->thunkpos;  if (!yymatchChar(G, '"')) goto l145;  goto l144;
  l145:;	  G->pos= yypos144; G->thunkpos= yythunkpos144;  if (!yy_Newline(G)) { goto l143; }
  }
  l144:;	  goto l142;
  l143:;	  G->pos= yypos143; G->thunkpos= yythunkpos143;
  }  if (!yymatchDot(G)) goto l142;  goto l141;
  l142:;	  G->pos= yypos142; G->thunkpos= yythunkpos142;
  }  if (!yymatchChar(G, '"')) goto l140;
  yyprintf((stderr, "  ok   %s @ %s\n", "TitleDoubleExt", G->buf+G->pos));
  return 1;
  l140:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "TitleDoubleExt", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_TitleSingleExt(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "TitleSingleExt"));  if (!yymatchChar(G, '\'')) goto l146;
  l147:;	
  {  int yypos148= G->pos, yythunkpos148= G->thunkpos;
  {  int yypos149= G->pos, yythunkpos149= G->thunkpos;
  {  int yypos150= G->pos, yythunkpos150= G->thunkpos;  if (!yymatchChar(G, '\'')) goto l151;  goto l150;
  l151:;	  G->pos= yypos150; G->thunkpos= yythunkpos150;  if (!yy_Newline(G)) { goto l149; }
  }
  l150:;	  goto l148;
  l149:;	  G->pos= yypos149; G->thunkpos= yythunkpos149;
  }  if (!yymatchDot(G)) goto l148;  goto l147;
  l148:;	  G->pos= yypos148; G->thunkpos= yythunkpos148;
  }  if (!yymatchChar(G, '\'')) goto l146;
  yyprintf((stderr, "  ok   %s @ %s\n", "TitleSingleExt", G->buf+G->pos));
  return 1;
  l146:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "TitleSingleExt", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_TitleDouble(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "TitleDouble"));  if (!yymatchChar(G, '"')) goto l152;
  l153:;	
  {  int yypos154= G->pos, yythunkpos154= G->thunkpos;
  {  int yypos155= G->pos, yythunkpos155= G->thunkpos;  if (!yymatchChar(G, '"')) goto l155;  if (!yy_Sp(G)) { goto l155; }
  {  int yypos156= G->pos, yythunkpos156= G->thunkpos;  if (!yymatchChar(G, ')')) goto l157;  goto l156;
  l157:;	  G->pos= yypos156; G->thunkpos= yythunkpos156;  if (!yy_Newline(G)) { goto l155; }
  }
  l156:;	  goto l154;
  l155:;	  G->pos= yypos155; G->thunkpos= yythunkpos155;
  }  if (!yymatchDot(G)) goto l154;  goto l153;
  l154:;	  G->pos= yypos154; G->thunkpos= yythunkpos154;
  }  if (!yymatchChar(G, '"')) goto l152;
  yyprintf((stderr, "  ok   %s @ %s\n", "TitleDouble", G->buf+G->pos));
  return 1;
  l152:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "TitleDouble", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_TitleSingle(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "TitleSingle"));  if (!yymatchChar(G, '\'')) goto l158;
  l159:;	
  {  int yypos160= G->pos, yythunkpos160= G->thunkpos;
  {  int yypos161= G->pos, yythunkpos161= G->thunkpos;  if (!yymatchChar(G, '\'')) goto l161;  if (!yy_Sp(G)) { goto l161; }
  {  int yypos162= G->pos, yythunkpos162= G->thunkpos;  if (!yymatchChar(G, ')')) goto l163;  goto l162;
  l163:;	  G->pos= yypos162; G->thunkpos= yythunkpos162;  if (!yy_Newline(G)) { goto l161; }
  }
  l162:;	  goto l160;
  l161:;	  G->pos= yypos161; G->thunkpos= yythunkpos161;
  }  if (!yymatchDot(G)) goto l160;  goto l159;
  l160:;	  G->pos= yypos160; G->thunkpos= yythunkpos160;
  }  if (!yymatchChar(G, '\'')) goto l158;
  yyprintf((stderr, "  ok   %s @ %s\n", "TitleSingle", G->buf+G->pos));
  return 1;
  l158:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "TitleSingle", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_SourceContents(GREG *G)
{
  yyprintf((stderr, "%s\n", "SourceContents"));
  l165:;	
  {  int yypos166= G->pos, yythunkpos166= G->thunkpos;
  {  int yypos167= G->pos, yythunkpos167= G->thunkpos;
  {  int yypos171= G->pos, yythunkpos171= G->thunkpos;  if (!yymatchChar(G, '(')) goto l171;  goto l168;
  l171:;	  G->pos= yypos171; G->thunkpos= yythunkpos171;
  }
  {  int yypos172= G->pos, yythunkpos172= G->thunkpos;  if (!yymatchChar(G, ')')) goto l172;  goto l168;
  l172:;	  G->pos= yypos172; G->thunkpos= yythunkpos172;
  }
  {  int yypos173= G->pos, yythunkpos173= G->thunkpos;  if (!yymatchChar(G, '>')) goto l173;  goto l168;
  l173:;	  G->pos= yypos173; G->thunkpos= yythunkpos173;
  }  if (!yy_Nonspacechar(G)) { goto l168; }
  l169:;	
  {  int yypos170= G->pos, yythunkpos170= G->thunkpos;
  {  int yypos174= G->pos, yythunkpos174= G->thunkpos;  if (!yymatchChar(G, '(')) goto l174;  goto l170;
  l174:;	  G->pos= yypos174; G->thunkpos= yythunkpos174;
  }
  {  int yypos175= G->pos, yythunkpos175= G->thunkpos;  if (!yymatchChar(G, ')')) goto l175;  goto l170;
  l175:;	  G->pos= yypos175; G->thunkpos= yythunkpos175;
  }
  {  int yypos176= G->pos, yythunkpos176= G->thunkpos;  if (!yymatchChar(G, '>')) goto l176;  goto l170;
  l176:;	  G->pos= yypos176; G->thunkpos= yythunkpos176;
  }  if (!yy_Nonspacechar(G)) { goto l170; }  goto l169;
  l170:;	  G->pos= yypos170; G->thunkpos= yythunkpos170;
  }  goto l167;
  l168:;	  G->pos= yypos167; G->thunkpos= yythunkpos167;  if (!yymatchChar(G, '(')) goto l166;  if (!yy_SourceContents(G)) { goto l166; }  if (!yymatchChar(G, ')')) goto l166;
  }
  l167:;	  goto l165;
  l166:;	  G->pos= yypos166; G->thunkpos= yythunkpos166;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "SourceContents", G->buf+G->pos));
  return 1;
}
YY_RULE(int) yy_ImageSize(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "ImageSize"));
  {  int yypos178= G->pos, yythunkpos178= G->thunkpos;  if (!yy_ImageSizeComplete(G)) { goto l179; }  goto l178;
  l179:;	  G->pos= yypos178; G->thunkpos= yythunkpos178;  if (!yy_ImageSizeWidth(G)) { goto l180; }  goto l178;
  l180:;	  G->pos= yypos178; G->thunkpos= yythunkpos178;  if (!yy_ImageSizeHeight(G)) { goto l181; }  goto l178;
  l181:;	  G->pos= yypos178; G->thunkpos= yythunkpos178;  if (!yymatchString(G, "")) goto l177;
  }
  l178:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "ImageSize", G->buf+G->pos));
  return 1;
  l177:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ImageSize", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_TitleExt(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "TitleExt"));
  {  int yypos183= G->pos, yythunkpos183= G->thunkpos;  if (!yy_TitleSingleExt(G)) { goto l184; }  goto l183;
  l184:;	  G->pos= yypos183; G->thunkpos= yythunkpos183;  if (!yy_TitleDoubleExt(G)) { goto l185; }  goto l183;
  l185:;	  G->pos= yypos183; G->thunkpos= yythunkpos183;  if (!yymatchString(G, "")) goto l182;
  }
  l183:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "TitleExt", G->buf+G->pos));
  return 1;
  l182:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "TitleExt", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Title(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Title"));
  {  int yypos187= G->pos, yythunkpos187= G->thunkpos;  if (!yy_TitleSingle(G)) { goto l188; }  goto l187;
  l188:;	  G->pos= yypos187; G->thunkpos= yythunkpos187;  if (!yy_TitleDouble(G)) { goto l189; }  goto l187;
  l189:;	  G->pos= yypos187; G->thunkpos= yythunkpos187;  if (!yymatchString(G, "")) goto l186;
  }
  l187:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Title", G->buf+G->pos));
  return 1;
  l186:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Title", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Source(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Source"));  yyDo(G, yy_1_Source, G->begin, G->end);
  {  int yypos191= G->pos, yythunkpos191= G->thunkpos;  if (!yymatchChar(G, '<')) goto l192;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l192;  if (!yy_SourceContents(G)) { goto l192; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l192;  yyDo(G, yy_2_Source, G->begin, G->end);  if (!yymatchChar(G, '>')) goto l192;  goto l191;
  l192:;	  G->pos= yypos191; G->thunkpos= yythunkpos191;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l190;  if (!yy_SourceContents(G)) { goto l190; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l190;  yyDo(G, yy_3_Source, G->begin, G->end);
  }
  l191:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Source", G->buf+G->pos));
  return 1;
  l190:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Source", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Label(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "Label"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l193;  if (!yy_LocMarker(G)) { goto l193; }  yyDo(G, yySet, -1, 0);  if (!yymatchChar(G, '[')) goto l193;
  {  int yypos194= G->pos, yythunkpos194= G->thunkpos;
  {  int yypos196= G->pos, yythunkpos196= G->thunkpos;  if (!yymatchChar(G, '^')) goto l196;  goto l195;
  l196:;	  G->pos= yypos196; G->thunkpos= yythunkpos196;
  }  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_NOTES) )) goto l195;  goto l194;
  l195:;	  G->pos= yypos194; G->thunkpos= yythunkpos194;
  {  int yypos197= G->pos, yythunkpos197= G->thunkpos;  if (!yymatchDot(G)) goto l193;  G->pos= yypos197; G->thunkpos= yythunkpos197;
  }  yyText(G, G->begin, G->end);  if (!( !EXT(pmh_EXT_NOTES) )) goto l193;
  }
  l194:;	  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l193;
  l198:;	
  {  int yypos199= G->pos, yythunkpos199= G->thunkpos;
  {  int yypos200= G->pos, yythunkpos200= G->thunkpos;  if (!yymatchChar(G, ']')) goto l200;  goto l199;
  l200:;	  G->pos= yypos200; G->thunkpos= yythunkpos200;
  }
  {  int yypos201= G->pos, yythunkpos201= G->thunkpos;  if (!yymatchChar(G, '[')) goto l201;  goto l199;
  l201:;	  G->pos= yypos201; G->thunkpos= yythunkpos201;
  }  if (!yy_LabelInline(G)) { goto l199; }  goto l198;
  l199:;	  G->pos= yypos199; G->thunkpos= yythunkpos199;
  }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l193;  yyDo(G, yy_1_Label, G->begin, G->end);  if (!yymatchChar(G, ']')) goto l193;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l193;  yyDo(G, yy_2_Label, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Label", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l193:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Label", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ReferenceLinkSingle(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "ReferenceLinkSingle"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l202;  if (!yy_Label(G)) { goto l202; }  yyDo(G, yySet, -1, 0);
  {  int yypos203= G->pos, yythunkpos203= G->thunkpos;  if (!yy_Spnl(G)) { goto l203; }  if (!yymatchString(G, "[]")) goto l203;  goto l204;
  l203:;	  G->pos= yypos203; G->thunkpos= yythunkpos203;
  }
  l204:;	  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l202;  yyDo(G, yy_1_ReferenceLinkSingle, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "ReferenceLinkSingle", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l202:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ReferenceLinkSingle", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ReferenceLinkDouble(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 2, 0);
  yyprintf((stderr, "%s\n", "ReferenceLinkDouble"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l205;  if (!yy_Label(G)) { goto l205; }  yyDo(G, yySet, -2, 0);  if (!yy_Spnl(G)) { goto l205; }
  {  int yypos206= G->pos, yythunkpos206= G->thunkpos;  if (!yymatchString(G, "[]")) goto l206;  goto l205;
  l206:;	  G->pos= yypos206; G->thunkpos= yythunkpos206;
  }  if (!yy_Label(G)) { goto l205; }  yyDo(G, yySet, -1, 0);  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l205;  yyDo(G, yy_1_ReferenceLinkDouble, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "ReferenceLinkDouble", G->buf+G->pos));  yyDo(G, yyPop, 2, 0);
  return 1;
  l205:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ReferenceLinkDouble", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_AutoLink(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "AutoLink"));
  {  int yypos208= G->pos, yythunkpos208= G->thunkpos;  if (!yy_AutoLinkUrl(G)) { goto l209; }  goto l208;
  l209:;	  G->pos= yypos208; G->thunkpos= yythunkpos208;  if (!yy_AutoLinkEmail(G)) { goto l207; }
  }
  l208:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "AutoLink", G->buf+G->pos));
  return 1;
  l207:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "AutoLink", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ReferenceLink(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "ReferenceLink"));
  {  int yypos211= G->pos, yythunkpos211= G->thunkpos;  if (!yy_ReferenceLinkDouble(G)) { goto l212; }  goto l211;
  l212:;	  G->pos= yypos211; G->thunkpos= yythunkpos211;  if (!yy_ReferenceLinkSingle(G)) { goto l210; }
  }
  l211:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "ReferenceLink", G->buf+G->pos));
  return 1;
  l210:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ReferenceLink", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ExplicitLinkSize(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 2, 0);
  yyprintf((stderr, "%s\n", "ExplicitLinkSize"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l213;  if (!yy_Label(G)) { goto l213; }  yyDo(G, yySet, -2, 0);  if (!yy_Spnl(G)) { goto l213; }  if (!yymatchChar(G, '(')) goto l213;  if (!yy_Sp(G)) { goto l213; }  if (!yy_Source(G)) { goto l213; }  yyDo(G, yySet, -1, 0);  if (!yy_Spnl(G)) { goto l213; }  if (!yy_TitleExt(G)) { goto l213; }  if (!yy_Sp(G)) { goto l213; }  if (!yy_ImageSize(G)) { goto l213; }  if (!yy_Sp(G)) { goto l213; }  if (!yymatchChar(G, ')')) goto l213;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l213;  yyDo(G, yy_1_ExplicitLinkSize, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "ExplicitLinkSize", G->buf+G->pos));  yyDo(G, yyPop, 2, 0);
  return 1;
  l213:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ExplicitLinkSize", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ExplicitLink(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 2, 0);
  yyprintf((stderr, "%s\n", "ExplicitLink"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l214;  if (!yy_Label(G)) { goto l214; }  yyDo(G, yySet, -2, 0);  if (!yy_Spnl(G)) { goto l214; }  if (!yymatchChar(G, '(')) goto l214;  if (!yy_Sp(G)) { goto l214; }  if (!yy_Source(G)) { goto l214; }  yyDo(G, yySet, -1, 0);  if (!yy_Spnl(G)) { goto l214; }  if (!yy_Title(G)) { goto l214; }  if (!yy_Sp(G)) { goto l214; }  if (!yymatchChar(G, ')')) goto l214;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l214;  yyDo(G, yy_1_ExplicitLink, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "ExplicitLink", G->buf+G->pos));  yyDo(G, yyPop, 2, 0);
  return 1;
  l214:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ExplicitLink", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_StrongUl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "StrongUl"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l215;  if (!yy_LocMarker(G)) { goto l215; }  yyDo(G, yySet, -1, 0);  if (!yymatchString(G, "__")) goto l215;
  {  int yypos216= G->pos, yythunkpos216= G->thunkpos;  if (!yy_Whitespace(G)) { goto l216; }  goto l215;
  l216:;	  G->pos= yypos216; G->thunkpos= yythunkpos216;
  }
  {  int yypos219= G->pos, yythunkpos219= G->thunkpos;  if (!yymatchString(G, "__")) goto l219;  goto l215;
  l219:;	  G->pos= yypos219; G->thunkpos= yythunkpos219;
  }  if (!yy_Inline(G)) { goto l215; }
  l217:;	
  {  int yypos218= G->pos, yythunkpos218= G->thunkpos;
  {  int yypos220= G->pos, yythunkpos220= G->thunkpos;  if (!yymatchString(G, "__")) goto l220;  goto l218;
  l220:;	  G->pos= yypos220; G->thunkpos= yythunkpos220;
  }  if (!yy_Inline(G)) { goto l218; }  goto l217;
  l218:;	  G->pos= yypos218; G->thunkpos= yythunkpos218;
  }  if (!yymatchString(G, "__")) goto l215;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l215;  yyDo(G, yy_1_StrongUl, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "StrongUl", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l215:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "StrongUl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_StrongStar(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "StrongStar"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l221;  if (!yy_LocMarker(G)) { goto l221; }  yyDo(G, yySet, -1, 0);  if (!yymatchString(G, "**")) goto l221;
  {  int yypos222= G->pos, yythunkpos222= G->thunkpos;  if (!yy_Whitespace(G)) { goto l222; }  goto l221;
  l222:;	  G->pos= yypos222; G->thunkpos= yythunkpos222;
  }
  {  int yypos225= G->pos, yythunkpos225= G->thunkpos;  if (!yymatchString(G, "**")) goto l225;  goto l221;
  l225:;	  G->pos= yypos225; G->thunkpos= yythunkpos225;
  }  if (!yy_Inline(G)) { goto l221; }
  l223:;	
  {  int yypos224= G->pos, yythunkpos224= G->thunkpos;
  {  int yypos226= G->pos, yythunkpos226= G->thunkpos;  if (!yymatchString(G, "**")) goto l226;  goto l224;
  l226:;	  G->pos= yypos226; G->thunkpos= yythunkpos226;
  }  if (!yy_Inline(G)) { goto l224; }  goto l223;
  l224:;	  G->pos= yypos224; G->thunkpos= yythunkpos224;
  }  if (!yymatchString(G, "**")) goto l221;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l221;  yyDo(G, yy_1_StrongStar, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "StrongStar", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l221:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "StrongStar", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Whitespace(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Whitespace"));
  {  int yypos228= G->pos, yythunkpos228= G->thunkpos;  if (!yy_Spacechar(G)) { goto l229; }  goto l228;
  l229:;	  G->pos= yypos228; G->thunkpos= yythunkpos228;  if (!yy_Newline(G)) { goto l227; }
  }
  l228:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Whitespace", G->buf+G->pos));
  return 1;
  l227:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Whitespace", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_EmphUl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "EmphUl"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l230;  if (!yy_LocMarker(G)) { goto l230; }  yyDo(G, yySet, -1, 0);  if (!yymatchChar(G, '_')) goto l230;
  {  int yypos231= G->pos, yythunkpos231= G->thunkpos;  if (!yy_Whitespace(G)) { goto l231; }  goto l230;
  l231:;	  G->pos= yypos231; G->thunkpos= yythunkpos231;
  }
  {  int yypos234= G->pos, yythunkpos234= G->thunkpos;
  {  int yypos236= G->pos, yythunkpos236= G->thunkpos;  if (!yymatchChar(G, '_')) goto l236;  goto l235;
  l236:;	  G->pos= yypos236; G->thunkpos= yythunkpos236;
  }  if (!yy_Inline(G)) { goto l235; }  goto l234;
  l235:;	  G->pos= yypos234; G->thunkpos= yythunkpos234;  if (!yy_StrongUl(G)) { goto l230; }
  }
  l234:;	
  l232:;	
  {  int yypos233= G->pos, yythunkpos233= G->thunkpos;
  {  int yypos237= G->pos, yythunkpos237= G->thunkpos;
  {  int yypos239= G->pos, yythunkpos239= G->thunkpos;  if (!yymatchChar(G, '_')) goto l239;  goto l238;
  l239:;	  G->pos= yypos239; G->thunkpos= yythunkpos239;
  }  if (!yy_Inline(G)) { goto l238; }  goto l237;
  l238:;	  G->pos= yypos237; G->thunkpos= yythunkpos237;  if (!yy_StrongUl(G)) { goto l233; }
  }
  l237:;	  goto l232;
  l233:;	  G->pos= yypos233; G->thunkpos= yythunkpos233;
  }  if (!yymatchChar(G, '_')) goto l230;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l230;  yyDo(G, yy_1_EmphUl, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "EmphUl", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l230:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "EmphUl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_EmphStar(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "EmphStar"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l240;  if (!yy_LocMarker(G)) { goto l240; }  yyDo(G, yySet, -1, 0);  if (!yymatchChar(G, '*')) goto l240;
  {  int yypos241= G->pos, yythunkpos241= G->thunkpos;  if (!yy_Whitespace(G)) { goto l241; }  goto l240;
  l241:;	  G->pos= yypos241; G->thunkpos= yythunkpos241;
  }
  {  int yypos244= G->pos, yythunkpos244= G->thunkpos;
  {  int yypos246= G->pos, yythunkpos246= G->thunkpos;  if (!yymatchChar(G, '*')) goto l246;  goto l245;
  l246:;	  G->pos= yypos246; G->thunkpos= yythunkpos246;
  }  if (!yy_Inline(G)) { goto l245; }  goto l244;
  l245:;	  G->pos= yypos244; G->thunkpos= yythunkpos244;  if (!yy_StrongStar(G)) { goto l240; }
  }
  l244:;	
  l242:;	
  {  int yypos243= G->pos, yythunkpos243= G->thunkpos;
  {  int yypos247= G->pos, yythunkpos247= G->thunkpos;
  {  int yypos249= G->pos, yythunkpos249= G->thunkpos;  if (!yymatchChar(G, '*')) goto l249;  goto l248;
  l249:;	  G->pos= yypos249; G->thunkpos= yythunkpos249;
  }  if (!yy_Inline(G)) { goto l248; }  goto l247;
  l248:;	  G->pos= yypos247; G->thunkpos= yythunkpos247;  if (!yy_StrongStar(G)) { goto l243; }
  }
  l247:;	  goto l242;
  l243:;	  G->pos= yypos243; G->thunkpos= yythunkpos243;
  }  if (!yymatchChar(G, '*')) goto l240;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l240;  yyDo(G, yy_1_EmphStar, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "EmphStar", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l240:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "EmphStar", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_StarLine(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "StarLine"));
  {  int yypos251= G->pos, yythunkpos251= G->thunkpos;  if (!yymatchString(G, "****")) goto l252;
  l253:;	
  {  int yypos254= G->pos, yythunkpos254= G->thunkpos;  if (!yymatchChar(G, '*')) goto l254;  goto l253;
  l254:;	  G->pos= yypos254; G->thunkpos= yythunkpos254;
  }  goto l251;
  l252:;	  G->pos= yypos251; G->thunkpos= yythunkpos251;  if (!yy_Spacechar(G)) { goto l250; }  if (!yymatchChar(G, '*')) goto l250;
  l255:;	
  {  int yypos256= G->pos, yythunkpos256= G->thunkpos;  if (!yymatchChar(G, '*')) goto l256;  goto l255;
  l256:;	  G->pos= yypos256; G->thunkpos= yythunkpos256;
  }
  {  int yypos257= G->pos, yythunkpos257= G->thunkpos;  if (!yy_Spacechar(G)) { goto l250; }  G->pos= yypos257; G->thunkpos= yythunkpos257;
  }
  }
  l251:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "StarLine", G->buf+G->pos));
  return 1;
  l250:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "StarLine", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_UlLine(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "UlLine"));
  {  int yypos259= G->pos, yythunkpos259= G->thunkpos;  if (!yymatchString(G, "____")) goto l260;
  l261:;	
  {  int yypos262= G->pos, yythunkpos262= G->thunkpos;  if (!yymatchChar(G, '_')) goto l262;  goto l261;
  l262:;	  G->pos= yypos262; G->thunkpos= yythunkpos262;
  }  goto l259;
  l260:;	  G->pos= yypos259; G->thunkpos= yythunkpos259;  if (!yy_Spacechar(G)) { goto l258; }  if (!yymatchChar(G, '_')) goto l258;
  l263:;	
  {  int yypos264= G->pos, yythunkpos264= G->thunkpos;  if (!yymatchChar(G, '_')) goto l264;  goto l263;
  l264:;	  G->pos= yypos264; G->thunkpos= yythunkpos264;
  }
  {  int yypos265= G->pos, yythunkpos265= G->thunkpos;  if (!yy_Spacechar(G)) { goto l258; }  G->pos= yypos265; G->thunkpos= yythunkpos265;
  }
  }
  l259:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "UlLine", G->buf+G->pos));
  return 1;
  l258:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "UlLine", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_SpecialChar(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "SpecialChar"));
  {  int yypos267= G->pos, yythunkpos267= G->thunkpos;  if (!yymatchChar(G, '~')) goto l268;  goto l267;
  l268:;	  G->pos= yypos267; G->thunkpos= yythunkpos267;  if (!yymatchChar(G, '*')) goto l269;  goto l267;
  l269:;	  G->pos= yypos267; G->thunkpos= yythunkpos267;  if (!yymatchChar(G, '_')) goto l270;  goto l267;
  l270:;	  G->pos= yypos267; G->thunkpos= yythunkpos267;  if (!yymatchChar(G, '`')) goto l271;  goto l267;
  l271:;	  G->pos= yypos267; G->thunkpos= yythunkpos267;  if (!yymatchChar(G, '&')) goto l272;  goto l267;
  l272:;	  G->pos= yypos267; G->thunkpos= yythunkpos267;  if (!yymatchChar(G, '[')) goto l273;  goto l267;
  l273:;	  G->pos= yypos267; G->thunkpos= yythunkpos267;  if (!yymatchChar(G, ']')) goto l274;  goto l267;
  l274:;	  G->pos= yypos267; G->thunkpos= yythunkpos267;  if (!yymatchChar(G, '(')) goto l275;  goto l267;
  l275:;	  G->pos= yypos267; G->thunkpos= yythunkpos267;  if (!yymatchChar(G, ')')) goto l276;  goto l267;
  l276:;	  G->pos= yypos267; G->thunkpos= yythunkpos267;  if (!yymatchChar(G, '<')) goto l277;  goto l267;
  l277:;	  G->pos= yypos267; G->thunkpos= yythunkpos267;  if (!yymatchChar(G, '!')) goto l278;  goto l267;
  l278:;	  G->pos= yypos267; G->thunkpos= yythunkpos267;  if (!yymatchChar(G, '#')) goto l279;  goto l267;
  l279:;	  G->pos= yypos267; G->thunkpos= yythunkpos267;  if (!yymatchChar(G, '\\')) goto l280;  goto l267;
  l280:;	  G->pos= yypos267; G->thunkpos= yythunkpos267;  if (!yymatchChar(G, '\'')) goto l281;  goto l267;
  l281:;	  G->pos= yypos267; G->thunkpos= yythunkpos267;  if (!yymatchChar(G, '"')) goto l282;  goto l267;
  l282:;	  G->pos= yypos267; G->thunkpos= yythunkpos267;  if (!yy_ExtendedSpecialChar(G)) { goto l266; }
  }
  l267:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "SpecialChar", G->buf+G->pos));
  return 1;
  l266:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "SpecialChar", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Eof(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Eof"));
  {  int yypos284= G->pos, yythunkpos284= G->thunkpos;  if (!yymatchDot(G)) goto l284;  goto l283;
  l284:;	  G->pos= yypos284; G->thunkpos= yythunkpos284;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "Eof", G->buf+G->pos));
  return 1;
  l283:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Eof", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_NormalEndline(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "NormalEndline"));  if (!yy_Sp(G)) { goto l285; }  if (!yy_Newline(G)) { goto l285; }
  {  int yypos286= G->pos, yythunkpos286= G->thunkpos;  if (!yy_BlankLine(G)) { goto l286; }  goto l285;
  l286:;	  G->pos= yypos286; G->thunkpos= yythunkpos286;
  }
  {  int yypos287= G->pos, yythunkpos287= G->thunkpos;  if (!yymatchChar(G, '>')) goto l287;  goto l285;
  l287:;	  G->pos= yypos287; G->thunkpos= yythunkpos287;
  }
  {  int yypos288= G->pos, yythunkpos288= G->thunkpos;  if (!yy_AtxStart(G)) { goto l288; }  goto l285;
  l288:;	  G->pos= yypos288; G->thunkpos= yythunkpos288;
  }
  {  int yypos289= G->pos, yythunkpos289= G->thunkpos;  if (!yy_Line(G)) { goto l289; }
  {  int yypos290= G->pos, yythunkpos290= G->thunkpos;  if (!yymatchChar(G, '=')) goto l291;
  l292:;	
  {  int yypos293= G->pos, yythunkpos293= G->thunkpos;  if (!yymatchChar(G, '=')) goto l293;  goto l292;
  l293:;	  G->pos= yypos293; G->thunkpos= yythunkpos293;
  }  goto l290;
  l291:;	  G->pos= yypos290; G->thunkpos= yythunkpos290;  if (!yymatchChar(G, '-')) goto l289;
  l294:;	
  {  int yypos295= G->pos, yythunkpos295= G->thunkpos;  if (!yymatchChar(G, '-')) goto l295;  goto l294;
  l295:;	  G->pos= yypos295; G->thunkpos= yythunkpos295;
  }
  }
  l290:;	  if (!yy_Newline(G)) { goto l289; }  goto l285;
  l289:;	  G->pos= yypos289; G->thunkpos= yythunkpos289;
  }
  {  int yypos296= G->pos, yythunkpos296= G->thunkpos;  if (!yy_FencedCodeBlockStartTick(G)) { goto l296; }  goto l285;
  l296:;	  G->pos= yypos296; G->thunkpos= yythunkpos296;
  }
  {  int yypos297= G->pos, yythunkpos297= G->thunkpos;  if (!yy_FencedCodeBlockStartTidle(G)) { goto l297; }  goto l285;
  l297:;	  G->pos= yypos297; G->thunkpos= yythunkpos297;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "NormalEndline", G->buf+G->pos));
  return 1;
  l285:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "NormalEndline", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_TerminalEndline(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "TerminalEndline"));  if (!yy_Sp(G)) { goto l298; }  if (!yy_Newline(G)) { goto l298; }  if (!yy_Eof(G)) { goto l298; }
  yyprintf((stderr, "  ok   %s @ %s\n", "TerminalEndline", G->buf+G->pos));
  return 1;
  l298:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "TerminalEndline", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_LineBreak(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "LineBreak"));  if (!yymatchString(G, "  ")) goto l299;  if (!yy_NormalEndline(G)) { goto l299; }
  yyprintf((stderr, "  ok   %s @ %s\n", "LineBreak", G->buf+G->pos));
  return 1;
  l299:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "LineBreak", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_CharEntity(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "CharEntity"));  if (!yymatchChar(G, '&')) goto l300;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\376\377\377\007\376\377\377\007\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l300;
  l301:;	
  {  int yypos302= G->pos, yythunkpos302= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\376\377\377\007\376\377\377\007\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l302;  goto l301;
  l302:;	  G->pos= yypos302; G->thunkpos= yythunkpos302;
  }  if (!yymatchChar(G, ';')) goto l300;
  yyprintf((stderr, "  ok   %s @ %s\n", "CharEntity", G->buf+G->pos));
  return 1;
  l300:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "CharEntity", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_DecEntity(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "DecEntity"));  if (!yymatchChar(G, '&')) goto l303;  if (!yymatchChar(G, '#')) goto l303;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l303;
  l304:;	
  {  int yypos305= G->pos, yythunkpos305= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l305;  goto l304;
  l305:;	  G->pos= yypos305; G->thunkpos= yythunkpos305;
  }  if (!yymatchChar(G, ';')) goto l303;
  yyprintf((stderr, "  ok   %s @ %s\n", "DecEntity", G->buf+G->pos));
  return 1;
  l303:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "DecEntity", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HexEntity(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HexEntity"));  if (!yymatchChar(G, '&')) goto l306;  if (!yymatchChar(G, '#')) goto l306;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\000\000\000\000\000\001\000\000\000\001\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l306;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\176\000\000\000\176\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l306;
  l307:;	
  {  int yypos308= G->pos, yythunkpos308= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\176\000\000\000\176\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l308;  goto l307;
  l308:;	  G->pos= yypos308; G->thunkpos= yythunkpos308;
  }  if (!yymatchChar(G, ';')) goto l306;
  yyprintf((stderr, "  ok   %s @ %s\n", "HexEntity", G->buf+G->pos));
  return 1;
  l306:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HexEntity", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ExtendedSpecialChar(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "ExtendedSpecialChar"));
  {  int yypos310= G->pos, yythunkpos310= G->thunkpos;  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_NOTES) )) goto l311;  if (!yymatchChar(G, '^')) goto l311;  goto l310;
  l311:;	  G->pos= yypos310; G->thunkpos= yythunkpos310;  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_MATH) )) goto l312;  if (!yymatchChar(G, '$')) goto l312;  goto l310;
  l312:;	  G->pos= yypos310; G->thunkpos= yythunkpos310;  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_TABLE) )) goto l309;  if (!yymatchChar(G, '|')) goto l309;
  }
  l310:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "ExtendedSpecialChar", G->buf+G->pos));
  return 1;
  l309:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ExtendedSpecialChar", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Alphanumeric(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Alphanumeric"));
  {  int yypos314= G->pos, yythunkpos314= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\376\377\377\007\376\377\377\007\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l315;  goto l314;
  l315:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\200")) goto l316;  goto l314;
  l316:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\201")) goto l317;  goto l314;
  l317:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\202")) goto l318;  goto l314;
  l318:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\203")) goto l319;  goto l314;
  l319:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\204")) goto l320;  goto l314;
  l320:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\205")) goto l321;  goto l314;
  l321:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\206")) goto l322;  goto l314;
  l322:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\207")) goto l323;  goto l314;
  l323:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\210")) goto l324;  goto l314;
  l324:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\211")) goto l325;  goto l314;
  l325:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\212")) goto l326;  goto l314;
  l326:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\213")) goto l327;  goto l314;
  l327:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\214")) goto l328;  goto l314;
  l328:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\215")) goto l329;  goto l314;
  l329:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\216")) goto l330;  goto l314;
  l330:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\217")) goto l331;  goto l314;
  l331:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\220")) goto l332;  goto l314;
  l332:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\221")) goto l333;  goto l314;
  l333:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\222")) goto l334;  goto l314;
  l334:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\223")) goto l335;  goto l314;
  l335:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\224")) goto l336;  goto l314;
  l336:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\225")) goto l337;  goto l314;
  l337:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\226")) goto l338;  goto l314;
  l338:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\227")) goto l339;  goto l314;
  l339:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\230")) goto l340;  goto l314;
  l340:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\231")) goto l341;  goto l314;
  l341:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\232")) goto l342;  goto l314;
  l342:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\233")) goto l343;  goto l314;
  l343:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\234")) goto l344;  goto l314;
  l344:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\235")) goto l345;  goto l314;
  l345:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\236")) goto l346;  goto l314;
  l346:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\237")) goto l347;  goto l314;
  l347:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\240")) goto l348;  goto l314;
  l348:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\241")) goto l349;  goto l314;
  l349:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\242")) goto l350;  goto l314;
  l350:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\243")) goto l351;  goto l314;
  l351:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\244")) goto l352;  goto l314;
  l352:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\245")) goto l353;  goto l314;
  l353:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\246")) goto l354;  goto l314;
  l354:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\247")) goto l355;  goto l314;
  l355:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\250")) goto l356;  goto l314;
  l356:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\251")) goto l357;  goto l314;
  l357:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\252")) goto l358;  goto l314;
  l358:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\253")) goto l359;  goto l314;
  l359:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\254")) goto l360;  goto l314;
  l360:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\255")) goto l361;  goto l314;
  l361:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\256")) goto l362;  goto l314;
  l362:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\257")) goto l363;  goto l314;
  l363:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\260")) goto l364;  goto l314;
  l364:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\261")) goto l365;  goto l314;
  l365:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\262")) goto l366;  goto l314;
  l366:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\263")) goto l367;  goto l314;
  l367:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\264")) goto l368;  goto l314;
  l368:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\265")) goto l369;  goto l314;
  l369:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\266")) goto l370;  goto l314;
  l370:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\267")) goto l371;  goto l314;
  l371:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\270")) goto l372;  goto l314;
  l372:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\271")) goto l373;  goto l314;
  l373:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\272")) goto l374;  goto l314;
  l374:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\273")) goto l375;  goto l314;
  l375:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\274")) goto l376;  goto l314;
  l376:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\275")) goto l377;  goto l314;
  l377:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\276")) goto l378;  goto l314;
  l378:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\277")) goto l379;  goto l314;
  l379:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\300")) goto l380;  goto l314;
  l380:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\301")) goto l381;  goto l314;
  l381:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\302")) goto l382;  goto l314;
  l382:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\303")) goto l383;  goto l314;
  l383:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\304")) goto l384;  goto l314;
  l384:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\305")) goto l385;  goto l314;
  l385:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\306")) goto l386;  goto l314;
  l386:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\307")) goto l387;  goto l314;
  l387:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\310")) goto l388;  goto l314;
  l388:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\311")) goto l389;  goto l314;
  l389:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\312")) goto l390;  goto l314;
  l390:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\313")) goto l391;  goto l314;
  l391:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\314")) goto l392;  goto l314;
  l392:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\315")) goto l393;  goto l314;
  l393:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\316")) goto l394;  goto l314;
  l394:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\317")) goto l395;  goto l314;
  l395:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\320")) goto l396;  goto l314;
  l396:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\321")) goto l397;  goto l314;
  l397:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\322")) goto l398;  goto l314;
  l398:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\323")) goto l399;  goto l314;
  l399:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\324")) goto l400;  goto l314;
  l400:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\325")) goto l401;  goto l314;
  l401:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\326")) goto l402;  goto l314;
  l402:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\327")) goto l403;  goto l314;
  l403:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\330")) goto l404;  goto l314;
  l404:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\331")) goto l405;  goto l314;
  l405:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\332")) goto l406;  goto l314;
  l406:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\333")) goto l407;  goto l314;
  l407:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\334")) goto l408;  goto l314;
  l408:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\335")) goto l409;  goto l314;
  l409:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\336")) goto l410;  goto l314;
  l410:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\337")) goto l411;  goto l314;
  l411:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\340")) goto l412;  goto l314;
  l412:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\341")) goto l413;  goto l314;
  l413:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\342")) goto l414;  goto l314;
  l414:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\343")) goto l415;  goto l314;
  l415:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\344")) goto l416;  goto l314;
  l416:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\345")) goto l417;  goto l314;
  l417:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\346")) goto l418;  goto l314;
  l418:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\347")) goto l419;  goto l314;
  l419:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\350")) goto l420;  goto l314;
  l420:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\351")) goto l421;  goto l314;
  l421:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\352")) goto l422;  goto l314;
  l422:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\353")) goto l423;  goto l314;
  l423:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\354")) goto l424;  goto l314;
  l424:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\355")) goto l425;  goto l314;
  l425:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\356")) goto l426;  goto l314;
  l426:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\357")) goto l427;  goto l314;
  l427:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\360")) goto l428;  goto l314;
  l428:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\361")) goto l429;  goto l314;
  l429:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\362")) goto l430;  goto l314;
  l430:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\363")) goto l431;  goto l314;
  l431:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\364")) goto l432;  goto l314;
  l432:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\365")) goto l433;  goto l314;
  l433:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\366")) goto l434;  goto l314;
  l434:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\367")) goto l435;  goto l314;
  l435:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\370")) goto l436;  goto l314;
  l436:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\371")) goto l437;  goto l314;
  l437:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\372")) goto l438;  goto l314;
  l438:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\373")) goto l439;  goto l314;
  l439:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\374")) goto l440;  goto l314;
  l440:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\375")) goto l441;  goto l314;
  l441:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\376")) goto l442;  goto l314;
  l442:;	  G->pos= yypos314; G->thunkpos= yythunkpos314;  if (!yymatchString(G, "\377")) goto l313;
  }
  l314:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Alphanumeric", G->buf+G->pos));
  return 1;
  l313:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Alphanumeric", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_NormalChar(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "NormalChar"));
  {  int yypos444= G->pos, yythunkpos444= G->thunkpos;
  {  int yypos445= G->pos, yythunkpos445= G->thunkpos;  if (!yy_SpecialChar(G)) { goto l446; }  goto l445;
  l446:;	  G->pos= yypos445; G->thunkpos= yythunkpos445;  if (!yy_Spacechar(G)) { goto l447; }  goto l445;
  l447:;	  G->pos= yypos445; G->thunkpos= yythunkpos445;  if (!yy_Newline(G)) { goto l444; }
  }
  l445:;	  goto l443;
  l444:;	  G->pos= yypos444; G->thunkpos= yythunkpos444;
  }  if (!yymatchDot(G)) goto l443;
  yyprintf((stderr, "  ok   %s @ %s\n", "NormalChar", G->buf+G->pos));
  return 1;
  l443:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "NormalChar", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Symbol(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Symbol"));  if (!yy_SpecialChar(G)) { goto l448; }
  yyprintf((stderr, "  ok   %s @ %s\n", "Symbol", G->buf+G->pos));
  return 1;
  l448:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Symbol", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_EscapedChar(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "EscapedChar"));  if (!yymatchChar(G, '\\')) goto l449;
  {  int yypos450= G->pos, yythunkpos450= G->thunkpos;  if (!yy_Newline(G)) { goto l450; }  goto l449;
  l450:;	  G->pos= yypos450; G->thunkpos= yythunkpos450;
  }
  {  int yypos451= G->pos, yythunkpos451= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\012\157\000\120\000\000\000\270\001\000\000\170\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l452;  goto l451;
  l452:;	  G->pos= yypos451; G->thunkpos= yythunkpos451;  if (!yy_ExtendedSpecialChar(G)) { goto l449; }
  }
  l451:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "EscapedChar", G->buf+G->pos));
  return 1;
  l449:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "EscapedChar", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Entity(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "Entity"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l453;  if (!yy_LocMarker(G)) { goto l453; }  yyDo(G, yySet, -1, 0);
  {  int yypos454= G->pos, yythunkpos454= G->thunkpos;  if (!yy_HexEntity(G)) { goto l455; }  goto l454;
  l455:;	  G->pos= yypos454; G->thunkpos= yythunkpos454;  if (!yy_DecEntity(G)) { goto l456; }  goto l454;
  l456:;	  G->pos= yypos454; G->thunkpos= yythunkpos454;  if (!yy_CharEntity(G)) { goto l453; }
  }
  l454:;	  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l453;  yyDo(G, yy_1_Entity, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Entity", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l453:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Entity", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_RawHtml(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "RawHtml"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l457;  if (!yy_LocMarker(G)) { goto l457; }  yyDo(G, yySet, -1, 0);
  {  int yypos458= G->pos, yythunkpos458= G->thunkpos;  if (!yy_HtmlComment(G)) { goto l459; }  goto l458;
  l459:;	  G->pos= yypos458; G->thunkpos= yythunkpos458;  if (!yy_HtmlBlockScript(G)) { goto l460; }  goto l458;
  l460:;	  G->pos= yypos458; G->thunkpos= yythunkpos458;  if (!yy_HtmlTag(G)) { goto l457; }
  }
  l458:;	  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l457;  yyDo(G, yy_1_RawHtml, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "RawHtml", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l457:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "RawHtml", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Mark(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 2, 0);
  yyprintf((stderr, "%s\n", "Mark"));  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_MARK) )) goto l461;  if (!yy_MarkTagOpen(G)) { goto l461; }  yyDo(G, yySet, -2, 0);  if (!yy_MarkTagText(G)) { goto l461; }  if (!yy_MarkTagClose(G)) { goto l461; }  yyDo(G, yySet, -1, 0);  yyDo(G, yy_1_Mark, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Mark", G->buf+G->pos));  yyDo(G, yyPop, 2, 0);
  return 1;
  l461:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Mark", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Code(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "Code"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l462;
  {  int yypos463= G->pos, yythunkpos463= G->thunkpos;  if (!yy_Ticks1(G)) { goto l464; }  yyDo(G, yySet, -1, 0);  if (!yy_Sp(G)) { goto l464; }
  {  int yypos467= G->pos, yythunkpos467= G->thunkpos;
  {  int yypos471= G->pos, yythunkpos471= G->thunkpos;  if (!yymatchChar(G, '`')) goto l471;  goto l468;
  l471:;	  G->pos= yypos471; G->thunkpos= yythunkpos471;
  }  if (!yy_Nonspacechar(G)) { goto l468; }
  l469:;	
  {  int yypos470= G->pos, yythunkpos470= G->thunkpos;
  {  int yypos472= G->pos, yythunkpos472= G->thunkpos;  if (!yymatchChar(G, '`')) goto l472;  goto l470;
  l472:;	  G->pos= yypos472; G->thunkpos= yythunkpos472;
  }  if (!yy_Nonspacechar(G)) { goto l470; }  goto l469;
  l470:;	  G->pos= yypos470; G->thunkpos= yythunkpos470;
  }  goto l467;
  l468:;	  G->pos= yypos467; G->thunkpos= yythunkpos467;
  {  int yypos474= G->pos, yythunkpos474= G->thunkpos;  if (!yy_Ticks1(G)) { goto l474; }  goto l473;
  l474:;	  G->pos= yypos474; G->thunkpos= yythunkpos474;
  }  if (!yymatchChar(G, '`')) goto l473;
  l475:;	
  {  int yypos476= G->pos, yythunkpos476= G->thunkpos;  if (!yymatchChar(G, '`')) goto l476;  goto l475;
  l476:;	  G->pos= yypos476; G->thunkpos= yythunkpos476;
  }  goto l467;
  l473:;	  G->pos= yypos467; G->thunkpos= yythunkpos467;
  {  int yypos477= G->pos, yythunkpos477= G->thunkpos;  if (!yy_Sp(G)) { goto l477; }  if (!yy_Ticks1(G)) { goto l477; }  goto l464;
  l477:;	  G->pos= yypos477; G->thunkpos= yythunkpos477;
  }
  {  int yypos478= G->pos, yythunkpos478= G->thunkpos;  if (!yy_Spacechar(G)) { goto l479; }  goto l478;
  l479:;	  G->pos= yypos478; G->thunkpos= yythunkpos478;  if (!yy_Newline(G)) { goto l464; }
  {  int yypos480= G->pos, yythunkpos480= G->thunkpos;  if (!yy_BlankLine(G)) { goto l480; }  goto l464;
  l480:;	  G->pos= yypos480; G->thunkpos= yythunkpos480;
  }
  }
  l478:;	
  }
  l467:;	
  l465:;	
  {  int yypos466= G->pos, yythunkpos466= G->thunkpos;
  {  int yypos481= G->pos, yythunkpos481= G->thunkpos;
  {  int yypos485= G->pos, yythunkpos485= G->thunkpos;  if (!yymatchChar(G, '`')) goto l485;  goto l482;
  l485:;	  G->pos= yypos485; G->thunkpos= yythunkpos485;
  }  if (!yy_Nonspacechar(G)) { goto l482; }
  l483:;	
  {  int yypos484= G->pos, yythunkpos484= G->thunkpos;
  {  int yypos486= G->pos, yythunkpos486= G->thunkpos;  if (!yymatchChar(G, '`')) goto l486;  goto l484;
  l486:;	  G->pos= yypos486; G->thunkpos= yythunkpos486;
  }  if (!yy_Nonspacechar(G)) { goto l484; }  goto l483;
  l484:;	  G->pos= yypos484; G->thunkpos= yythunkpos484;
  }  goto l481;
  l482:;	  G->pos= yypos481; G->thunkpos= yythunkpos481;
  {  int yypos488= G->pos, yythunkpos488= G->thunkpos;  if (!yy_Ticks1(G)) { goto l488; }  goto l487;
  l488:;	  G->pos= yypos488; G->thunkpos= yythunkpos488;
  }  if (!yymatchChar(G, '`')) goto l487;
  l489:;	
  {  int yypos490= G->pos, yythunkpos490= G->thunkpos;  if (!yymatchChar(G, '`')) goto l490;  goto l489;
  l490:;	  G->pos= yypos490; G->thunkpos= yythunkpos490;
  }  goto l481;
  l487:;	  G->pos= yypos481; G->thunkpos= yythunkpos481;
  {  int yypos491= G->pos, yythunkpos491= G->thunkpos;  if (!yy_Sp(G)) { goto l491; }  if (!yy_Ticks1(G)) { goto l491; }  goto l466;
  l491:;	  G->pos= yypos491; G->thunkpos= yythunkpos491;
  }
  {  int yypos492= G->pos, yythunkpos492= G->thunkpos;  if (!yy_Spacechar(G)) { goto l493; }  goto l492;
  l493:;	  G->pos= yypos492; G->thunkpos= yythunkpos492;  if (!yy_Newline(G)) { goto l466; }
  {  int yypos494= G->pos, yythunkpos494= G->thunkpos;  if (!yy_BlankLine(G)) { goto l494; }  goto l466;
  l494:;	  G->pos= yypos494; G->thunkpos= yythunkpos494;
  }
  }
  l492:;	
  }
  l481:;	  goto l465;
  l466:;	  G->pos= yypos466; G->thunkpos= yythunkpos466;
  }  if (!yy_Sp(G)) { goto l464; }  if (!yy_Ticks1(G)) { goto l464; }  goto l463;
  l464:;	  G->pos= yypos463; G->thunkpos= yythunkpos463;  if (!yy_Ticks2(G)) { goto l495; }  yyDo(G, yySet, -1, 0);  if (!yy_Sp(G)) { goto l495; }
  {  int yypos498= G->pos, yythunkpos498= G->thunkpos;
  {  int yypos502= G->pos, yythunkpos502= G->thunkpos;  if (!yymatchChar(G, '`')) goto l502;  goto l499;
  l502:;	  G->pos= yypos502; G->thunkpos= yythunkpos502;
  }  if (!yy_Nonspacechar(G)) { goto l499; }
  l500:;	
  {  int yypos501= G->pos, yythunkpos501= G->thunkpos;
  {  int yypos503= G->pos, yythunkpos503= G->thunkpos;  if (!yymatchChar(G, '`')) goto l503;  goto l501;
  l503:;	  G->pos= yypos503; G->thunkpos= yythunkpos503;
  }  if (!yy_Nonspacechar(G)) { goto l501; }  goto l500;
  l501:;	  G->pos= yypos501; G->thunkpos= yythunkpos501;
  }  goto l498;
  l499:;	  G->pos= yypos498; G->thunkpos= yythunkpos498;
  {  int yypos505= G->pos, yythunkpos505= G->thunkpos;  if (!yy_Ticks2(G)) { goto l505; }  goto l504;
  l505:;	  G->pos= yypos505; G->thunkpos= yythunkpos505;
  }  if (!yymatchChar(G, '`')) goto l504;
  l506:;	
  {  int yypos507= G->pos, yythunkpos507= G->thunkpos;  if (!yymatchChar(G, '`')) goto l507;  goto l506;
  l507:;	  G->pos= yypos507; G->thunkpos= yythunkpos507;
  }  goto l498;
  l504:;	  G->pos= yypos498; G->thunkpos= yythunkpos498;
  {  int yypos508= G->pos, yythunkpos508= G->thunkpos;  if (!yy_Sp(G)) { goto l508; }  if (!yy_Ticks2(G)) { goto l508; }  goto l495;
  l508:;	  G->pos= yypos508; G->thunkpos= yythunkpos508;
  }
  {  int yypos509= G->pos, yythunkpos509= G->thunkpos;  if (!yy_Spacechar(G)) { goto l510; }  goto l509;
  l510:;	  G->pos= yypos509; G->thunkpos= yythunkpos509;  if (!yy_Newline(G)) { goto l495; }
  {  int yypos511= G->pos, yythunkpos511= G->thunkpos;  if (!yy_BlankLine(G)) { goto l511; }  goto l495;
  l511:;	  G->pos= yypos511; G->thunkpos= yythunkpos511;
  }
  }
  l509:;	
  }
  l498:;	
  l496:;	
  {  int yypos497= G->pos, yythunkpos497= G->thunkpos;
  {  int yypos512= G->pos, yythunkpos512= G->thunkpos;
  {  int yypos516= G->pos, yythunkpos516= G->thunkpos;  if (!yymatchChar(G, '`')) goto l516;  goto l513;
  l516:;	  G->pos= yypos516; G->thunkpos= yythunkpos516;
  }  if (!yy_Nonspacechar(G)) { goto l513; }
  l514:;	
  {  int yypos515= G->pos, yythunkpos515= G->thunkpos;
  {  int yypos517= G->pos, yythunkpos517= G->thunkpos;  if (!yymatchChar(G, '`')) goto l517;  goto l515;
  l517:;	  G->pos= yypos517; G->thunkpos= yythunkpos517;
  }  if (!yy_Nonspacechar(G)) { goto l515; }  goto l514;
  l515:;	  G->pos= yypos515; G->thunkpos= yythunkpos515;
  }  goto l512;
  l513:;	  G->pos= yypos512; G->thunkpos= yythunkpos512;
  {  int yypos519= G->pos, yythunkpos519= G->thunkpos;  if (!yy_Ticks2(G)) { goto l519; }  goto l518;
  l519:;	  G->pos= yypos519; G->thunkpos= yythunkpos519;
  }  if (!yymatchChar(G, '`')) goto l518;
  l520:;	
  {  int yypos521= G->pos, yythunkpos521= G->thunkpos;  if (!yymatchChar(G, '`')) goto l521;  goto l520;
  l521:;	  G->pos= yypos521; G->thunkpos= yythunkpos521;
  }  goto l512;
  l518:;	  G->pos= yypos512; G->thunkpos= yythunkpos512;
  {  int yypos522= G->pos, yythunkpos522= G->thunkpos;  if (!yy_Sp(G)) { goto l522; }  if (!yy_Ticks2(G)) { goto l522; }  goto l497;
  l522:;	  G->pos= yypos522; G->thunkpos= yythunkpos522;
  }
  {  int yypos523= G->pos, yythunkpos523= G->thunkpos;  if (!yy_Spacechar(G)) { goto l524; }  goto l523;
  l524:;	  G->pos= yypos523; G->thunkpos= yythunkpos523;  if (!yy_Newline(G)) { goto l497; }
  {  int yypos525= G->pos, yythunkpos525= G->thunkpos;  if (!yy_BlankLine(G)) { goto l525; }  goto l497;
  l525:;	  G->pos= yypos525; G->thunkpos= yythunkpos525;
  }
  }
  l523:;	
  }
  l512:;	  goto l496;
  l497:;	  G->pos= yypos497; G->thunkpos= yythunkpos497;
  }  if (!yy_Sp(G)) { goto l495; }  if (!yy_Ticks2(G)) { goto l495; }  goto l463;
  l495:;	  G->pos= yypos463; G->thunkpos= yythunkpos463;
  {  int yypos527= G->pos, yythunkpos527= G->thunkpos;  if (!yy_FencedCodeBlockStartTickLine(G)) { goto l527; }  goto l526;
  l527:;	  G->pos= yypos527; G->thunkpos= yythunkpos527;
  }  if (!yy_Ticks3(G)) { goto l526; }  yyDo(G, yySet, -1, 0);  if (!yy_Sp(G)) { goto l526; }
  {  int yypos530= G->pos, yythunkpos530= G->thunkpos;
  {  int yypos534= G->pos, yythunkpos534= G->thunkpos;  if (!yymatchChar(G, '`')) goto l534;  goto l531;
  l534:;	  G->pos= yypos534; G->thunkpos= yythunkpos534;
  }  if (!yy_Nonspacechar(G)) { goto l531; }
  l532:;	
  {  int yypos533= G->pos, yythunkpos533= G->thunkpos;
  {  int yypos535= G->pos, yythunkpos535= G->thunkpos;  if (!yymatchChar(G, '`')) goto l535;  goto l533;
  l535:;	  G->pos= yypos535; G->thunkpos= yythunkpos535;
  }  if (!yy_Nonspacechar(G)) { goto l533; }  goto l532;
  l533:;	  G->pos= yypos533; G->thunkpos= yythunkpos533;
  }  goto l530;
  l531:;	  G->pos= yypos530; G->thunkpos= yythunkpos530;
  {  int yypos537= G->pos, yythunkpos537= G->thunkpos;  if (!yy_Ticks3(G)) { goto l537; }  goto l536;
  l537:;	  G->pos= yypos537; G->thunkpos= yythunkpos537;
  }  if (!yymatchChar(G, '`')) goto l536;
  l538:;	
  {  int yypos539= G->pos, yythunkpos539= G->thunkpos;  if (!yymatchChar(G, '`')) goto l539;  goto l538;
  l539:;	  G->pos= yypos539; G->thunkpos= yythunkpos539;
  }  goto l530;
  l536:;	  G->pos= yypos530; G->thunkpos= yythunkpos530;
  {  int yypos540= G->pos, yythunkpos540= G->thunkpos;  if (!yy_Sp(G)) { goto l540; }  if (!yy_Ticks3(G)) { goto l540; }  goto l526;
  l540:;	  G->pos= yypos540; G->thunkpos= yythunkpos540;
  }
  {  int yypos541= G->pos, yythunkpos541= G->thunkpos;  if (!yy_Spacechar(G)) { goto l542; }  goto l541;
  l542:;	  G->pos= yypos541; G->thunkpos= yythunkpos541;  if (!yy_Newline(G)) { goto l526; }
  {  int yypos543= G->pos, yythunkpos543= G->thunkpos;  if (!yy_BlankLine(G)) { goto l543; }  goto l526;
  l543:;	  G->pos= yypos543; G->thunkpos= yythunkpos543;
  }
  }
  l541:;	
  }
  l530:;	
  l528:;	
  {  int yypos529= G->pos, yythunkpos529= G->thunkpos;
  {  int yypos544= G->pos, yythunkpos544= G->thunkpos;
  {  int yypos548= G->pos, yythunkpos548= G->thunkpos;  if (!yymatchChar(G, '`')) goto l548;  goto l545;
  l548:;	  G->pos= yypos548; G->thunkpos= yythunkpos548;
  }  if (!yy_Nonspacechar(G)) { goto l545; }
  l546:;	
  {  int yypos547= G->pos, yythunkpos547= G->thunkpos;
  {  int yypos549= G->pos, yythunkpos549= G->thunkpos;  if (!yymatchChar(G, '`')) goto l549;  goto l547;
  l549:;	  G->pos= yypos549; G->thunkpos= yythunkpos549;
  }  if (!yy_Nonspacechar(G)) { goto l547; }  goto l546;
  l547:;	  G->pos= yypos547; G->thunkpos= yythunkpos547;
  }  goto l544;
  l545:;	  G->pos= yypos544; G->thunkpos= yythunkpos544;
  {  int yypos551= G->pos, yythunkpos551= G->thunkpos;  if (!yy_Ticks3(G)) { goto l551; }  goto l550;
  l551:;	  G->pos= yypos551; G->thunkpos= yythunkpos551;
  }  if (!yymatchChar(G, '`')) goto l550;
  l552:;	
  {  int yypos553= G->pos, yythunkpos553= G->thunkpos;  if (!yymatchChar(G, '`')) goto l553;  goto l552;
  l553:;	  G->pos= yypos553; G->thunkpos= yythunkpos553;
  }  goto l544;
  l550:;	  G->pos= yypos544; G->thunkpos= yythunkpos544;
  {  int yypos554= G->pos, yythunkpos554= G->thunkpos;  if (!yy_Sp(G)) { goto l554; }  if (!yy_Ticks3(G)) { goto l554; }  goto l529;
  l554:;	  G->pos= yypos554; G->thunkpos= yythunkpos554;
  }
  {  int yypos555= G->pos, yythunkpos555= G->thunkpos;  if (!yy_Spacechar(G)) { goto l556; }  goto l555;
  l556:;	  G->pos= yypos555; G->thunkpos= yythunkpos555;  if (!yy_Newline(G)) { goto l529; }
  {  int yypos557= G->pos, yythunkpos557= G->thunkpos;  if (!yy_BlankLine(G)) { goto l557; }  goto l529;
  l557:;	  G->pos= yypos557; G->thunkpos= yythunkpos557;
  }
  }
  l555:;	
  }
  l544:;	  goto l528;
  l529:;	  G->pos= yypos529; G->thunkpos= yythunkpos529;
  }  if (!yy_Sp(G)) { goto l526; }  if (!yy_Ticks3(G)) { goto l526; }  goto l463;
  l526:;	  G->pos= yypos463; G->thunkpos= yythunkpos463;  if (!yy_Ticks4(G)) { goto l558; }  yyDo(G, yySet, -1, 0);  if (!yy_Sp(G)) { goto l558; }
  {  int yypos561= G->pos, yythunkpos561= G->thunkpos;
  {  int yypos565= G->pos, yythunkpos565= G->thunkpos;  if (!yymatchChar(G, '`')) goto l565;  goto l562;
  l565:;	  G->pos= yypos565; G->thunkpos= yythunkpos565;
  }  if (!yy_Nonspacechar(G)) { goto l562; }
  l563:;	
  {  int yypos564= G->pos, yythunkpos564= G->thunkpos;
  {  int yypos566= G->pos, yythunkpos566= G->thunkpos;  if (!yymatchChar(G, '`')) goto l566;  goto l564;
  l566:;	  G->pos= yypos566; G->thunkpos= yythunkpos566;
  }  if (!yy_Nonspacechar(G)) { goto l564; }  goto l563;
  l564:;	  G->pos= yypos564; G->thunkpos= yythunkpos564;
  }  goto l561;
  l562:;	  G->pos= yypos561; G->thunkpos= yythunkpos561;
  {  int yypos568= G->pos, yythunkpos568= G->thunkpos;  if (!yy_Ticks4(G)) { goto l568; }  goto l567;
  l568:;	  G->pos= yypos568; G->thunkpos= yythunkpos568;
  }  if (!yymatchChar(G, '`')) goto l567;
  l569:;	
  {  int yypos570= G->pos, yythunkpos570= G->thunkpos;  if (!yymatchChar(G, '`')) goto l570;  goto l569;
  l570:;	  G->pos= yypos570; G->thunkpos= yythunkpos570;
  }  goto l561;
  l567:;	  G->pos= yypos561; G->thunkpos= yythunkpos561;
  {  int yypos571= G->pos, yythunkpos571= G->thunkpos;  if (!yy_Sp(G)) { goto l571; }  if (!yy_Ticks4(G)) { goto l571; }  goto l558;
  l571:;	  G->pos= yypos571; G->thunkpos= yythunkpos571;
  }
  {  int yypos572= G->pos, yythunkpos572= G->thunkpos;  if (!yy_Spacechar(G)) { goto l573; }  goto l572;
  l573:;	  G->pos= yypos572; G->thunkpos= yythunkpos572;  if (!yy_Newline(G)) { goto l558; }
  {  int yypos574= G->pos, yythunkpos574= G->thunkpos;  if (!yy_BlankLine(G)) { goto l574; }  goto l558;
  l574:;	  G->pos= yypos574; G->thunkpos= yythunkpos574;
  }
  }
  l572:;	
  }
  l561:;	
  l559:;	
  {  int yypos560= G->pos, yythunkpos560= G->thunkpos;
  {  int yypos575= G->pos, yythunkpos575= G->thunkpos;
  {  int yypos579= G->pos, yythunkpos579= G->thunkpos;  if (!yymatchChar(G, '`')) goto l579;  goto l576;
  l579:;	  G->pos= yypos579; G->thunkpos= yythunkpos579;
  }  if (!yy_Nonspacechar(G)) { goto l576; }
  l577:;	
  {  int yypos578= G->pos, yythunkpos578= G->thunkpos;
  {  int yypos580= G->pos, yythunkpos580= G->thunkpos;  if (!yymatchChar(G, '`')) goto l580;  goto l578;
  l580:;	  G->pos= yypos580; G->thunkpos= yythunkpos580;
  }  if (!yy_Nonspacechar(G)) { goto l578; }  goto l577;
  l578:;	  G->pos= yypos578; G->thunkpos= yythunkpos578;
  }  goto l575;
  l576:;	  G->pos= yypos575; G->thunkpos= yythunkpos575;
  {  int yypos582= G->pos, yythunkpos582= G->thunkpos;  if (!yy_Ticks4(G)) { goto l582; }  goto l581;
  l582:;	  G->pos= yypos582; G->thunkpos= yythunkpos582;
  }  if (!yymatchChar(G, '`')) goto l581;
  l583:;	
  {  int yypos584= G->pos, yythunkpos584= G->thunkpos;  if (!yymatchChar(G, '`')) goto l584;  goto l583;
  l584:;	  G->pos= yypos584; G->thunkpos= yythunkpos584;
  }  goto l575;
  l581:;	  G->pos= yypos575; G->thunkpos= yythunkpos575;
  {  int yypos585= G->pos, yythunkpos585= G->thunkpos;  if (!yy_Sp(G)) { goto l585; }  if (!yy_Ticks4(G)) { goto l585; }  goto l560;
  l585:;	  G->pos= yypos585; G->thunkpos= yythunkpos585;
  }
  {  int yypos586= G->pos, yythunkpos586= G->thunkpos;  if (!yy_Spacechar(G)) { goto l587; }  goto l586;
  l587:;	  G->pos= yypos586; G->thunkpos= yythunkpos586;  if (!yy_Newline(G)) { goto l560; }
  {  int yypos588= G->pos, yythunkpos588= G->thunkpos;  if (!yy_BlankLine(G)) { goto l588; }  goto l560;
  l588:;	  G->pos= yypos588; G->thunkpos= yythunkpos588;
  }
  }
  l586:;	
  }
  l575:;	  goto l559;
  l560:;	  G->pos= yypos560; G->thunkpos= yythunkpos560;
  }  if (!yy_Sp(G)) { goto l558; }  if (!yy_Ticks4(G)) { goto l558; }  goto l463;
  l558:;	  G->pos= yypos463; G->thunkpos= yythunkpos463;  if (!yy_Ticks5(G)) { goto l462; }  yyDo(G, yySet, -1, 0);  if (!yy_Sp(G)) { goto l462; }
  {  int yypos591= G->pos, yythunkpos591= G->thunkpos;
  {  int yypos595= G->pos, yythunkpos595= G->thunkpos;  if (!yymatchChar(G, '`')) goto l595;  goto l592;
  l595:;	  G->pos= yypos595; G->thunkpos= yythunkpos595;
  }  if (!yy_Nonspacechar(G)) { goto l592; }
  l593:;	
  {  int yypos594= G->pos, yythunkpos594= G->thunkpos;
  {  int yypos596= G->pos, yythunkpos596= G->thunkpos;  if (!yymatchChar(G, '`')) goto l596;  goto l594;
  l596:;	  G->pos= yypos596; G->thunkpos= yythunkpos596;
  }  if (!yy_Nonspacechar(G)) { goto l594; }  goto l593;
  l594:;	  G->pos= yypos594; G->thunkpos= yythunkpos594;
  }  goto l591;
  l592:;	  G->pos= yypos591; G->thunkpos= yythunkpos591;
  {  int yypos598= G->pos, yythunkpos598= G->thunkpos;  if (!yy_Ticks5(G)) { goto l598; }  goto l597;
  l598:;	  G->pos= yypos598; G->thunkpos= yythunkpos598;
  }  if (!yymatchChar(G, '`')) goto l597;
  l599:;	
  {  int yypos600= G->pos, yythunkpos600= G->thunkpos;  if (!yymatchChar(G, '`')) goto l600;  goto l599;
  l600:;	  G->pos= yypos600; G->thunkpos= yythunkpos600;
  }  goto l591;
  l597:;	  G->pos= yypos591; G->thunkpos= yythunkpos591;
  {  int yypos601= G->pos, yythunkpos601= G->thunkpos;  if (!yy_Sp(G)) { goto l601; }  if (!yy_Ticks5(G)) { goto l601; }  goto l462;
  l601:;	  G->pos= yypos601; G->thunkpos= yythunkpos601;
  }
  {  int yypos602= G->pos, yythunkpos602= G->thunkpos;  if (!yy_Spacechar(G)) { goto l603; }  goto l602;
  l603:;	  G->pos= yypos602; G->thunkpos= yythunkpos602;  if (!yy_Newline(G)) { goto l462; }
  {  int yypos604= G->pos, yythunkpos604= G->thunkpos;  if (!yy_BlankLine(G)) { goto l604; }  goto l462;
  l604:;	  G->pos= yypos604; G->thunkpos= yythunkpos604;
  }
  }
  l602:;	
  }
  l591:;	
  l589:;	
  {  int yypos590= G->pos, yythunkpos590= G->thunkpos;
  {  int yypos605= G->pos, yythunkpos605= G->thunkpos;
  {  int yypos609= G->pos, yythunkpos609= G->thunkpos;  if (!yymatchChar(G, '`')) goto l609;  goto l606;
  l609:;	  G->pos= yypos609; G->thunkpos= yythunkpos609;
  }  if (!yy_Nonspacechar(G)) { goto l606; }
  l607:;	
  {  int yypos608= G->pos, yythunkpos608= G->thunkpos;
  {  int yypos610= G->pos, yythunkpos610= G->thunkpos;  if (!yymatchChar(G, '`')) goto l610;  goto l608;
  l610:;	  G->pos= yypos610; G->thunkpos= yythunkpos610;
  }  if (!yy_Nonspacechar(G)) { goto l608; }  goto l607;
  l608:;	  G->pos= yypos608; G->thunkpos= yythunkpos608;
  }  goto l605;
  l606:;	  G->pos= yypos605; G->thunkpos= yythunkpos605;
  {  int yypos612= G->pos, yythunkpos612= G->thunkpos;  if (!yy_Ticks5(G)) { goto l612; }  goto l611;
  l612:;	  G->pos= yypos612; G->thunkpos= yythunkpos612;
  }  if (!yymatchChar(G, '`')) goto l611;
  l613:;	
  {  int yypos614= G->pos, yythunkpos614= G->thunkpos;  if (!yymatchChar(G, '`')) goto l614;  goto l613;
  l614:;	  G->pos= yypos614; G->thunkpos= yythunkpos614;
  }  goto l605;
  l611:;	  G->pos= yypos605; G->thunkpos= yythunkpos605;
  {  int yypos615= G->pos, yythunkpos615= G->thunkpos;  if (!yy_Sp(G)) { goto l615; }  if (!yy_Ticks5(G)) { goto l615; }  goto l590;
  l615:;	  G->pos= yypos615; G->thunkpos= yythunkpos615;
  }
  {  int yypos616= G->pos, yythunkpos616= G->thunkpos;  if (!yy_Spacechar(G)) { goto l617; }  goto l616;
  l617:;	  G->pos= yypos616; G->thunkpos= yythunkpos616;  if (!yy_Newline(G)) { goto l590; }
  {  int yypos618= G->pos, yythunkpos618= G->thunkpos;  if (!yy_BlankLine(G)) { goto l618; }  goto l590;
  l618:;	  G->pos= yypos618; G->thunkpos= yythunkpos618;
  }
  }
  l616:;	
  }
  l605:;	  goto l589;
  l590:;	  G->pos= yypos590; G->thunkpos= yythunkpos590;
  }  if (!yy_Sp(G)) { goto l462; }  if (!yy_Ticks5(G)) { goto l462; }
  }
  l463:;	  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l462;  yyDo(G, yy_1_Code, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Code", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l462:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Code", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_InlineNote(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "InlineNote"));  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_NOTES) )) goto l619;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l619;  if (!yy_LocMarker(G)) { goto l619; }  yyDo(G, yySet, -1, 0);  if (!yymatchString(G, "^[")) goto l619;
  {  int yypos622= G->pos, yythunkpos622= G->thunkpos;  if (!yymatchChar(G, ']')) goto l622;  goto l619;
  l622:;	  G->pos= yypos622; G->thunkpos= yythunkpos622;
  }  if (!yy_Inline(G)) { goto l619; }
  l620:;	
  {  int yypos621= G->pos, yythunkpos621= G->thunkpos;
  {  int yypos623= G->pos, yythunkpos623= G->thunkpos;  if (!yymatchChar(G, ']')) goto l623;  goto l621;
  l623:;	  G->pos= yypos623; G->thunkpos= yythunkpos623;
  }  if (!yy_Inline(G)) { goto l621; }  goto l620;
  l621:;	  G->pos= yypos621; G->thunkpos= yythunkpos621;
  }  if (!yymatchChar(G, ']')) goto l619;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l619;  yyDo(G, yy_1_InlineNote, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "InlineNote", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l619:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "InlineNote", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_NoteReference(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "NoteReference"));  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_NOTES) )) goto l624;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l624;  if (!yy_RawNoteReference(G)) { goto l624; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l624;  yyDo(G, yy_1_NoteReference, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "NoteReference", G->buf+G->pos));
  return 1;
  l624:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "NoteReference", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Link(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Link"));
  {  int yypos626= G->pos, yythunkpos626= G->thunkpos;  if (!yy_ExplicitLink(G)) { goto l627; }  goto l626;
  l627:;	  G->pos= yypos626; G->thunkpos= yythunkpos626;  if (!yy_ReferenceLink(G)) { goto l628; }  goto l626;
  l628:;	  G->pos= yypos626; G->thunkpos= yythunkpos626;  if (!yy_AutoLink(G)) { goto l625; }
  }
  l626:;	  yyDo(G, yy_1_Link, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Link", G->buf+G->pos));
  return 1;
  l625:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Link", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Image(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Image"));  if (!yymatchChar(G, '!')) goto l629;
  {  int yypos630= G->pos, yythunkpos630= G->thunkpos;  if (!yy_ExplicitLink(G)) { goto l631; }  goto l630;
  l631:;	  G->pos= yypos630; G->thunkpos= yythunkpos630;  if (!yy_ExplicitLinkSize(G)) { goto l632; }  goto l630;
  l632:;	  G->pos= yypos630; G->thunkpos= yythunkpos630;  if (!yy_ReferenceLink(G)) { goto l629; }
  }
  l630:;	  yyDo(G, yy_1_Image, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Image", G->buf+G->pos));
  return 1;
  l629:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Image", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Strike(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "Strike"));  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_STRIKE) )) goto l633;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l633;  if (!yy_LocMarker(G)) { goto l633; }  yyDo(G, yySet, -1, 0);  if (!yymatchString(G, "~~")) goto l633;
  {  int yypos634= G->pos, yythunkpos634= G->thunkpos;  if (!yy_Whitespace(G)) { goto l634; }  goto l633;
  l634:;	  G->pos= yypos634; G->thunkpos= yythunkpos634;
  }
  {  int yypos637= G->pos, yythunkpos637= G->thunkpos;  if (!yymatchString(G, "~~")) goto l637;  goto l633;
  l637:;	  G->pos= yypos637; G->thunkpos= yythunkpos637;
  }  if (!yy_Inline(G)) { goto l633; }
  l635:;	
  {  int yypos636= G->pos, yythunkpos636= G->thunkpos;
  {  int yypos638= G->pos, yythunkpos638= G->thunkpos;  if (!yymatchString(G, "~~")) goto l638;  goto l636;
  l638:;	  G->pos= yypos638; G->thunkpos= yythunkpos638;
  }  if (!yy_Inline(G)) { goto l636; }  goto l635;
  l636:;	  G->pos= yypos636; G->thunkpos= yythunkpos636;
  }  yyText(G, G->begin, G->end);  if (!( STRIKE_POST )) goto l633;  if (!yymatchString(G, "~~")) goto l633;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l633;  yyDo(G, yy_1_Strike, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Strike", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l633:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Strike", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Emph(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Emph"));
  {  int yypos640= G->pos, yythunkpos640= G->thunkpos;  if (!yy_EmphStar(G)) { goto l641; }  goto l640;
  l641:;	  G->pos= yypos640; G->thunkpos= yythunkpos640;  if (!yy_EmphUl(G)) { goto l639; }
  }
  l640:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Emph", G->buf+G->pos));
  return 1;
  l639:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Emph", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Strong(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Strong"));
  {  int yypos643= G->pos, yythunkpos643= G->thunkpos;  if (!yy_StrongStar(G)) { goto l644; }  goto l643;
  l644:;	  G->pos= yypos643; G->thunkpos= yythunkpos643;  if (!yy_StrongUl(G)) { goto l642; }
  }
  l643:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Strong", G->buf+G->pos));
  return 1;
  l642:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Strong", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_UlOrStarLine(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "UlOrStarLine"));
  {  int yypos646= G->pos, yythunkpos646= G->thunkpos;  if (!yy_UlLine(G)) { goto l647; }  goto l646;
  l647:;	  G->pos= yypos646; G->thunkpos= yythunkpos646;  if (!yy_StarLine(G)) { goto l645; }
  }
  l646:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "UlOrStarLine", G->buf+G->pos));
  return 1;
  l645:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "UlOrStarLine", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Str(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Str"));  if (!yy_NormalChar(G)) { goto l648; }
  l649:;	
  {  int yypos650= G->pos, yythunkpos650= G->thunkpos;
  {  int yypos651= G->pos, yythunkpos651= G->thunkpos;  if (!yy_NormalChar(G)) { goto l652; }  goto l651;
  l652:;	  G->pos= yypos651; G->thunkpos= yythunkpos651;  if (!yymatchChar(G, '_')) goto l650;
  l653:;	
  {  int yypos654= G->pos, yythunkpos654= G->thunkpos;  if (!yymatchChar(G, '_')) goto l654;  goto l653;
  l654:;	  G->pos= yypos654; G->thunkpos= yythunkpos654;
  }
  {  int yypos655= G->pos, yythunkpos655= G->thunkpos;  if (!yy_Alphanumeric(G)) { goto l650; }  G->pos= yypos655; G->thunkpos= yythunkpos655;
  }
  }
  l651:;	  goto l649;
  l650:;	  G->pos= yypos650; G->thunkpos= yythunkpos650;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "Str", G->buf+G->pos));
  return 1;
  l648:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Str", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_InStyleTags(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "InStyleTags"));  if (!yy_StyleOpen(G)) { goto l656; }
  l657:;	
  {  int yypos658= G->pos, yythunkpos658= G->thunkpos;
  {  int yypos659= G->pos, yythunkpos659= G->thunkpos;  if (!yy_StyleClose(G)) { goto l659; }  goto l658;
  l659:;	  G->pos= yypos659; G->thunkpos= yythunkpos659;
  }  if (!yymatchDot(G)) goto l658;  goto l657;
  l658:;	  G->pos= yypos658; G->thunkpos= yythunkpos658;
  }  if (!yy_StyleClose(G)) { goto l656; }
  yyprintf((stderr, "  ok   %s @ %s\n", "InStyleTags", G->buf+G->pos));
  return 1;
  l656:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "InStyleTags", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_StyleClose(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "StyleClose"));  if (!yymatchChar(G, '<')) goto l660;  if (!yy_Spnl(G)) { goto l660; }  if (!yymatchChar(G, '/')) goto l660;
  {  int yypos661= G->pos, yythunkpos661= G->thunkpos;  if (!yymatchString(G, "style")) goto l662;  goto l661;
  l662:;	  G->pos= yypos661; G->thunkpos= yythunkpos661;  if (!yymatchString(G, "STYLE")) goto l660;
  }
  l661:;	  if (!yy_Spnl(G)) { goto l660; }  if (!yymatchChar(G, '>')) goto l660;
  yyprintf((stderr, "  ok   %s @ %s\n", "StyleClose", G->buf+G->pos));
  return 1;
  l660:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "StyleClose", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_StyleOpen(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "StyleOpen"));  if (!yymatchChar(G, '<')) goto l663;  if (!yy_Spnl(G)) { goto l663; }
  {  int yypos664= G->pos, yythunkpos664= G->thunkpos;  if (!yymatchString(G, "style")) goto l665;  goto l664;
  l665:;	  G->pos= yypos664; G->thunkpos= yythunkpos664;  if (!yymatchString(G, "STYLE")) goto l663;
  }
  l664:;	  if (!yy_Spnl(G)) { goto l663; }
  l666:;	
  {  int yypos667= G->pos, yythunkpos667= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l667; }  goto l666;
  l667:;	  G->pos= yypos667; G->thunkpos= yythunkpos667;
  }  if (!yymatchChar(G, '>')) goto l663;
  yyprintf((stderr, "  ok   %s @ %s\n", "StyleOpen", G->buf+G->pos));
  return 1;
  l663:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "StyleOpen", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockType(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockType"));
  {  int yypos669= G->pos, yythunkpos669= G->thunkpos;  if (!yymatchString(G, "address")) goto l670;  goto l669;
  l670:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "blockquote")) goto l671;  goto l669;
  l671:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "center")) goto l672;  goto l669;
  l672:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "dir")) goto l673;  goto l669;
  l673:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "div")) goto l674;  goto l669;
  l674:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "dl")) goto l675;  goto l669;
  l675:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "fieldset")) goto l676;  goto l669;
  l676:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "form")) goto l677;  goto l669;
  l677:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "h1")) goto l678;  goto l669;
  l678:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "h2")) goto l679;  goto l669;
  l679:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "h3")) goto l680;  goto l669;
  l680:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "h4")) goto l681;  goto l669;
  l681:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "h5")) goto l682;  goto l669;
  l682:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "h6")) goto l683;  goto l669;
  l683:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "hr")) goto l684;  goto l669;
  l684:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "isindex")) goto l685;  goto l669;
  l685:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "menu")) goto l686;  goto l669;
  l686:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "noframes")) goto l687;  goto l669;
  l687:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "noscript")) goto l688;  goto l669;
  l688:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "ol")) goto l689;  goto l669;
  l689:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchChar(G, 'p')) goto l690;  goto l669;
  l690:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "pre")) goto l691;  goto l669;
  l691:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "table")) goto l692;  goto l669;
  l692:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "ul")) goto l693;  goto l669;
  l693:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "dd")) goto l694;  goto l669;
  l694:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "dt")) goto l695;  goto l669;
  l695:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "frameset")) goto l696;  goto l669;
  l696:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "li")) goto l697;  goto l669;
  l697:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "tbody")) goto l698;  goto l669;
  l698:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "td")) goto l699;  goto l669;
  l699:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "tfoot")) goto l700;  goto l669;
  l700:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "th")) goto l701;  goto l669;
  l701:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "thead")) goto l702;  goto l669;
  l702:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "tr")) goto l703;  goto l669;
  l703:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "script")) goto l704;  goto l669;
  l704:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "ADDRESS")) goto l705;  goto l669;
  l705:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "BLOCKQUOTE")) goto l706;  goto l669;
  l706:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "CENTER")) goto l707;  goto l669;
  l707:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "DIR")) goto l708;  goto l669;
  l708:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "DIV")) goto l709;  goto l669;
  l709:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "DL")) goto l710;  goto l669;
  l710:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "FIELDSET")) goto l711;  goto l669;
  l711:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "FORM")) goto l712;  goto l669;
  l712:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "H1")) goto l713;  goto l669;
  l713:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "H2")) goto l714;  goto l669;
  l714:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "H3")) goto l715;  goto l669;
  l715:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "H4")) goto l716;  goto l669;
  l716:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "H5")) goto l717;  goto l669;
  l717:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "H6")) goto l718;  goto l669;
  l718:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "HR")) goto l719;  goto l669;
  l719:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "ISINDEX")) goto l720;  goto l669;
  l720:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "MENU")) goto l721;  goto l669;
  l721:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "NOFRAMES")) goto l722;  goto l669;
  l722:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "NOSCRIPT")) goto l723;  goto l669;
  l723:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "OL")) goto l724;  goto l669;
  l724:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchChar(G, 'P')) goto l725;  goto l669;
  l725:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "PRE")) goto l726;  goto l669;
  l726:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "TABLE")) goto l727;  goto l669;
  l727:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "UL")) goto l728;  goto l669;
  l728:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "DD")) goto l729;  goto l669;
  l729:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "DT")) goto l730;  goto l669;
  l730:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "FRAMESET")) goto l731;  goto l669;
  l731:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "LI")) goto l732;  goto l669;
  l732:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "TBODY")) goto l733;  goto l669;
  l733:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "TD")) goto l734;  goto l669;
  l734:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "TFOOT")) goto l735;  goto l669;
  l735:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "TH")) goto l736;  goto l669;
  l736:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "THEAD")) goto l737;  goto l669;
  l737:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "TR")) goto l738;  goto l669;
  l738:;	  G->pos= yypos669; G->thunkpos= yythunkpos669;  if (!yymatchString(G, "SCRIPT")) goto l668;
  }
  l669:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockType", G->buf+G->pos));
  return 1;
  l668:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockType", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockSelfClosing(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockSelfClosing"));  if (!yymatchChar(G, '<')) goto l739;  if (!yy_Spnl(G)) { goto l739; }  if (!yy_HtmlBlockType(G)) { goto l739; }  if (!yy_Spnl(G)) { goto l739; }
  l740:;	
  {  int yypos741= G->pos, yythunkpos741= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l741; }  goto l740;
  l741:;	  G->pos= yypos741; G->thunkpos= yythunkpos741;
  }  if (!yymatchChar(G, '/')) goto l739;  if (!yy_Spnl(G)) { goto l739; }  if (!yymatchChar(G, '>')) goto l739;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockSelfClosing", G->buf+G->pos));
  return 1;
  l739:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockSelfClosing", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlComment(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "HtmlComment"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l742;  if (!yy_LocMarker(G)) { goto l742; }  yyDo(G, yySet, -1, 0);  if (!yymatchString(G, "<!--")) goto l742;
  l743:;	
  {  int yypos744= G->pos, yythunkpos744= G->thunkpos;
  {  int yypos745= G->pos, yythunkpos745= G->thunkpos;  if (!yymatchString(G, "-->")) goto l745;  goto l744;
  l745:;	  G->pos= yypos745; G->thunkpos= yythunkpos745;
  }  if (!yymatchDot(G)) goto l744;  goto l743;
  l744:;	  G->pos= yypos744; G->thunkpos= yythunkpos744;
  }  if (!yymatchString(G, "-->")) goto l742;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l742;  yyDo(G, yy_1_HtmlComment, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlComment", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l742:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlComment", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockInTags(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockInTags"));
  {  int yypos747= G->pos, yythunkpos747= G->thunkpos;  if (!yy_HtmlBlockAddress(G)) { goto l748; }  goto l747;
  l748:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockBlockquote(G)) { goto l749; }  goto l747;
  l749:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockCenter(G)) { goto l750; }  goto l747;
  l750:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockDir(G)) { goto l751; }  goto l747;
  l751:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockDiv(G)) { goto l752; }  goto l747;
  l752:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockDl(G)) { goto l753; }  goto l747;
  l753:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockFieldset(G)) { goto l754; }  goto l747;
  l754:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockForm(G)) { goto l755; }  goto l747;
  l755:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockH1(G)) { goto l756; }  goto l747;
  l756:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockH2(G)) { goto l757; }  goto l747;
  l757:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockH3(G)) { goto l758; }  goto l747;
  l758:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockH4(G)) { goto l759; }  goto l747;
  l759:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockH5(G)) { goto l760; }  goto l747;
  l760:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockH6(G)) { goto l761; }  goto l747;
  l761:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockMenu(G)) { goto l762; }  goto l747;
  l762:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockNoframes(G)) { goto l763; }  goto l747;
  l763:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockNoscript(G)) { goto l764; }  goto l747;
  l764:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockOl(G)) { goto l765; }  goto l747;
  l765:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockP(G)) { goto l766; }  goto l747;
  l766:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockPre(G)) { goto l767; }  goto l747;
  l767:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockTable(G)) { goto l768; }  goto l747;
  l768:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockUl(G)) { goto l769; }  goto l747;
  l769:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockDd(G)) { goto l770; }  goto l747;
  l770:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockDt(G)) { goto l771; }  goto l747;
  l771:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockFrameset(G)) { goto l772; }  goto l747;
  l772:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockLi(G)) { goto l773; }  goto l747;
  l773:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockTbody(G)) { goto l774; }  goto l747;
  l774:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockTd(G)) { goto l775; }  goto l747;
  l775:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockTfoot(G)) { goto l776; }  goto l747;
  l776:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockTh(G)) { goto l777; }  goto l747;
  l777:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockThead(G)) { goto l778; }  goto l747;
  l778:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockTr(G)) { goto l779; }  goto l747;
  l779:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockScript(G)) { goto l780; }  goto l747;
  l780:;	  G->pos= yypos747; G->thunkpos= yythunkpos747;  if (!yy_HtmlBlockHead(G)) { goto l746; }
  }
  l747:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockInTags", G->buf+G->pos));
  return 1;
  l746:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockInTags", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockHead(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockHead"));  if (!yy_HtmlBlockOpenHead(G)) { goto l781; }
  l782:;	
  {  int yypos783= G->pos, yythunkpos783= G->thunkpos;
  {  int yypos784= G->pos, yythunkpos784= G->thunkpos;  if (!yy_HtmlBlockCloseHead(G)) { goto l784; }  goto l783;
  l784:;	  G->pos= yypos784; G->thunkpos= yythunkpos784;
  }  if (!yymatchDot(G)) goto l783;  goto l782;
  l783:;	  G->pos= yypos783; G->thunkpos= yythunkpos783;
  }  if (!yy_HtmlBlockCloseHead(G)) { goto l781; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockHead", G->buf+G->pos));
  return 1;
  l781:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockHead", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseHead(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseHead"));  if (!yymatchChar(G, '<')) goto l785;  if (!yy_Spnl(G)) { goto l785; }  if (!yymatchChar(G, '/')) goto l785;
  {  int yypos786= G->pos, yythunkpos786= G->thunkpos;  if (!yymatchString(G, "head")) goto l787;  goto l786;
  l787:;	  G->pos= yypos786; G->thunkpos= yythunkpos786;  if (!yymatchString(G, "HEAD")) goto l785;
  }
  l786:;	  if (!yy_Spnl(G)) { goto l785; }  if (!yymatchChar(G, '>')) goto l785;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseHead", G->buf+G->pos));
  return 1;
  l785:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseHead", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenHead(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenHead"));  if (!yymatchChar(G, '<')) goto l788;  if (!yy_Spnl(G)) { goto l788; }
  {  int yypos789= G->pos, yythunkpos789= G->thunkpos;  if (!yymatchString(G, "head")) goto l790;  goto l789;
  l790:;	  G->pos= yypos789; G->thunkpos= yythunkpos789;  if (!yymatchString(G, "HEAD")) goto l788;
  }
  l789:;	  if (!yy_Spnl(G)) { goto l788; }
  l791:;	
  {  int yypos792= G->pos, yythunkpos792= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l792; }  goto l791;
  l792:;	  G->pos= yypos792; G->thunkpos= yythunkpos792;
  }  if (!yymatchChar(G, '>')) goto l788;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenHead", G->buf+G->pos));
  return 1;
  l788:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenHead", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockScript(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockScript"));  if (!yy_HtmlBlockOpenScript(G)) { goto l793; }
  l794:;	
  {  int yypos795= G->pos, yythunkpos795= G->thunkpos;
  {  int yypos796= G->pos, yythunkpos796= G->thunkpos;  if (!yy_HtmlBlockCloseScript(G)) { goto l796; }  goto l795;
  l796:;	  G->pos= yypos796; G->thunkpos= yythunkpos796;
  }  if (!yymatchDot(G)) goto l795;  goto l794;
  l795:;	  G->pos= yypos795; G->thunkpos= yythunkpos795;
  }  if (!yy_HtmlBlockCloseScript(G)) { goto l793; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockScript", G->buf+G->pos));
  return 1;
  l793:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockScript", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseScript(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseScript"));  if (!yymatchChar(G, '<')) goto l797;  if (!yy_Spnl(G)) { goto l797; }  if (!yymatchChar(G, '/')) goto l797;
  {  int yypos798= G->pos, yythunkpos798= G->thunkpos;  if (!yymatchString(G, "script")) goto l799;  goto l798;
  l799:;	  G->pos= yypos798; G->thunkpos= yythunkpos798;  if (!yymatchString(G, "SCRIPT")) goto l797;
  }
  l798:;	  if (!yy_Spnl(G)) { goto l797; }  if (!yymatchChar(G, '>')) goto l797;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseScript", G->buf+G->pos));
  return 1;
  l797:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseScript", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenScript(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenScript"));  if (!yymatchChar(G, '<')) goto l800;  if (!yy_Spnl(G)) { goto l800; }
  {  int yypos801= G->pos, yythunkpos801= G->thunkpos;  if (!yymatchString(G, "script")) goto l802;  goto l801;
  l802:;	  G->pos= yypos801; G->thunkpos= yythunkpos801;  if (!yymatchString(G, "SCRIPT")) goto l800;
  }
  l801:;	  if (!yy_Spnl(G)) { goto l800; }
  l803:;	
  {  int yypos804= G->pos, yythunkpos804= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l804; }  goto l803;
  l804:;	  G->pos= yypos804; G->thunkpos= yythunkpos804;
  }  if (!yymatchChar(G, '>')) goto l800;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenScript", G->buf+G->pos));
  return 1;
  l800:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenScript", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockTr(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockTr"));  if (!yy_HtmlBlockOpenTr(G)) { goto l805; }
  l806:;	
  {  int yypos807= G->pos, yythunkpos807= G->thunkpos;
  {  int yypos808= G->pos, yythunkpos808= G->thunkpos;  if (!yy_HtmlBlockTr(G)) { goto l809; }  goto l808;
  l809:;	  G->pos= yypos808; G->thunkpos= yythunkpos808;
  {  int yypos810= G->pos, yythunkpos810= G->thunkpos;  if (!yy_HtmlBlockCloseTr(G)) { goto l810; }  goto l807;
  l810:;	  G->pos= yypos810; G->thunkpos= yythunkpos810;
  }  if (!yymatchDot(G)) goto l807;
  }
  l808:;	  goto l806;
  l807:;	  G->pos= yypos807; G->thunkpos= yythunkpos807;
  }  if (!yy_HtmlBlockCloseTr(G)) { goto l805; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockTr", G->buf+G->pos));
  return 1;
  l805:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockTr", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseTr(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseTr"));  if (!yymatchChar(G, '<')) goto l811;  if (!yy_Spnl(G)) { goto l811; }  if (!yymatchChar(G, '/')) goto l811;
  {  int yypos812= G->pos, yythunkpos812= G->thunkpos;  if (!yymatchString(G, "tr")) goto l813;  goto l812;
  l813:;	  G->pos= yypos812; G->thunkpos= yythunkpos812;  if (!yymatchString(G, "TR")) goto l811;
  }
  l812:;	  if (!yy_Spnl(G)) { goto l811; }  if (!yymatchChar(G, '>')) goto l811;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseTr", G->buf+G->pos));
  return 1;
  l811:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseTr", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenTr(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenTr"));  if (!yymatchChar(G, '<')) goto l814;  if (!yy_Spnl(G)) { goto l814; }
  {  int yypos815= G->pos, yythunkpos815= G->thunkpos;  if (!yymatchString(G, "tr")) goto l816;  goto l815;
  l816:;	  G->pos= yypos815; G->thunkpos= yythunkpos815;  if (!yymatchString(G, "TR")) goto l814;
  }
  l815:;	  if (!yy_Spnl(G)) { goto l814; }
  l817:;	
  {  int yypos818= G->pos, yythunkpos818= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l818; }  goto l817;
  l818:;	  G->pos= yypos818; G->thunkpos= yythunkpos818;
  }  if (!yymatchChar(G, '>')) goto l814;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenTr", G->buf+G->pos));
  return 1;
  l814:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenTr", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockThead(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockThead"));  if (!yy_HtmlBlockOpenThead(G)) { goto l819; }
  l820:;	
  {  int yypos821= G->pos, yythunkpos821= G->thunkpos;
  {  int yypos822= G->pos, yythunkpos822= G->thunkpos;  if (!yy_HtmlBlockThead(G)) { goto l823; }  goto l822;
  l823:;	  G->pos= yypos822; G->thunkpos= yythunkpos822;
  {  int yypos824= G->pos, yythunkpos824= G->thunkpos;  if (!yy_HtmlBlockCloseThead(G)) { goto l824; }  goto l821;
  l824:;	  G->pos= yypos824; G->thunkpos= yythunkpos824;
  }  if (!yymatchDot(G)) goto l821;
  }
  l822:;	  goto l820;
  l821:;	  G->pos= yypos821; G->thunkpos= yythunkpos821;
  }  if (!yy_HtmlBlockCloseThead(G)) { goto l819; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockThead", G->buf+G->pos));
  return 1;
  l819:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockThead", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseThead(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseThead"));  if (!yymatchChar(G, '<')) goto l825;  if (!yy_Spnl(G)) { goto l825; }  if (!yymatchChar(G, '/')) goto l825;
  {  int yypos826= G->pos, yythunkpos826= G->thunkpos;  if (!yymatchString(G, "thead")) goto l827;  goto l826;
  l827:;	  G->pos= yypos826; G->thunkpos= yythunkpos826;  if (!yymatchString(G, "THEAD")) goto l825;
  }
  l826:;	  if (!yy_Spnl(G)) { goto l825; }  if (!yymatchChar(G, '>')) goto l825;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseThead", G->buf+G->pos));
  return 1;
  l825:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseThead", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenThead(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenThead"));  if (!yymatchChar(G, '<')) goto l828;  if (!yy_Spnl(G)) { goto l828; }
  {  int yypos829= G->pos, yythunkpos829= G->thunkpos;  if (!yymatchString(G, "thead")) goto l830;  goto l829;
  l830:;	  G->pos= yypos829; G->thunkpos= yythunkpos829;  if (!yymatchString(G, "THEAD")) goto l828;
  }
  l829:;	  if (!yy_Spnl(G)) { goto l828; }
  l831:;	
  {  int yypos832= G->pos, yythunkpos832= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l832; }  goto l831;
  l832:;	  G->pos= yypos832; G->thunkpos= yythunkpos832;
  }  if (!yymatchChar(G, '>')) goto l828;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenThead", G->buf+G->pos));
  return 1;
  l828:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenThead", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockTh(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockTh"));  if (!yy_HtmlBlockOpenTh(G)) { goto l833; }
  l834:;	
  {  int yypos835= G->pos, yythunkpos835= G->thunkpos;
  {  int yypos836= G->pos, yythunkpos836= G->thunkpos;  if (!yy_HtmlBlockTh(G)) { goto l837; }  goto l836;
  l837:;	  G->pos= yypos836; G->thunkpos= yythunkpos836;
  {  int yypos838= G->pos, yythunkpos838= G->thunkpos;  if (!yy_HtmlBlockCloseTh(G)) { goto l838; }  goto l835;
  l838:;	  G->pos= yypos838; G->thunkpos= yythunkpos838;
  }  if (!yymatchDot(G)) goto l835;
  }
  l836:;	  goto l834;
  l835:;	  G->pos= yypos835; G->thunkpos= yythunkpos835;
  }  if (!yy_HtmlBlockCloseTh(G)) { goto l833; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockTh", G->buf+G->pos));
  return 1;
  l833:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockTh", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseTh(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseTh"));  if (!yymatchChar(G, '<')) goto l839;  if (!yy_Spnl(G)) { goto l839; }  if (!yymatchChar(G, '/')) goto l839;
  {  int yypos840= G->pos, yythunkpos840= G->thunkpos;  if (!yymatchString(G, "th")) goto l841;  goto l840;
  l841:;	  G->pos= yypos840; G->thunkpos= yythunkpos840;  if (!yymatchString(G, "TH")) goto l839;
  }
  l840:;	  if (!yy_Spnl(G)) { goto l839; }  if (!yymatchChar(G, '>')) goto l839;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseTh", G->buf+G->pos));
  return 1;
  l839:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseTh", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenTh(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenTh"));  if (!yymatchChar(G, '<')) goto l842;  if (!yy_Spnl(G)) { goto l842; }
  {  int yypos843= G->pos, yythunkpos843= G->thunkpos;  if (!yymatchString(G, "th")) goto l844;  goto l843;
  l844:;	  G->pos= yypos843; G->thunkpos= yythunkpos843;  if (!yymatchString(G, "TH")) goto l842;
  }
  l843:;	  if (!yy_Spnl(G)) { goto l842; }
  l845:;	
  {  int yypos846= G->pos, yythunkpos846= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l846; }  goto l845;
  l846:;	  G->pos= yypos846; G->thunkpos= yythunkpos846;
  }  if (!yymatchChar(G, '>')) goto l842;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenTh", G->buf+G->pos));
  return 1;
  l842:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenTh", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockTfoot(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockTfoot"));  if (!yy_HtmlBlockOpenTfoot(G)) { goto l847; }
  l848:;	
  {  int yypos849= G->pos, yythunkpos849= G->thunkpos;
  {  int yypos850= G->pos, yythunkpos850= G->thunkpos;  if (!yy_HtmlBlockTfoot(G)) { goto l851; }  goto l850;
  l851:;	  G->pos= yypos850; G->thunkpos= yythunkpos850;
  {  int yypos852= G->pos, yythunkpos852= G->thunkpos;  if (!yy_HtmlBlockCloseTfoot(G)) { goto l852; }  goto l849;
  l852:;	  G->pos= yypos852; G->thunkpos= yythunkpos852;
  }  if (!yymatchDot(G)) goto l849;
  }
  l850:;	  goto l848;
  l849:;	  G->pos= yypos849; G->thunkpos= yythunkpos849;
  }  if (!yy_HtmlBlockCloseTfoot(G)) { goto l847; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockTfoot", G->buf+G->pos));
  return 1;
  l847:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockTfoot", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseTfoot(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseTfoot"));  if (!yymatchChar(G, '<')) goto l853;  if (!yy_Spnl(G)) { goto l853; }  if (!yymatchChar(G, '/')) goto l853;
  {  int yypos854= G->pos, yythunkpos854= G->thunkpos;  if (!yymatchString(G, "tfoot")) goto l855;  goto l854;
  l855:;	  G->pos= yypos854; G->thunkpos= yythunkpos854;  if (!yymatchString(G, "TFOOT")) goto l853;
  }
  l854:;	  if (!yy_Spnl(G)) { goto l853; }  if (!yymatchChar(G, '>')) goto l853;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseTfoot", G->buf+G->pos));
  return 1;
  l853:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseTfoot", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenTfoot(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenTfoot"));  if (!yymatchChar(G, '<')) goto l856;  if (!yy_Spnl(G)) { goto l856; }
  {  int yypos857= G->pos, yythunkpos857= G->thunkpos;  if (!yymatchString(G, "tfoot")) goto l858;  goto l857;
  l858:;	  G->pos= yypos857; G->thunkpos= yythunkpos857;  if (!yymatchString(G, "TFOOT")) goto l856;
  }
  l857:;	  if (!yy_Spnl(G)) { goto l856; }
  l859:;	
  {  int yypos860= G->pos, yythunkpos860= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l860; }  goto l859;
  l860:;	  G->pos= yypos860; G->thunkpos= yythunkpos860;
  }  if (!yymatchChar(G, '>')) goto l856;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenTfoot", G->buf+G->pos));
  return 1;
  l856:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenTfoot", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockTd(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockTd"));  if (!yy_HtmlBlockOpenTd(G)) { goto l861; }
  l862:;	
  {  int yypos863= G->pos, yythunkpos863= G->thunkpos;
  {  int yypos864= G->pos, yythunkpos864= G->thunkpos;  if (!yy_HtmlBlockTd(G)) { goto l865; }  goto l864;
  l865:;	  G->pos= yypos864; G->thunkpos= yythunkpos864;
  {  int yypos866= G->pos, yythunkpos866= G->thunkpos;  if (!yy_HtmlBlockCloseTd(G)) { goto l866; }  goto l863;
  l866:;	  G->pos= yypos866; G->thunkpos= yythunkpos866;
  }  if (!yymatchDot(G)) goto l863;
  }
  l864:;	  goto l862;
  l863:;	  G->pos= yypos863; G->thunkpos= yythunkpos863;
  }  if (!yy_HtmlBlockCloseTd(G)) { goto l861; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockTd", G->buf+G->pos));
  return 1;
  l861:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockTd", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseTd(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseTd"));  if (!yymatchChar(G, '<')) goto l867;  if (!yy_Spnl(G)) { goto l867; }  if (!yymatchChar(G, '/')) goto l867;
  {  int yypos868= G->pos, yythunkpos868= G->thunkpos;  if (!yymatchString(G, "td")) goto l869;  goto l868;
  l869:;	  G->pos= yypos868; G->thunkpos= yythunkpos868;  if (!yymatchString(G, "TD")) goto l867;
  }
  l868:;	  if (!yy_Spnl(G)) { goto l867; }  if (!yymatchChar(G, '>')) goto l867;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseTd", G->buf+G->pos));
  return 1;
  l867:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseTd", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenTd(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenTd"));  if (!yymatchChar(G, '<')) goto l870;  if (!yy_Spnl(G)) { goto l870; }
  {  int yypos871= G->pos, yythunkpos871= G->thunkpos;  if (!yymatchString(G, "td")) goto l872;  goto l871;
  l872:;	  G->pos= yypos871; G->thunkpos= yythunkpos871;  if (!yymatchString(G, "TD")) goto l870;
  }
  l871:;	  if (!yy_Spnl(G)) { goto l870; }
  l873:;	
  {  int yypos874= G->pos, yythunkpos874= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l874; }  goto l873;
  l874:;	  G->pos= yypos874; G->thunkpos= yythunkpos874;
  }  if (!yymatchChar(G, '>')) goto l870;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenTd", G->buf+G->pos));
  return 1;
  l870:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenTd", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockTbody(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockTbody"));  if (!yy_HtmlBlockOpenTbody(G)) { goto l875; }
  l876:;	
  {  int yypos877= G->pos, yythunkpos877= G->thunkpos;
  {  int yypos878= G->pos, yythunkpos878= G->thunkpos;  if (!yy_HtmlBlockTbody(G)) { goto l879; }  goto l878;
  l879:;	  G->pos= yypos878; G->thunkpos= yythunkpos878;
  {  int yypos880= G->pos, yythunkpos880= G->thunkpos;  if (!yy_HtmlBlockCloseTbody(G)) { goto l880; }  goto l877;
  l880:;	  G->pos= yypos880; G->thunkpos= yythunkpos880;
  }  if (!yymatchDot(G)) goto l877;
  }
  l878:;	  goto l876;
  l877:;	  G->pos= yypos877; G->thunkpos= yythunkpos877;
  }  if (!yy_HtmlBlockCloseTbody(G)) { goto l875; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockTbody", G->buf+G->pos));
  return 1;
  l875:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockTbody", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseTbody(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseTbody"));  if (!yymatchChar(G, '<')) goto l881;  if (!yy_Spnl(G)) { goto l881; }  if (!yymatchChar(G, '/')) goto l881;
  {  int yypos882= G->pos, yythunkpos882= G->thunkpos;  if (!yymatchString(G, "tbody")) goto l883;  goto l882;
  l883:;	  G->pos= yypos882; G->thunkpos= yythunkpos882;  if (!yymatchString(G, "TBODY")) goto l881;
  }
  l882:;	  if (!yy_Spnl(G)) { goto l881; }  if (!yymatchChar(G, '>')) goto l881;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseTbody", G->buf+G->pos));
  return 1;
  l881:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseTbody", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenTbody(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenTbody"));  if (!yymatchChar(G, '<')) goto l884;  if (!yy_Spnl(G)) { goto l884; }
  {  int yypos885= G->pos, yythunkpos885= G->thunkpos;  if (!yymatchString(G, "tbody")) goto l886;  goto l885;
  l886:;	  G->pos= yypos885; G->thunkpos= yythunkpos885;  if (!yymatchString(G, "TBODY")) goto l884;
  }
  l885:;	  if (!yy_Spnl(G)) { goto l884; }
  l887:;	
  {  int yypos888= G->pos, yythunkpos888= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l888; }  goto l887;
  l888:;	  G->pos= yypos888; G->thunkpos= yythunkpos888;
  }  if (!yymatchChar(G, '>')) goto l884;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenTbody", G->buf+G->pos));
  return 1;
  l884:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenTbody", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockLi(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockLi"));  if (!yy_HtmlBlockOpenLi(G)) { goto l889; }
  l890:;	
  {  int yypos891= G->pos, yythunkpos891= G->thunkpos;
  {  int yypos892= G->pos, yythunkpos892= G->thunkpos;  if (!yy_HtmlBlockLi(G)) { goto l893; }  goto l892;
  l893:;	  G->pos= yypos892; G->thunkpos= yythunkpos892;
  {  int yypos894= G->pos, yythunkpos894= G->thunkpos;  if (!yy_HtmlBlockCloseLi(G)) { goto l894; }  goto l891;
  l894:;	  G->pos= yypos894; G->thunkpos= yythunkpos894;
  }  if (!yymatchDot(G)) goto l891;
  }
  l892:;	  goto l890;
  l891:;	  G->pos= yypos891; G->thunkpos= yythunkpos891;
  }  if (!yy_HtmlBlockCloseLi(G)) { goto l889; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockLi", G->buf+G->pos));
  return 1;
  l889:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockLi", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseLi(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseLi"));  if (!yymatchChar(G, '<')) goto l895;  if (!yy_Spnl(G)) { goto l895; }  if (!yymatchChar(G, '/')) goto l895;
  {  int yypos896= G->pos, yythunkpos896= G->thunkpos;  if (!yymatchString(G, "li")) goto l897;  goto l896;
  l897:;	  G->pos= yypos896; G->thunkpos= yythunkpos896;  if (!yymatchString(G, "LI")) goto l895;
  }
  l896:;	  if (!yy_Spnl(G)) { goto l895; }  if (!yymatchChar(G, '>')) goto l895;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseLi", G->buf+G->pos));
  return 1;
  l895:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseLi", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenLi(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenLi"));  if (!yymatchChar(G, '<')) goto l898;  if (!yy_Spnl(G)) { goto l898; }
  {  int yypos899= G->pos, yythunkpos899= G->thunkpos;  if (!yymatchString(G, "li")) goto l900;  goto l899;
  l900:;	  G->pos= yypos899; G->thunkpos= yythunkpos899;  if (!yymatchString(G, "LI")) goto l898;
  }
  l899:;	  if (!yy_Spnl(G)) { goto l898; }
  l901:;	
  {  int yypos902= G->pos, yythunkpos902= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l902; }  goto l901;
  l902:;	  G->pos= yypos902; G->thunkpos= yythunkpos902;
  }  if (!yymatchChar(G, '>')) goto l898;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenLi", G->buf+G->pos));
  return 1;
  l898:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenLi", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockFrameset(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockFrameset"));  if (!yy_HtmlBlockOpenFrameset(G)) { goto l903; }
  l904:;	
  {  int yypos905= G->pos, yythunkpos905= G->thunkpos;
  {  int yypos906= G->pos, yythunkpos906= G->thunkpos;  if (!yy_HtmlBlockFrameset(G)) { goto l907; }  goto l906;
  l907:;	  G->pos= yypos906; G->thunkpos= yythunkpos906;
  {  int yypos908= G->pos, yythunkpos908= G->thunkpos;  if (!yy_HtmlBlockCloseFrameset(G)) { goto l908; }  goto l905;
  l908:;	  G->pos= yypos908; G->thunkpos= yythunkpos908;
  }  if (!yymatchDot(G)) goto l905;
  }
  l906:;	  goto l904;
  l905:;	  G->pos= yypos905; G->thunkpos= yythunkpos905;
  }  if (!yy_HtmlBlockCloseFrameset(G)) { goto l903; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockFrameset", G->buf+G->pos));
  return 1;
  l903:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockFrameset", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseFrameset(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseFrameset"));  if (!yymatchChar(G, '<')) goto l909;  if (!yy_Spnl(G)) { goto l909; }  if (!yymatchChar(G, '/')) goto l909;
  {  int yypos910= G->pos, yythunkpos910= G->thunkpos;  if (!yymatchString(G, "frameset")) goto l911;  goto l910;
  l911:;	  G->pos= yypos910; G->thunkpos= yythunkpos910;  if (!yymatchString(G, "FRAMESET")) goto l909;
  }
  l910:;	  if (!yy_Spnl(G)) { goto l909; }  if (!yymatchChar(G, '>')) goto l909;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseFrameset", G->buf+G->pos));
  return 1;
  l909:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseFrameset", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenFrameset(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenFrameset"));  if (!yymatchChar(G, '<')) goto l912;  if (!yy_Spnl(G)) { goto l912; }
  {  int yypos913= G->pos, yythunkpos913= G->thunkpos;  if (!yymatchString(G, "frameset")) goto l914;  goto l913;
  l914:;	  G->pos= yypos913; G->thunkpos= yythunkpos913;  if (!yymatchString(G, "FRAMESET")) goto l912;
  }
  l913:;	  if (!yy_Spnl(G)) { goto l912; }
  l915:;	
  {  int yypos916= G->pos, yythunkpos916= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l916; }  goto l915;
  l916:;	  G->pos= yypos916; G->thunkpos= yythunkpos916;
  }  if (!yymatchChar(G, '>')) goto l912;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenFrameset", G->buf+G->pos));
  return 1;
  l912:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenFrameset", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockDt(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockDt"));  if (!yy_HtmlBlockOpenDt(G)) { goto l917; }
  l918:;	
  {  int yypos919= G->pos, yythunkpos919= G->thunkpos;
  {  int yypos920= G->pos, yythunkpos920= G->thunkpos;  if (!yy_HtmlBlockDt(G)) { goto l921; }  goto l920;
  l921:;	  G->pos= yypos920; G->thunkpos= yythunkpos920;
  {  int yypos922= G->pos, yythunkpos922= G->thunkpos;  if (!yy_HtmlBlockCloseDt(G)) { goto l922; }  goto l919;
  l922:;	  G->pos= yypos922; G->thunkpos= yythunkpos922;
  }  if (!yymatchDot(G)) goto l919;
  }
  l920:;	  goto l918;
  l919:;	  G->pos= yypos919; G->thunkpos= yythunkpos919;
  }  if (!yy_HtmlBlockCloseDt(G)) { goto l917; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockDt", G->buf+G->pos));
  return 1;
  l917:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockDt", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseDt(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseDt"));  if (!yymatchChar(G, '<')) goto l923;  if (!yy_Spnl(G)) { goto l923; }  if (!yymatchChar(G, '/')) goto l923;
  {  int yypos924= G->pos, yythunkpos924= G->thunkpos;  if (!yymatchString(G, "dt")) goto l925;  goto l924;
  l925:;	  G->pos= yypos924; G->thunkpos= yythunkpos924;  if (!yymatchString(G, "DT")) goto l923;
  }
  l924:;	  if (!yy_Spnl(G)) { goto l923; }  if (!yymatchChar(G, '>')) goto l923;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseDt", G->buf+G->pos));
  return 1;
  l923:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseDt", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenDt(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenDt"));  if (!yymatchChar(G, '<')) goto l926;  if (!yy_Spnl(G)) { goto l926; }
  {  int yypos927= G->pos, yythunkpos927= G->thunkpos;  if (!yymatchString(G, "dt")) goto l928;  goto l927;
  l928:;	  G->pos= yypos927; G->thunkpos= yythunkpos927;  if (!yymatchString(G, "DT")) goto l926;
  }
  l927:;	  if (!yy_Spnl(G)) { goto l926; }
  l929:;	
  {  int yypos930= G->pos, yythunkpos930= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l930; }  goto l929;
  l930:;	  G->pos= yypos930; G->thunkpos= yythunkpos930;
  }  if (!yymatchChar(G, '>')) goto l926;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenDt", G->buf+G->pos));
  return 1;
  l926:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenDt", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockDd(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockDd"));  if (!yy_HtmlBlockOpenDd(G)) { goto l931; }
  l932:;	
  {  int yypos933= G->pos, yythunkpos933= G->thunkpos;
  {  int yypos934= G->pos, yythunkpos934= G->thunkpos;  if (!yy_HtmlBlockDd(G)) { goto l935; }  goto l934;
  l935:;	  G->pos= yypos934; G->thunkpos= yythunkpos934;
  {  int yypos936= G->pos, yythunkpos936= G->thunkpos;  if (!yy_HtmlBlockCloseDd(G)) { goto l936; }  goto l933;
  l936:;	  G->pos= yypos936; G->thunkpos= yythunkpos936;
  }  if (!yymatchDot(G)) goto l933;
  }
  l934:;	  goto l932;
  l933:;	  G->pos= yypos933; G->thunkpos= yythunkpos933;
  }  if (!yy_HtmlBlockCloseDd(G)) { goto l931; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockDd", G->buf+G->pos));
  return 1;
  l931:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockDd", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseDd(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseDd"));  if (!yymatchChar(G, '<')) goto l937;  if (!yy_Spnl(G)) { goto l937; }  if (!yymatchChar(G, '/')) goto l937;
  {  int yypos938= G->pos, yythunkpos938= G->thunkpos;  if (!yymatchString(G, "dd")) goto l939;  goto l938;
  l939:;	  G->pos= yypos938; G->thunkpos= yythunkpos938;  if (!yymatchString(G, "DD")) goto l937;
  }
  l938:;	  if (!yy_Spnl(G)) { goto l937; }  if (!yymatchChar(G, '>')) goto l937;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseDd", G->buf+G->pos));
  return 1;
  l937:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseDd", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenDd(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenDd"));  if (!yymatchChar(G, '<')) goto l940;  if (!yy_Spnl(G)) { goto l940; }
  {  int yypos941= G->pos, yythunkpos941= G->thunkpos;  if (!yymatchString(G, "dd")) goto l942;  goto l941;
  l942:;	  G->pos= yypos941; G->thunkpos= yythunkpos941;  if (!yymatchString(G, "DD")) goto l940;
  }
  l941:;	  if (!yy_Spnl(G)) { goto l940; }
  l943:;	
  {  int yypos944= G->pos, yythunkpos944= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l944; }  goto l943;
  l944:;	  G->pos= yypos944; G->thunkpos= yythunkpos944;
  }  if (!yymatchChar(G, '>')) goto l940;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenDd", G->buf+G->pos));
  return 1;
  l940:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenDd", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockUl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockUl"));  if (!yy_HtmlBlockOpenUl(G)) { goto l945; }
  l946:;	
  {  int yypos947= G->pos, yythunkpos947= G->thunkpos;
  {  int yypos948= G->pos, yythunkpos948= G->thunkpos;  if (!yy_HtmlBlockUl(G)) { goto l949; }  goto l948;
  l949:;	  G->pos= yypos948; G->thunkpos= yythunkpos948;
  {  int yypos950= G->pos, yythunkpos950= G->thunkpos;  if (!yy_HtmlBlockCloseUl(G)) { goto l950; }  goto l947;
  l950:;	  G->pos= yypos950; G->thunkpos= yythunkpos950;
  }  if (!yymatchDot(G)) goto l947;
  }
  l948:;	  goto l946;
  l947:;	  G->pos= yypos947; G->thunkpos= yythunkpos947;
  }  if (!yy_HtmlBlockCloseUl(G)) { goto l945; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockUl", G->buf+G->pos));
  return 1;
  l945:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockUl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseUl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseUl"));  if (!yymatchChar(G, '<')) goto l951;  if (!yy_Spnl(G)) { goto l951; }  if (!yymatchChar(G, '/')) goto l951;
  {  int yypos952= G->pos, yythunkpos952= G->thunkpos;  if (!yymatchString(G, "ul")) goto l953;  goto l952;
  l953:;	  G->pos= yypos952; G->thunkpos= yythunkpos952;  if (!yymatchString(G, "UL")) goto l951;
  }
  l952:;	  if (!yy_Spnl(G)) { goto l951; }  if (!yymatchChar(G, '>')) goto l951;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseUl", G->buf+G->pos));
  return 1;
  l951:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseUl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenUl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenUl"));  if (!yymatchChar(G, '<')) goto l954;  if (!yy_Spnl(G)) { goto l954; }
  {  int yypos955= G->pos, yythunkpos955= G->thunkpos;  if (!yymatchString(G, "ul")) goto l956;  goto l955;
  l956:;	  G->pos= yypos955; G->thunkpos= yythunkpos955;  if (!yymatchString(G, "UL")) goto l954;
  }
  l955:;	  if (!yy_Spnl(G)) { goto l954; }
  l957:;	
  {  int yypos958= G->pos, yythunkpos958= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l958; }  goto l957;
  l958:;	  G->pos= yypos958; G->thunkpos= yythunkpos958;
  }  if (!yymatchChar(G, '>')) goto l954;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenUl", G->buf+G->pos));
  return 1;
  l954:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenUl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockTable(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockTable"));  if (!yy_HtmlBlockOpenTable(G)) { goto l959; }
  l960:;	
  {  int yypos961= G->pos, yythunkpos961= G->thunkpos;
  {  int yypos962= G->pos, yythunkpos962= G->thunkpos;  if (!yy_HtmlBlockTable(G)) { goto l963; }  goto l962;
  l963:;	  G->pos= yypos962; G->thunkpos= yythunkpos962;
  {  int yypos964= G->pos, yythunkpos964= G->thunkpos;  if (!yy_HtmlBlockCloseTable(G)) { goto l964; }  goto l961;
  l964:;	  G->pos= yypos964; G->thunkpos= yythunkpos964;
  }  if (!yymatchDot(G)) goto l961;
  }
  l962:;	  goto l960;
  l961:;	  G->pos= yypos961; G->thunkpos= yythunkpos961;
  }  if (!yy_HtmlBlockCloseTable(G)) { goto l959; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockTable", G->buf+G->pos));
  return 1;
  l959:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockTable", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseTable(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseTable"));  if (!yymatchChar(G, '<')) goto l965;  if (!yy_Spnl(G)) { goto l965; }  if (!yymatchChar(G, '/')) goto l965;
  {  int yypos966= G->pos, yythunkpos966= G->thunkpos;  if (!yymatchString(G, "table")) goto l967;  goto l966;
  l967:;	  G->pos= yypos966; G->thunkpos= yythunkpos966;  if (!yymatchString(G, "TABLE")) goto l965;
  }
  l966:;	  if (!yy_Spnl(G)) { goto l965; }  if (!yymatchChar(G, '>')) goto l965;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseTable", G->buf+G->pos));
  return 1;
  l965:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseTable", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenTable(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenTable"));  if (!yymatchChar(G, '<')) goto l968;  if (!yy_Spnl(G)) { goto l968; }
  {  int yypos969= G->pos, yythunkpos969= G->thunkpos;  if (!yymatchString(G, "table")) goto l970;  goto l969;
  l970:;	  G->pos= yypos969; G->thunkpos= yythunkpos969;  if (!yymatchString(G, "TABLE")) goto l968;
  }
  l969:;	  if (!yy_Spnl(G)) { goto l968; }
  l971:;	
  {  int yypos972= G->pos, yythunkpos972= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l972; }  goto l971;
  l972:;	  G->pos= yypos972; G->thunkpos= yythunkpos972;
  }  if (!yymatchChar(G, '>')) goto l968;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenTable", G->buf+G->pos));
  return 1;
  l968:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenTable", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockPre(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockPre"));  if (!yy_HtmlBlockOpenPre(G)) { goto l973; }
  l974:;	
  {  int yypos975= G->pos, yythunkpos975= G->thunkpos;
  {  int yypos976= G->pos, yythunkpos976= G->thunkpos;  if (!yy_HtmlBlockPre(G)) { goto l977; }  goto l976;
  l977:;	  G->pos= yypos976; G->thunkpos= yythunkpos976;
  {  int yypos978= G->pos, yythunkpos978= G->thunkpos;  if (!yy_HtmlBlockClosePre(G)) { goto l978; }  goto l975;
  l978:;	  G->pos= yypos978; G->thunkpos= yythunkpos978;
  }  if (!yymatchDot(G)) goto l975;
  }
  l976:;	  goto l974;
  l975:;	  G->pos= yypos975; G->thunkpos= yythunkpos975;
  }  if (!yy_HtmlBlockClosePre(G)) { goto l973; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockPre", G->buf+G->pos));
  return 1;
  l973:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockPre", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockClosePre(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockClosePre"));  if (!yymatchChar(G, '<')) goto l979;  if (!yy_Spnl(G)) { goto l979; }  if (!yymatchChar(G, '/')) goto l979;
  {  int yypos980= G->pos, yythunkpos980= G->thunkpos;  if (!yymatchString(G, "pre")) goto l981;  goto l980;
  l981:;	  G->pos= yypos980; G->thunkpos= yythunkpos980;  if (!yymatchString(G, "PRE")) goto l979;
  }
  l980:;	  if (!yy_Spnl(G)) { goto l979; }  if (!yymatchChar(G, '>')) goto l979;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockClosePre", G->buf+G->pos));
  return 1;
  l979:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockClosePre", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenPre(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenPre"));  if (!yymatchChar(G, '<')) goto l982;  if (!yy_Spnl(G)) { goto l982; }
  {  int yypos983= G->pos, yythunkpos983= G->thunkpos;  if (!yymatchString(G, "pre")) goto l984;  goto l983;
  l984:;	  G->pos= yypos983; G->thunkpos= yythunkpos983;  if (!yymatchString(G, "PRE")) goto l982;
  }
  l983:;	  if (!yy_Spnl(G)) { goto l982; }
  l985:;	
  {  int yypos986= G->pos, yythunkpos986= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l986; }  goto l985;
  l986:;	  G->pos= yypos986; G->thunkpos= yythunkpos986;
  }  if (!yymatchChar(G, '>')) goto l982;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenPre", G->buf+G->pos));
  return 1;
  l982:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenPre", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockP(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockP"));  if (!yy_HtmlBlockOpenP(G)) { goto l987; }
  l988:;	
  {  int yypos989= G->pos, yythunkpos989= G->thunkpos;
  {  int yypos990= G->pos, yythunkpos990= G->thunkpos;  if (!yy_HtmlBlockP(G)) { goto l991; }  goto l990;
  l991:;	  G->pos= yypos990; G->thunkpos= yythunkpos990;
  {  int yypos992= G->pos, yythunkpos992= G->thunkpos;  if (!yy_HtmlBlockCloseP(G)) { goto l992; }  goto l989;
  l992:;	  G->pos= yypos992; G->thunkpos= yythunkpos992;
  }  if (!yymatchDot(G)) goto l989;
  }
  l990:;	  goto l988;
  l989:;	  G->pos= yypos989; G->thunkpos= yythunkpos989;
  }  if (!yy_HtmlBlockCloseP(G)) { goto l987; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockP", G->buf+G->pos));
  return 1;
  l987:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockP", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseP(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseP"));  if (!yymatchChar(G, '<')) goto l993;  if (!yy_Spnl(G)) { goto l993; }  if (!yymatchChar(G, '/')) goto l993;
  {  int yypos994= G->pos, yythunkpos994= G->thunkpos;  if (!yymatchChar(G, 'p')) goto l995;  goto l994;
  l995:;	  G->pos= yypos994; G->thunkpos= yythunkpos994;  if (!yymatchChar(G, 'P')) goto l993;
  }
  l994:;	  if (!yy_Spnl(G)) { goto l993; }  if (!yymatchChar(G, '>')) goto l993;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseP", G->buf+G->pos));
  return 1;
  l993:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseP", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenP(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenP"));  if (!yymatchChar(G, '<')) goto l996;  if (!yy_Spnl(G)) { goto l996; }
  {  int yypos997= G->pos, yythunkpos997= G->thunkpos;  if (!yymatchChar(G, 'p')) goto l998;  goto l997;
  l998:;	  G->pos= yypos997; G->thunkpos= yythunkpos997;  if (!yymatchChar(G, 'P')) goto l996;
  }
  l997:;	  if (!yy_Spnl(G)) { goto l996; }
  l999:;	
  {  int yypos1000= G->pos, yythunkpos1000= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1000; }  goto l999;
  l1000:;	  G->pos= yypos1000; G->thunkpos= yythunkpos1000;
  }  if (!yymatchChar(G, '>')) goto l996;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenP", G->buf+G->pos));
  return 1;
  l996:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenP", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOl"));  if (!yy_HtmlBlockOpenOl(G)) { goto l1001; }
  l1002:;	
  {  int yypos1003= G->pos, yythunkpos1003= G->thunkpos;
  {  int yypos1004= G->pos, yythunkpos1004= G->thunkpos;  if (!yy_HtmlBlockOl(G)) { goto l1005; }  goto l1004;
  l1005:;	  G->pos= yypos1004; G->thunkpos= yythunkpos1004;
  {  int yypos1006= G->pos, yythunkpos1006= G->thunkpos;  if (!yy_HtmlBlockCloseOl(G)) { goto l1006; }  goto l1003;
  l1006:;	  G->pos= yypos1006; G->thunkpos= yythunkpos1006;
  }  if (!yymatchDot(G)) goto l1003;
  }
  l1004:;	  goto l1002;
  l1003:;	  G->pos= yypos1003; G->thunkpos= yythunkpos1003;
  }  if (!yy_HtmlBlockCloseOl(G)) { goto l1001; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOl", G->buf+G->pos));
  return 1;
  l1001:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseOl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseOl"));  if (!yymatchChar(G, '<')) goto l1007;  if (!yy_Spnl(G)) { goto l1007; }  if (!yymatchChar(G, '/')) goto l1007;
  {  int yypos1008= G->pos, yythunkpos1008= G->thunkpos;  if (!yymatchString(G, "ol")) goto l1009;  goto l1008;
  l1009:;	  G->pos= yypos1008; G->thunkpos= yythunkpos1008;  if (!yymatchString(G, "OL")) goto l1007;
  }
  l1008:;	  if (!yy_Spnl(G)) { goto l1007; }  if (!yymatchChar(G, '>')) goto l1007;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseOl", G->buf+G->pos));
  return 1;
  l1007:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseOl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenOl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenOl"));  if (!yymatchChar(G, '<')) goto l1010;  if (!yy_Spnl(G)) { goto l1010; }
  {  int yypos1011= G->pos, yythunkpos1011= G->thunkpos;  if (!yymatchString(G, "ol")) goto l1012;  goto l1011;
  l1012:;	  G->pos= yypos1011; G->thunkpos= yythunkpos1011;  if (!yymatchString(G, "OL")) goto l1010;
  }
  l1011:;	  if (!yy_Spnl(G)) { goto l1010; }
  l1013:;	
  {  int yypos1014= G->pos, yythunkpos1014= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1014; }  goto l1013;
  l1014:;	  G->pos= yypos1014; G->thunkpos= yythunkpos1014;
  }  if (!yymatchChar(G, '>')) goto l1010;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenOl", G->buf+G->pos));
  return 1;
  l1010:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenOl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockNoscript(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockNoscript"));  if (!yy_HtmlBlockOpenNoscript(G)) { goto l1015; }
  l1016:;	
  {  int yypos1017= G->pos, yythunkpos1017= G->thunkpos;
  {  int yypos1018= G->pos, yythunkpos1018= G->thunkpos;  if (!yy_HtmlBlockNoscript(G)) { goto l1019; }  goto l1018;
  l1019:;	  G->pos= yypos1018; G->thunkpos= yythunkpos1018;
  {  int yypos1020= G->pos, yythunkpos1020= G->thunkpos;  if (!yy_HtmlBlockCloseNoscript(G)) { goto l1020; }  goto l1017;
  l1020:;	  G->pos= yypos1020; G->thunkpos= yythunkpos1020;
  }  if (!yymatchDot(G)) goto l1017;
  }
  l1018:;	  goto l1016;
  l1017:;	  G->pos= yypos1017; G->thunkpos= yythunkpos1017;
  }  if (!yy_HtmlBlockCloseNoscript(G)) { goto l1015; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockNoscript", G->buf+G->pos));
  return 1;
  l1015:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockNoscript", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseNoscript(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseNoscript"));  if (!yymatchChar(G, '<')) goto l1021;  if (!yy_Spnl(G)) { goto l1021; }  if (!yymatchChar(G, '/')) goto l1021;
  {  int yypos1022= G->pos, yythunkpos1022= G->thunkpos;  if (!yymatchString(G, "noscript")) goto l1023;  goto l1022;
  l1023:;	  G->pos= yypos1022; G->thunkpos= yythunkpos1022;  if (!yymatchString(G, "NOSCRIPT")) goto l1021;
  }
  l1022:;	  if (!yy_Spnl(G)) { goto l1021; }  if (!yymatchChar(G, '>')) goto l1021;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseNoscript", G->buf+G->pos));
  return 1;
  l1021:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseNoscript", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenNoscript(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenNoscript"));  if (!yymatchChar(G, '<')) goto l1024;  if (!yy_Spnl(G)) { goto l1024; }
  {  int yypos1025= G->pos, yythunkpos1025= G->thunkpos;  if (!yymatchString(G, "noscript")) goto l1026;  goto l1025;
  l1026:;	  G->pos= yypos1025; G->thunkpos= yythunkpos1025;  if (!yymatchString(G, "NOSCRIPT")) goto l1024;
  }
  l1025:;	  if (!yy_Spnl(G)) { goto l1024; }
  l1027:;	
  {  int yypos1028= G->pos, yythunkpos1028= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1028; }  goto l1027;
  l1028:;	  G->pos= yypos1028; G->thunkpos= yythunkpos1028;
  }  if (!yymatchChar(G, '>')) goto l1024;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenNoscript", G->buf+G->pos));
  return 1;
  l1024:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenNoscript", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockNoframes(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockNoframes"));  if (!yy_HtmlBlockOpenNoframes(G)) { goto l1029; }
  l1030:;	
  {  int yypos1031= G->pos, yythunkpos1031= G->thunkpos;
  {  int yypos1032= G->pos, yythunkpos1032= G->thunkpos;  if (!yy_HtmlBlockNoframes(G)) { goto l1033; }  goto l1032;
  l1033:;	  G->pos= yypos1032; G->thunkpos= yythunkpos1032;
  {  int yypos1034= G->pos, yythunkpos1034= G->thunkpos;  if (!yy_HtmlBlockCloseNoframes(G)) { goto l1034; }  goto l1031;
  l1034:;	  G->pos= yypos1034; G->thunkpos= yythunkpos1034;
  }  if (!yymatchDot(G)) goto l1031;
  }
  l1032:;	  goto l1030;
  l1031:;	  G->pos= yypos1031; G->thunkpos= yythunkpos1031;
  }  if (!yy_HtmlBlockCloseNoframes(G)) { goto l1029; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockNoframes", G->buf+G->pos));
  return 1;
  l1029:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockNoframes", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseNoframes(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseNoframes"));  if (!yymatchChar(G, '<')) goto l1035;  if (!yy_Spnl(G)) { goto l1035; }  if (!yymatchChar(G, '/')) goto l1035;
  {  int yypos1036= G->pos, yythunkpos1036= G->thunkpos;  if (!yymatchString(G, "noframes")) goto l1037;  goto l1036;
  l1037:;	  G->pos= yypos1036; G->thunkpos= yythunkpos1036;  if (!yymatchString(G, "NOFRAMES")) goto l1035;
  }
  l1036:;	  if (!yy_Spnl(G)) { goto l1035; }  if (!yymatchChar(G, '>')) goto l1035;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseNoframes", G->buf+G->pos));
  return 1;
  l1035:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseNoframes", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenNoframes(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenNoframes"));  if (!yymatchChar(G, '<')) goto l1038;  if (!yy_Spnl(G)) { goto l1038; }
  {  int yypos1039= G->pos, yythunkpos1039= G->thunkpos;  if (!yymatchString(G, "noframes")) goto l1040;  goto l1039;
  l1040:;	  G->pos= yypos1039; G->thunkpos= yythunkpos1039;  if (!yymatchString(G, "NOFRAMES")) goto l1038;
  }
  l1039:;	  if (!yy_Spnl(G)) { goto l1038; }
  l1041:;	
  {  int yypos1042= G->pos, yythunkpos1042= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1042; }  goto l1041;
  l1042:;	  G->pos= yypos1042; G->thunkpos= yythunkpos1042;
  }  if (!yymatchChar(G, '>')) goto l1038;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenNoframes", G->buf+G->pos));
  return 1;
  l1038:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenNoframes", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockMenu(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockMenu"));  if (!yy_HtmlBlockOpenMenu(G)) { goto l1043; }
  l1044:;	
  {  int yypos1045= G->pos, yythunkpos1045= G->thunkpos;
  {  int yypos1046= G->pos, yythunkpos1046= G->thunkpos;  if (!yy_HtmlBlockMenu(G)) { goto l1047; }  goto l1046;
  l1047:;	  G->pos= yypos1046; G->thunkpos= yythunkpos1046;
  {  int yypos1048= G->pos, yythunkpos1048= G->thunkpos;  if (!yy_HtmlBlockCloseMenu(G)) { goto l1048; }  goto l1045;
  l1048:;	  G->pos= yypos1048; G->thunkpos= yythunkpos1048;
  }  if (!yymatchDot(G)) goto l1045;
  }
  l1046:;	  goto l1044;
  l1045:;	  G->pos= yypos1045; G->thunkpos= yythunkpos1045;
  }  if (!yy_HtmlBlockCloseMenu(G)) { goto l1043; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockMenu", G->buf+G->pos));
  return 1;
  l1043:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockMenu", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseMenu(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseMenu"));  if (!yymatchChar(G, '<')) goto l1049;  if (!yy_Spnl(G)) { goto l1049; }  if (!yymatchChar(G, '/')) goto l1049;
  {  int yypos1050= G->pos, yythunkpos1050= G->thunkpos;  if (!yymatchString(G, "menu")) goto l1051;  goto l1050;
  l1051:;	  G->pos= yypos1050; G->thunkpos= yythunkpos1050;  if (!yymatchString(G, "MENU")) goto l1049;
  }
  l1050:;	  if (!yy_Spnl(G)) { goto l1049; }  if (!yymatchChar(G, '>')) goto l1049;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseMenu", G->buf+G->pos));
  return 1;
  l1049:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseMenu", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenMenu(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenMenu"));  if (!yymatchChar(G, '<')) goto l1052;  if (!yy_Spnl(G)) { goto l1052; }
  {  int yypos1053= G->pos, yythunkpos1053= G->thunkpos;  if (!yymatchString(G, "menu")) goto l1054;  goto l1053;
  l1054:;	  G->pos= yypos1053; G->thunkpos= yythunkpos1053;  if (!yymatchString(G, "MENU")) goto l1052;
  }
  l1053:;	  if (!yy_Spnl(G)) { goto l1052; }
  l1055:;	
  {  int yypos1056= G->pos, yythunkpos1056= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1056; }  goto l1055;
  l1056:;	  G->pos= yypos1056; G->thunkpos= yythunkpos1056;
  }  if (!yymatchChar(G, '>')) goto l1052;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenMenu", G->buf+G->pos));
  return 1;
  l1052:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenMenu", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockH6(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "HtmlBlockH6"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1057;  if (!yy_LocMarker(G)) { goto l1057; }  yyDo(G, yySet, -1, 0);  if (!yy_HtmlBlockOpenH6(G)) { goto l1057; }
  l1058:;	
  {  int yypos1059= G->pos, yythunkpos1059= G->thunkpos;
  {  int yypos1060= G->pos, yythunkpos1060= G->thunkpos;  if (!yy_HtmlBlockH6(G)) { goto l1061; }  goto l1060;
  l1061:;	  G->pos= yypos1060; G->thunkpos= yythunkpos1060;
  {  int yypos1062= G->pos, yythunkpos1062= G->thunkpos;  if (!yy_HtmlBlockCloseH6(G)) { goto l1062; }  goto l1059;
  l1062:;	  G->pos= yypos1062; G->thunkpos= yythunkpos1062;
  }  if (!yymatchDot(G)) goto l1059;
  }
  l1060:;	  goto l1058;
  l1059:;	  G->pos= yypos1059; G->thunkpos= yythunkpos1059;
  }  if (!yy_HtmlBlockCloseH6(G)) { goto l1057; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1057;  yyDo(G, yy_1_HtmlBlockH6, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockH6", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1057:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockH6", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseH6(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseH6"));  if (!yymatchChar(G, '<')) goto l1063;  if (!yy_Spnl(G)) { goto l1063; }  if (!yymatchChar(G, '/')) goto l1063;
  {  int yypos1064= G->pos, yythunkpos1064= G->thunkpos;  if (!yymatchString(G, "h6")) goto l1065;  goto l1064;
  l1065:;	  G->pos= yypos1064; G->thunkpos= yythunkpos1064;  if (!yymatchString(G, "H6")) goto l1063;
  }
  l1064:;	  if (!yy_Spnl(G)) { goto l1063; }  if (!yymatchChar(G, '>')) goto l1063;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseH6", G->buf+G->pos));
  return 1;
  l1063:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseH6", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenH6(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenH6"));  if (!yymatchChar(G, '<')) goto l1066;  if (!yy_Spnl(G)) { goto l1066; }
  {  int yypos1067= G->pos, yythunkpos1067= G->thunkpos;  if (!yymatchString(G, "h6")) goto l1068;  goto l1067;
  l1068:;	  G->pos= yypos1067; G->thunkpos= yythunkpos1067;  if (!yymatchString(G, "H6")) goto l1066;
  }
  l1067:;	  if (!yy_Spnl(G)) { goto l1066; }
  l1069:;	
  {  int yypos1070= G->pos, yythunkpos1070= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1070; }  goto l1069;
  l1070:;	  G->pos= yypos1070; G->thunkpos= yythunkpos1070;
  }  if (!yymatchChar(G, '>')) goto l1066;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenH6", G->buf+G->pos));
  return 1;
  l1066:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenH6", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockH5(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "HtmlBlockH5"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1071;  if (!yy_LocMarker(G)) { goto l1071; }  yyDo(G, yySet, -1, 0);  if (!yy_HtmlBlockOpenH5(G)) { goto l1071; }
  l1072:;	
  {  int yypos1073= G->pos, yythunkpos1073= G->thunkpos;
  {  int yypos1074= G->pos, yythunkpos1074= G->thunkpos;  if (!yy_HtmlBlockH5(G)) { goto l1075; }  goto l1074;
  l1075:;	  G->pos= yypos1074; G->thunkpos= yythunkpos1074;
  {  int yypos1076= G->pos, yythunkpos1076= G->thunkpos;  if (!yy_HtmlBlockCloseH5(G)) { goto l1076; }  goto l1073;
  l1076:;	  G->pos= yypos1076; G->thunkpos= yythunkpos1076;
  }  if (!yymatchDot(G)) goto l1073;
  }
  l1074:;	  goto l1072;
  l1073:;	  G->pos= yypos1073; G->thunkpos= yythunkpos1073;
  }  if (!yy_HtmlBlockCloseH5(G)) { goto l1071; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1071;  yyDo(G, yy_1_HtmlBlockH5, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockH5", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1071:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockH5", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseH5(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseH5"));  if (!yymatchChar(G, '<')) goto l1077;  if (!yy_Spnl(G)) { goto l1077; }  if (!yymatchChar(G, '/')) goto l1077;
  {  int yypos1078= G->pos, yythunkpos1078= G->thunkpos;  if (!yymatchString(G, "h5")) goto l1079;  goto l1078;
  l1079:;	  G->pos= yypos1078; G->thunkpos= yythunkpos1078;  if (!yymatchString(G, "H5")) goto l1077;
  }
  l1078:;	  if (!yy_Spnl(G)) { goto l1077; }  if (!yymatchChar(G, '>')) goto l1077;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseH5", G->buf+G->pos));
  return 1;
  l1077:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseH5", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenH5(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenH5"));  if (!yymatchChar(G, '<')) goto l1080;  if (!yy_Spnl(G)) { goto l1080; }
  {  int yypos1081= G->pos, yythunkpos1081= G->thunkpos;  if (!yymatchString(G, "h5")) goto l1082;  goto l1081;
  l1082:;	  G->pos= yypos1081; G->thunkpos= yythunkpos1081;  if (!yymatchString(G, "H5")) goto l1080;
  }
  l1081:;	  if (!yy_Spnl(G)) { goto l1080; }
  l1083:;	
  {  int yypos1084= G->pos, yythunkpos1084= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1084; }  goto l1083;
  l1084:;	  G->pos= yypos1084; G->thunkpos= yythunkpos1084;
  }  if (!yymatchChar(G, '>')) goto l1080;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenH5", G->buf+G->pos));
  return 1;
  l1080:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenH5", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockH4(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "HtmlBlockH4"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1085;  if (!yy_LocMarker(G)) { goto l1085; }  yyDo(G, yySet, -1, 0);  if (!yy_HtmlBlockOpenH4(G)) { goto l1085; }
  l1086:;	
  {  int yypos1087= G->pos, yythunkpos1087= G->thunkpos;
  {  int yypos1088= G->pos, yythunkpos1088= G->thunkpos;  if (!yy_HtmlBlockH4(G)) { goto l1089; }  goto l1088;
  l1089:;	  G->pos= yypos1088; G->thunkpos= yythunkpos1088;
  {  int yypos1090= G->pos, yythunkpos1090= G->thunkpos;  if (!yy_HtmlBlockCloseH4(G)) { goto l1090; }  goto l1087;
  l1090:;	  G->pos= yypos1090; G->thunkpos= yythunkpos1090;
  }  if (!yymatchDot(G)) goto l1087;
  }
  l1088:;	  goto l1086;
  l1087:;	  G->pos= yypos1087; G->thunkpos= yythunkpos1087;
  }  if (!yy_HtmlBlockCloseH4(G)) { goto l1085; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1085;  yyDo(G, yy_1_HtmlBlockH4, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockH4", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1085:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockH4", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseH4(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseH4"));  if (!yymatchChar(G, '<')) goto l1091;  if (!yy_Spnl(G)) { goto l1091; }  if (!yymatchChar(G, '/')) goto l1091;
  {  int yypos1092= G->pos, yythunkpos1092= G->thunkpos;  if (!yymatchString(G, "h4")) goto l1093;  goto l1092;
  l1093:;	  G->pos= yypos1092; G->thunkpos= yythunkpos1092;  if (!yymatchString(G, "H4")) goto l1091;
  }
  l1092:;	  if (!yy_Spnl(G)) { goto l1091; }  if (!yymatchChar(G, '>')) goto l1091;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseH4", G->buf+G->pos));
  return 1;
  l1091:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseH4", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenH4(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenH4"));  if (!yymatchChar(G, '<')) goto l1094;  if (!yy_Spnl(G)) { goto l1094; }
  {  int yypos1095= G->pos, yythunkpos1095= G->thunkpos;  if (!yymatchString(G, "h4")) goto l1096;  goto l1095;
  l1096:;	  G->pos= yypos1095; G->thunkpos= yythunkpos1095;  if (!yymatchString(G, "H4")) goto l1094;
  }
  l1095:;	  if (!yy_Spnl(G)) { goto l1094; }
  l1097:;	
  {  int yypos1098= G->pos, yythunkpos1098= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1098; }  goto l1097;
  l1098:;	  G->pos= yypos1098; G->thunkpos= yythunkpos1098;
  }  if (!yymatchChar(G, '>')) goto l1094;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenH4", G->buf+G->pos));
  return 1;
  l1094:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenH4", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockH3(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "HtmlBlockH3"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1099;  if (!yy_LocMarker(G)) { goto l1099; }  yyDo(G, yySet, -1, 0);  if (!yy_HtmlBlockOpenH3(G)) { goto l1099; }
  l1100:;	
  {  int yypos1101= G->pos, yythunkpos1101= G->thunkpos;
  {  int yypos1102= G->pos, yythunkpos1102= G->thunkpos;  if (!yy_HtmlBlockH3(G)) { goto l1103; }  goto l1102;
  l1103:;	  G->pos= yypos1102; G->thunkpos= yythunkpos1102;
  {  int yypos1104= G->pos, yythunkpos1104= G->thunkpos;  if (!yy_HtmlBlockCloseH3(G)) { goto l1104; }  goto l1101;
  l1104:;	  G->pos= yypos1104; G->thunkpos= yythunkpos1104;
  }  if (!yymatchDot(G)) goto l1101;
  }
  l1102:;	  goto l1100;
  l1101:;	  G->pos= yypos1101; G->thunkpos= yythunkpos1101;
  }  if (!yy_HtmlBlockCloseH3(G)) { goto l1099; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1099;  yyDo(G, yy_1_HtmlBlockH3, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockH3", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1099:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockH3", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseH3(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseH3"));  if (!yymatchChar(G, '<')) goto l1105;  if (!yy_Spnl(G)) { goto l1105; }  if (!yymatchChar(G, '/')) goto l1105;
  {  int yypos1106= G->pos, yythunkpos1106= G->thunkpos;  if (!yymatchString(G, "h3")) goto l1107;  goto l1106;
  l1107:;	  G->pos= yypos1106; G->thunkpos= yythunkpos1106;  if (!yymatchString(G, "H3")) goto l1105;
  }
  l1106:;	  if (!yy_Spnl(G)) { goto l1105; }  if (!yymatchChar(G, '>')) goto l1105;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseH3", G->buf+G->pos));
  return 1;
  l1105:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseH3", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenH3(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenH3"));  if (!yymatchChar(G, '<')) goto l1108;  if (!yy_Spnl(G)) { goto l1108; }
  {  int yypos1109= G->pos, yythunkpos1109= G->thunkpos;  if (!yymatchString(G, "h3")) goto l1110;  goto l1109;
  l1110:;	  G->pos= yypos1109; G->thunkpos= yythunkpos1109;  if (!yymatchString(G, "H3")) goto l1108;
  }
  l1109:;	  if (!yy_Spnl(G)) { goto l1108; }
  l1111:;	
  {  int yypos1112= G->pos, yythunkpos1112= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1112; }  goto l1111;
  l1112:;	  G->pos= yypos1112; G->thunkpos= yythunkpos1112;
  }  if (!yymatchChar(G, '>')) goto l1108;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenH3", G->buf+G->pos));
  return 1;
  l1108:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenH3", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockH2(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "HtmlBlockH2"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1113;  if (!yy_LocMarker(G)) { goto l1113; }  yyDo(G, yySet, -1, 0);  if (!yy_HtmlBlockOpenH2(G)) { goto l1113; }
  l1114:;	
  {  int yypos1115= G->pos, yythunkpos1115= G->thunkpos;
  {  int yypos1116= G->pos, yythunkpos1116= G->thunkpos;  if (!yy_HtmlBlockH2(G)) { goto l1117; }  goto l1116;
  l1117:;	  G->pos= yypos1116; G->thunkpos= yythunkpos1116;
  {  int yypos1118= G->pos, yythunkpos1118= G->thunkpos;  if (!yy_HtmlBlockCloseH2(G)) { goto l1118; }  goto l1115;
  l1118:;	  G->pos= yypos1118; G->thunkpos= yythunkpos1118;
  }  if (!yymatchDot(G)) goto l1115;
  }
  l1116:;	  goto l1114;
  l1115:;	  G->pos= yypos1115; G->thunkpos= yythunkpos1115;
  }  if (!yy_HtmlBlockCloseH2(G)) { goto l1113; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1113;  yyDo(G, yy_1_HtmlBlockH2, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockH2", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1113:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockH2", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseH2(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseH2"));  if (!yymatchChar(G, '<')) goto l1119;  if (!yy_Spnl(G)) { goto l1119; }  if (!yymatchChar(G, '/')) goto l1119;
  {  int yypos1120= G->pos, yythunkpos1120= G->thunkpos;  if (!yymatchString(G, "h2")) goto l1121;  goto l1120;
  l1121:;	  G->pos= yypos1120; G->thunkpos= yythunkpos1120;  if (!yymatchString(G, "H2")) goto l1119;
  }
  l1120:;	  if (!yy_Spnl(G)) { goto l1119; }  if (!yymatchChar(G, '>')) goto l1119;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseH2", G->buf+G->pos));
  return 1;
  l1119:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseH2", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenH2(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenH2"));  if (!yymatchChar(G, '<')) goto l1122;  if (!yy_Spnl(G)) { goto l1122; }
  {  int yypos1123= G->pos, yythunkpos1123= G->thunkpos;  if (!yymatchString(G, "h2")) goto l1124;  goto l1123;
  l1124:;	  G->pos= yypos1123; G->thunkpos= yythunkpos1123;  if (!yymatchString(G, "H2")) goto l1122;
  }
  l1123:;	  if (!yy_Spnl(G)) { goto l1122; }
  l1125:;	
  {  int yypos1126= G->pos, yythunkpos1126= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1126; }  goto l1125;
  l1126:;	  G->pos= yypos1126; G->thunkpos= yythunkpos1126;
  }  if (!yymatchChar(G, '>')) goto l1122;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenH2", G->buf+G->pos));
  return 1;
  l1122:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenH2", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockH1(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "HtmlBlockH1"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1127;  if (!yy_LocMarker(G)) { goto l1127; }  yyDo(G, yySet, -1, 0);  if (!yy_HtmlBlockOpenH1(G)) { goto l1127; }
  l1128:;	
  {  int yypos1129= G->pos, yythunkpos1129= G->thunkpos;
  {  int yypos1130= G->pos, yythunkpos1130= G->thunkpos;  if (!yy_HtmlBlockH1(G)) { goto l1131; }  goto l1130;
  l1131:;	  G->pos= yypos1130; G->thunkpos= yythunkpos1130;
  {  int yypos1132= G->pos, yythunkpos1132= G->thunkpos;  if (!yy_HtmlBlockCloseH1(G)) { goto l1132; }  goto l1129;
  l1132:;	  G->pos= yypos1132; G->thunkpos= yythunkpos1132;
  }  if (!yymatchDot(G)) goto l1129;
  }
  l1130:;	  goto l1128;
  l1129:;	  G->pos= yypos1129; G->thunkpos= yythunkpos1129;
  }  if (!yy_HtmlBlockCloseH1(G)) { goto l1127; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1127;  yyDo(G, yy_1_HtmlBlockH1, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockH1", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1127:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockH1", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseH1(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseH1"));  if (!yymatchChar(G, '<')) goto l1133;  if (!yy_Spnl(G)) { goto l1133; }  if (!yymatchChar(G, '/')) goto l1133;
  {  int yypos1134= G->pos, yythunkpos1134= G->thunkpos;  if (!yymatchString(G, "h1")) goto l1135;  goto l1134;
  l1135:;	  G->pos= yypos1134; G->thunkpos= yythunkpos1134;  if (!yymatchString(G, "H1")) goto l1133;
  }
  l1134:;	  if (!yy_Spnl(G)) { goto l1133; }  if (!yymatchChar(G, '>')) goto l1133;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseH1", G->buf+G->pos));
  return 1;
  l1133:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseH1", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenH1(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenH1"));  if (!yymatchChar(G, '<')) goto l1136;  if (!yy_Spnl(G)) { goto l1136; }
  {  int yypos1137= G->pos, yythunkpos1137= G->thunkpos;  if (!yymatchString(G, "h1")) goto l1138;  goto l1137;
  l1138:;	  G->pos= yypos1137; G->thunkpos= yythunkpos1137;  if (!yymatchString(G, "H1")) goto l1136;
  }
  l1137:;	  if (!yy_Spnl(G)) { goto l1136; }
  l1139:;	
  {  int yypos1140= G->pos, yythunkpos1140= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1140; }  goto l1139;
  l1140:;	  G->pos= yypos1140; G->thunkpos= yythunkpos1140;
  }  if (!yymatchChar(G, '>')) goto l1136;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenH1", G->buf+G->pos));
  return 1;
  l1136:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenH1", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockForm(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockForm"));  if (!yy_HtmlBlockOpenForm(G)) { goto l1141; }
  l1142:;	
  {  int yypos1143= G->pos, yythunkpos1143= G->thunkpos;
  {  int yypos1144= G->pos, yythunkpos1144= G->thunkpos;  if (!yy_HtmlBlockForm(G)) { goto l1145; }  goto l1144;
  l1145:;	  G->pos= yypos1144; G->thunkpos= yythunkpos1144;
  {  int yypos1146= G->pos, yythunkpos1146= G->thunkpos;  if (!yy_HtmlBlockCloseForm(G)) { goto l1146; }  goto l1143;
  l1146:;	  G->pos= yypos1146; G->thunkpos= yythunkpos1146;
  }  if (!yymatchDot(G)) goto l1143;
  }
  l1144:;	  goto l1142;
  l1143:;	  G->pos= yypos1143; G->thunkpos= yythunkpos1143;
  }  if (!yy_HtmlBlockCloseForm(G)) { goto l1141; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockForm", G->buf+G->pos));
  return 1;
  l1141:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockForm", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseForm(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseForm"));  if (!yymatchChar(G, '<')) goto l1147;  if (!yy_Spnl(G)) { goto l1147; }  if (!yymatchChar(G, '/')) goto l1147;
  {  int yypos1148= G->pos, yythunkpos1148= G->thunkpos;  if (!yymatchString(G, "form")) goto l1149;  goto l1148;
  l1149:;	  G->pos= yypos1148; G->thunkpos= yythunkpos1148;  if (!yymatchString(G, "FORM")) goto l1147;
  }
  l1148:;	  if (!yy_Spnl(G)) { goto l1147; }  if (!yymatchChar(G, '>')) goto l1147;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseForm", G->buf+G->pos));
  return 1;
  l1147:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseForm", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenForm(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenForm"));  if (!yymatchChar(G, '<')) goto l1150;  if (!yy_Spnl(G)) { goto l1150; }
  {  int yypos1151= G->pos, yythunkpos1151= G->thunkpos;  if (!yymatchString(G, "form")) goto l1152;  goto l1151;
  l1152:;	  G->pos= yypos1151; G->thunkpos= yythunkpos1151;  if (!yymatchString(G, "FORM")) goto l1150;
  }
  l1151:;	  if (!yy_Spnl(G)) { goto l1150; }
  l1153:;	
  {  int yypos1154= G->pos, yythunkpos1154= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1154; }  goto l1153;
  l1154:;	  G->pos= yypos1154; G->thunkpos= yythunkpos1154;
  }  if (!yymatchChar(G, '>')) goto l1150;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenForm", G->buf+G->pos));
  return 1;
  l1150:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenForm", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockFieldset(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockFieldset"));  if (!yy_HtmlBlockOpenFieldset(G)) { goto l1155; }
  l1156:;	
  {  int yypos1157= G->pos, yythunkpos1157= G->thunkpos;
  {  int yypos1158= G->pos, yythunkpos1158= G->thunkpos;  if (!yy_HtmlBlockFieldset(G)) { goto l1159; }  goto l1158;
  l1159:;	  G->pos= yypos1158; G->thunkpos= yythunkpos1158;
  {  int yypos1160= G->pos, yythunkpos1160= G->thunkpos;  if (!yy_HtmlBlockCloseFieldset(G)) { goto l1160; }  goto l1157;
  l1160:;	  G->pos= yypos1160; G->thunkpos= yythunkpos1160;
  }  if (!yymatchDot(G)) goto l1157;
  }
  l1158:;	  goto l1156;
  l1157:;	  G->pos= yypos1157; G->thunkpos= yythunkpos1157;
  }  if (!yy_HtmlBlockCloseFieldset(G)) { goto l1155; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockFieldset", G->buf+G->pos));
  return 1;
  l1155:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockFieldset", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseFieldset(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseFieldset"));  if (!yymatchChar(G, '<')) goto l1161;  if (!yy_Spnl(G)) { goto l1161; }  if (!yymatchChar(G, '/')) goto l1161;
  {  int yypos1162= G->pos, yythunkpos1162= G->thunkpos;  if (!yymatchString(G, "fieldset")) goto l1163;  goto l1162;
  l1163:;	  G->pos= yypos1162; G->thunkpos= yythunkpos1162;  if (!yymatchString(G, "FIELDSET")) goto l1161;
  }
  l1162:;	  if (!yy_Spnl(G)) { goto l1161; }  if (!yymatchChar(G, '>')) goto l1161;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseFieldset", G->buf+G->pos));
  return 1;
  l1161:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseFieldset", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenFieldset(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenFieldset"));  if (!yymatchChar(G, '<')) goto l1164;  if (!yy_Spnl(G)) { goto l1164; }
  {  int yypos1165= G->pos, yythunkpos1165= G->thunkpos;  if (!yymatchString(G, "fieldset")) goto l1166;  goto l1165;
  l1166:;	  G->pos= yypos1165; G->thunkpos= yythunkpos1165;  if (!yymatchString(G, "FIELDSET")) goto l1164;
  }
  l1165:;	  if (!yy_Spnl(G)) { goto l1164; }
  l1167:;	
  {  int yypos1168= G->pos, yythunkpos1168= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1168; }  goto l1167;
  l1168:;	  G->pos= yypos1168; G->thunkpos= yythunkpos1168;
  }  if (!yymatchChar(G, '>')) goto l1164;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenFieldset", G->buf+G->pos));
  return 1;
  l1164:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenFieldset", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockDl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockDl"));  if (!yy_HtmlBlockOpenDl(G)) { goto l1169; }
  l1170:;	
  {  int yypos1171= G->pos, yythunkpos1171= G->thunkpos;
  {  int yypos1172= G->pos, yythunkpos1172= G->thunkpos;  if (!yy_HtmlBlockDl(G)) { goto l1173; }  goto l1172;
  l1173:;	  G->pos= yypos1172; G->thunkpos= yythunkpos1172;
  {  int yypos1174= G->pos, yythunkpos1174= G->thunkpos;  if (!yy_HtmlBlockCloseDl(G)) { goto l1174; }  goto l1171;
  l1174:;	  G->pos= yypos1174; G->thunkpos= yythunkpos1174;
  }  if (!yymatchDot(G)) goto l1171;
  }
  l1172:;	  goto l1170;
  l1171:;	  G->pos= yypos1171; G->thunkpos= yythunkpos1171;
  }  if (!yy_HtmlBlockCloseDl(G)) { goto l1169; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockDl", G->buf+G->pos));
  return 1;
  l1169:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockDl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseDl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseDl"));  if (!yymatchChar(G, '<')) goto l1175;  if (!yy_Spnl(G)) { goto l1175; }  if (!yymatchChar(G, '/')) goto l1175;
  {  int yypos1176= G->pos, yythunkpos1176= G->thunkpos;  if (!yymatchString(G, "dl")) goto l1177;  goto l1176;
  l1177:;	  G->pos= yypos1176; G->thunkpos= yythunkpos1176;  if (!yymatchString(G, "DL")) goto l1175;
  }
  l1176:;	  if (!yy_Spnl(G)) { goto l1175; }  if (!yymatchChar(G, '>')) goto l1175;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseDl", G->buf+G->pos));
  return 1;
  l1175:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseDl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenDl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenDl"));  if (!yymatchChar(G, '<')) goto l1178;  if (!yy_Spnl(G)) { goto l1178; }
  {  int yypos1179= G->pos, yythunkpos1179= G->thunkpos;  if (!yymatchString(G, "dl")) goto l1180;  goto l1179;
  l1180:;	  G->pos= yypos1179; G->thunkpos= yythunkpos1179;  if (!yymatchString(G, "DL")) goto l1178;
  }
  l1179:;	  if (!yy_Spnl(G)) { goto l1178; }
  l1181:;	
  {  int yypos1182= G->pos, yythunkpos1182= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1182; }  goto l1181;
  l1182:;	  G->pos= yypos1182; G->thunkpos= yythunkpos1182;
  }  if (!yymatchChar(G, '>')) goto l1178;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenDl", G->buf+G->pos));
  return 1;
  l1178:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenDl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockDiv(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockDiv"));  if (!yy_HtmlBlockOpenDiv(G)) { goto l1183; }
  l1184:;	
  {  int yypos1185= G->pos, yythunkpos1185= G->thunkpos;
  {  int yypos1186= G->pos, yythunkpos1186= G->thunkpos;  if (!yy_HtmlBlockDiv(G)) { goto l1187; }  goto l1186;
  l1187:;	  G->pos= yypos1186; G->thunkpos= yythunkpos1186;
  {  int yypos1188= G->pos, yythunkpos1188= G->thunkpos;  if (!yy_HtmlBlockCloseDiv(G)) { goto l1188; }  goto l1185;
  l1188:;	  G->pos= yypos1188; G->thunkpos= yythunkpos1188;
  }  if (!yymatchDot(G)) goto l1185;
  }
  l1186:;	  goto l1184;
  l1185:;	  G->pos= yypos1185; G->thunkpos= yythunkpos1185;
  }  if (!yy_HtmlBlockCloseDiv(G)) { goto l1183; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockDiv", G->buf+G->pos));
  return 1;
  l1183:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockDiv", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseDiv(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseDiv"));  if (!yymatchChar(G, '<')) goto l1189;  if (!yy_Spnl(G)) { goto l1189; }  if (!yymatchChar(G, '/')) goto l1189;
  {  int yypos1190= G->pos, yythunkpos1190= G->thunkpos;  if (!yymatchString(G, "div")) goto l1191;  goto l1190;
  l1191:;	  G->pos= yypos1190; G->thunkpos= yythunkpos1190;  if (!yymatchString(G, "DIV")) goto l1189;
  }
  l1190:;	  if (!yy_Spnl(G)) { goto l1189; }  if (!yymatchChar(G, '>')) goto l1189;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseDiv", G->buf+G->pos));
  return 1;
  l1189:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseDiv", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenDiv(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenDiv"));  if (!yymatchChar(G, '<')) goto l1192;  if (!yy_Spnl(G)) { goto l1192; }
  {  int yypos1193= G->pos, yythunkpos1193= G->thunkpos;  if (!yymatchString(G, "div")) goto l1194;  goto l1193;
  l1194:;	  G->pos= yypos1193; G->thunkpos= yythunkpos1193;  if (!yymatchString(G, "DIV")) goto l1192;
  }
  l1193:;	  if (!yy_Spnl(G)) { goto l1192; }
  l1195:;	
  {  int yypos1196= G->pos, yythunkpos1196= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1196; }  goto l1195;
  l1196:;	  G->pos= yypos1196; G->thunkpos= yythunkpos1196;
  }  if (!yymatchChar(G, '>')) goto l1192;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenDiv", G->buf+G->pos));
  return 1;
  l1192:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenDiv", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockDir(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockDir"));  if (!yy_HtmlBlockOpenDir(G)) { goto l1197; }
  l1198:;	
  {  int yypos1199= G->pos, yythunkpos1199= G->thunkpos;
  {  int yypos1200= G->pos, yythunkpos1200= G->thunkpos;  if (!yy_HtmlBlockDir(G)) { goto l1201; }  goto l1200;
  l1201:;	  G->pos= yypos1200; G->thunkpos= yythunkpos1200;
  {  int yypos1202= G->pos, yythunkpos1202= G->thunkpos;  if (!yy_HtmlBlockCloseDir(G)) { goto l1202; }  goto l1199;
  l1202:;	  G->pos= yypos1202; G->thunkpos= yythunkpos1202;
  }  if (!yymatchDot(G)) goto l1199;
  }
  l1200:;	  goto l1198;
  l1199:;	  G->pos= yypos1199; G->thunkpos= yythunkpos1199;
  }  if (!yy_HtmlBlockCloseDir(G)) { goto l1197; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockDir", G->buf+G->pos));
  return 1;
  l1197:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockDir", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseDir(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseDir"));  if (!yymatchChar(G, '<')) goto l1203;  if (!yy_Spnl(G)) { goto l1203; }  if (!yymatchChar(G, '/')) goto l1203;
  {  int yypos1204= G->pos, yythunkpos1204= G->thunkpos;  if (!yymatchString(G, "dir")) goto l1205;  goto l1204;
  l1205:;	  G->pos= yypos1204; G->thunkpos= yythunkpos1204;  if (!yymatchString(G, "DIR")) goto l1203;
  }
  l1204:;	  if (!yy_Spnl(G)) { goto l1203; }  if (!yymatchChar(G, '>')) goto l1203;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseDir", G->buf+G->pos));
  return 1;
  l1203:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseDir", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenDir(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenDir"));  if (!yymatchChar(G, '<')) goto l1206;  if (!yy_Spnl(G)) { goto l1206; }
  {  int yypos1207= G->pos, yythunkpos1207= G->thunkpos;  if (!yymatchString(G, "dir")) goto l1208;  goto l1207;
  l1208:;	  G->pos= yypos1207; G->thunkpos= yythunkpos1207;  if (!yymatchString(G, "DIR")) goto l1206;
  }
  l1207:;	  if (!yy_Spnl(G)) { goto l1206; }
  l1209:;	
  {  int yypos1210= G->pos, yythunkpos1210= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1210; }  goto l1209;
  l1210:;	  G->pos= yypos1210; G->thunkpos= yythunkpos1210;
  }  if (!yymatchChar(G, '>')) goto l1206;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenDir", G->buf+G->pos));
  return 1;
  l1206:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenDir", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCenter(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCenter"));  if (!yy_HtmlBlockOpenCenter(G)) { goto l1211; }
  l1212:;	
  {  int yypos1213= G->pos, yythunkpos1213= G->thunkpos;
  {  int yypos1214= G->pos, yythunkpos1214= G->thunkpos;  if (!yy_HtmlBlockCenter(G)) { goto l1215; }  goto l1214;
  l1215:;	  G->pos= yypos1214; G->thunkpos= yythunkpos1214;
  {  int yypos1216= G->pos, yythunkpos1216= G->thunkpos;  if (!yy_HtmlBlockCloseCenter(G)) { goto l1216; }  goto l1213;
  l1216:;	  G->pos= yypos1216; G->thunkpos= yythunkpos1216;
  }  if (!yymatchDot(G)) goto l1213;
  }
  l1214:;	  goto l1212;
  l1213:;	  G->pos= yypos1213; G->thunkpos= yythunkpos1213;
  }  if (!yy_HtmlBlockCloseCenter(G)) { goto l1211; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCenter", G->buf+G->pos));
  return 1;
  l1211:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCenter", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseCenter(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseCenter"));  if (!yymatchChar(G, '<')) goto l1217;  if (!yy_Spnl(G)) { goto l1217; }  if (!yymatchChar(G, '/')) goto l1217;
  {  int yypos1218= G->pos, yythunkpos1218= G->thunkpos;  if (!yymatchString(G, "center")) goto l1219;  goto l1218;
  l1219:;	  G->pos= yypos1218; G->thunkpos= yythunkpos1218;  if (!yymatchString(G, "CENTER")) goto l1217;
  }
  l1218:;	  if (!yy_Spnl(G)) { goto l1217; }  if (!yymatchChar(G, '>')) goto l1217;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseCenter", G->buf+G->pos));
  return 1;
  l1217:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseCenter", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenCenter(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenCenter"));  if (!yymatchChar(G, '<')) goto l1220;  if (!yy_Spnl(G)) { goto l1220; }
  {  int yypos1221= G->pos, yythunkpos1221= G->thunkpos;  if (!yymatchString(G, "center")) goto l1222;  goto l1221;
  l1222:;	  G->pos= yypos1221; G->thunkpos= yythunkpos1221;  if (!yymatchString(G, "CENTER")) goto l1220;
  }
  l1221:;	  if (!yy_Spnl(G)) { goto l1220; }
  l1223:;	
  {  int yypos1224= G->pos, yythunkpos1224= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1224; }  goto l1223;
  l1224:;	  G->pos= yypos1224; G->thunkpos= yythunkpos1224;
  }  if (!yymatchChar(G, '>')) goto l1220;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenCenter", G->buf+G->pos));
  return 1;
  l1220:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenCenter", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockBlockquote(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockBlockquote"));  if (!yy_HtmlBlockOpenBlockquote(G)) { goto l1225; }
  l1226:;	
  {  int yypos1227= G->pos, yythunkpos1227= G->thunkpos;
  {  int yypos1228= G->pos, yythunkpos1228= G->thunkpos;  if (!yy_HtmlBlockBlockquote(G)) { goto l1229; }  goto l1228;
  l1229:;	  G->pos= yypos1228; G->thunkpos= yythunkpos1228;
  {  int yypos1230= G->pos, yythunkpos1230= G->thunkpos;  if (!yy_HtmlBlockCloseBlockquote(G)) { goto l1230; }  goto l1227;
  l1230:;	  G->pos= yypos1230; G->thunkpos= yythunkpos1230;
  }  if (!yymatchDot(G)) goto l1227;
  }
  l1228:;	  goto l1226;
  l1227:;	  G->pos= yypos1227; G->thunkpos= yythunkpos1227;
  }  if (!yy_HtmlBlockCloseBlockquote(G)) { goto l1225; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockBlockquote", G->buf+G->pos));
  return 1;
  l1225:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockBlockquote", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseBlockquote(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseBlockquote"));  if (!yymatchChar(G, '<')) goto l1231;  if (!yy_Spnl(G)) { goto l1231; }  if (!yymatchChar(G, '/')) goto l1231;
  {  int yypos1232= G->pos, yythunkpos1232= G->thunkpos;  if (!yymatchString(G, "blockquote")) goto l1233;  goto l1232;
  l1233:;	  G->pos= yypos1232; G->thunkpos= yythunkpos1232;  if (!yymatchString(G, "BLOCKQUOTE")) goto l1231;
  }
  l1232:;	  if (!yy_Spnl(G)) { goto l1231; }  if (!yymatchChar(G, '>')) goto l1231;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseBlockquote", G->buf+G->pos));
  return 1;
  l1231:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseBlockquote", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenBlockquote(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenBlockquote"));  if (!yymatchChar(G, '<')) goto l1234;  if (!yy_Spnl(G)) { goto l1234; }
  {  int yypos1235= G->pos, yythunkpos1235= G->thunkpos;  if (!yymatchString(G, "blockquote")) goto l1236;  goto l1235;
  l1236:;	  G->pos= yypos1235; G->thunkpos= yythunkpos1235;  if (!yymatchString(G, "BLOCKQUOTE")) goto l1234;
  }
  l1235:;	  if (!yy_Spnl(G)) { goto l1234; }
  l1237:;	
  {  int yypos1238= G->pos, yythunkpos1238= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1238; }  goto l1237;
  l1238:;	  G->pos= yypos1238; G->thunkpos= yythunkpos1238;
  }  if (!yymatchChar(G, '>')) goto l1234;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenBlockquote", G->buf+G->pos));
  return 1;
  l1234:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenBlockquote", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockAddress(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockAddress"));  if (!yy_HtmlBlockOpenAddress(G)) { goto l1239; }
  l1240:;	
  {  int yypos1241= G->pos, yythunkpos1241= G->thunkpos;
  {  int yypos1242= G->pos, yythunkpos1242= G->thunkpos;  if (!yy_HtmlBlockAddress(G)) { goto l1243; }  goto l1242;
  l1243:;	  G->pos= yypos1242; G->thunkpos= yythunkpos1242;
  {  int yypos1244= G->pos, yythunkpos1244= G->thunkpos;  if (!yy_HtmlBlockCloseAddress(G)) { goto l1244; }  goto l1241;
  l1244:;	  G->pos= yypos1244; G->thunkpos= yythunkpos1244;
  }  if (!yymatchDot(G)) goto l1241;
  }
  l1242:;	  goto l1240;
  l1241:;	  G->pos= yypos1241; G->thunkpos= yythunkpos1241;
  }  if (!yy_HtmlBlockCloseAddress(G)) { goto l1239; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockAddress", G->buf+G->pos));
  return 1;
  l1239:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockAddress", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockCloseAddress(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockCloseAddress"));  if (!yymatchChar(G, '<')) goto l1245;  if (!yy_Spnl(G)) { goto l1245; }  if (!yymatchChar(G, '/')) goto l1245;
  {  int yypos1246= G->pos, yythunkpos1246= G->thunkpos;  if (!yymatchString(G, "address")) goto l1247;  goto l1246;
  l1247:;	  G->pos= yypos1246; G->thunkpos= yythunkpos1246;  if (!yymatchString(G, "ADDRESS")) goto l1245;
  }
  l1246:;	  if (!yy_Spnl(G)) { goto l1245; }  if (!yymatchChar(G, '>')) goto l1245;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockCloseAddress", G->buf+G->pos));
  return 1;
  l1245:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockCloseAddress", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlAttribute(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlAttribute"));
  {  int yypos1251= G->pos, yythunkpos1251= G->thunkpos;  if (!yy_AlphanumericAscii(G)) { goto l1252; }  goto l1251;
  l1252:;	  G->pos= yypos1251; G->thunkpos= yythunkpos1251;  if (!yymatchChar(G, '-')) goto l1248;
  }
  l1251:;	
  l1249:;	
  {  int yypos1250= G->pos, yythunkpos1250= G->thunkpos;
  {  int yypos1253= G->pos, yythunkpos1253= G->thunkpos;  if (!yy_AlphanumericAscii(G)) { goto l1254; }  goto l1253;
  l1254:;	  G->pos= yypos1253; G->thunkpos= yythunkpos1253;  if (!yymatchChar(G, '-')) goto l1250;
  }
  l1253:;	  goto l1249;
  l1250:;	  G->pos= yypos1250; G->thunkpos= yythunkpos1250;
  }  if (!yy_Spnl(G)) { goto l1248; }
  {  int yypos1255= G->pos, yythunkpos1255= G->thunkpos;  if (!yymatchChar(G, '=')) goto l1255;  if (!yy_Spnl(G)) { goto l1255; }
  {  int yypos1257= G->pos, yythunkpos1257= G->thunkpos;  if (!yy_Quoted(G)) { goto l1258; }  goto l1257;
  l1258:;	  G->pos= yypos1257; G->thunkpos= yythunkpos1257;
  {  int yypos1261= G->pos, yythunkpos1261= G->thunkpos;  if (!yymatchChar(G, '>')) goto l1261;  goto l1255;
  l1261:;	  G->pos= yypos1261; G->thunkpos= yythunkpos1261;
  }  if (!yy_Nonspacechar(G)) { goto l1255; }
  l1259:;	
  {  int yypos1260= G->pos, yythunkpos1260= G->thunkpos;
  {  int yypos1262= G->pos, yythunkpos1262= G->thunkpos;  if (!yymatchChar(G, '>')) goto l1262;  goto l1260;
  l1262:;	  G->pos= yypos1262; G->thunkpos= yythunkpos1262;
  }  if (!yy_Nonspacechar(G)) { goto l1260; }  goto l1259;
  l1260:;	  G->pos= yypos1260; G->thunkpos= yythunkpos1260;
  }
  }
  l1257:;	  goto l1256;
  l1255:;	  G->pos= yypos1255; G->thunkpos= yythunkpos1255;
  }
  l1256:;	  if (!yy_Spnl(G)) { goto l1248; }
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlAttribute", G->buf+G->pos));
  return 1;
  l1248:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlAttribute", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlockOpenAddress(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HtmlBlockOpenAddress"));  if (!yymatchChar(G, '<')) goto l1263;  if (!yy_Spnl(G)) { goto l1263; }
  {  int yypos1264= G->pos, yythunkpos1264= G->thunkpos;  if (!yymatchString(G, "address")) goto l1265;  goto l1264;
  l1265:;	  G->pos= yypos1264; G->thunkpos= yythunkpos1264;  if (!yymatchString(G, "ADDRESS")) goto l1263;
  }
  l1264:;	  if (!yy_Spnl(G)) { goto l1263; }
  l1266:;	
  {  int yypos1267= G->pos, yythunkpos1267= G->thunkpos;  if (!yy_HtmlAttribute(G)) { goto l1267; }  goto l1266;
  l1267:;	  G->pos= yypos1267; G->thunkpos= yythunkpos1267;
  }  if (!yymatchChar(G, '>')) goto l1263;
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlockOpenAddress", G->buf+G->pos));
  return 1;
  l1263:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlockOpenAddress", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_OptionallyIndentedLine(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "OptionallyIndentedLine"));
  {  int yypos1269= G->pos, yythunkpos1269= G->thunkpos;  if (!yy_Indent(G)) { goto l1269; }  goto l1270;
  l1269:;	  G->pos= yypos1269; G->thunkpos= yythunkpos1269;
  }
  l1270:;	  if (!yy_Line(G)) { goto l1268; }
  yyprintf((stderr, "  ok   %s @ %s\n", "OptionallyIndentedLine", G->buf+G->pos));
  return 1;
  l1268:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "OptionallyIndentedLine", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Indent(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Indent"));
  {  int yypos1272= G->pos, yythunkpos1272= G->thunkpos;  if (!yymatchChar(G, '\t')) goto l1273;  goto l1272;
  l1273:;	  G->pos= yypos1272; G->thunkpos= yythunkpos1272;  if (!yymatchString(G, "    ")) goto l1271;
  }
  l1272:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Indent", G->buf+G->pos));
  return 1;
  l1271:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Indent", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ListBlockLine(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "ListBlockLine"));
  {  int yypos1275= G->pos, yythunkpos1275= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1275; }  goto l1274;
  l1275:;	  G->pos= yypos1275; G->thunkpos= yythunkpos1275;
  }
  {  int yypos1276= G->pos, yythunkpos1276= G->thunkpos;
  {  int yypos1277= G->pos, yythunkpos1277= G->thunkpos;  if (!yy_Indent(G)) { goto l1277; }  goto l1278;
  l1277:;	  G->pos= yypos1277; G->thunkpos= yythunkpos1277;
  }
  l1278:;	
  {  int yypos1279= G->pos, yythunkpos1279= G->thunkpos;  if (!yy_Bullet(G)) { goto l1280; }  goto l1279;
  l1280:;	  G->pos= yypos1279; G->thunkpos= yythunkpos1279;  if (!yy_Enumerator(G)) { goto l1276; }
  }
  l1279:;	  goto l1274;
  l1276:;	  G->pos= yypos1276; G->thunkpos= yythunkpos1276;
  }
  {  int yypos1281= G->pos, yythunkpos1281= G->thunkpos;  if (!yy_HorizontalRule(G)) { goto l1281; }  goto l1274;
  l1281:;	  G->pos= yypos1281; G->thunkpos= yythunkpos1281;
  }  if (!yy_OptionallyIndentedLine(G)) { goto l1274; }
  yyprintf((stderr, "  ok   %s @ %s\n", "ListBlockLine", G->buf+G->pos));
  return 1;
  l1274:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ListBlockLine", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ListContinuationBlock(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "ListContinuationBlock"));  if (!yy_StartList(G)) { goto l1282; }  yyDo(G, yySet, -1, 0);  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1282;
  l1283:;	
  {  int yypos1284= G->pos, yythunkpos1284= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1284; }  goto l1283;
  l1284:;	  G->pos= yypos1284; G->thunkpos= yythunkpos1284;
  }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1282;  yyDo(G, yy_1_ListContinuationBlock, G->begin, G->end);  if (!yy_Indent(G)) { goto l1282; }  if (!yy_ListBlock(G)) { goto l1282; }  yyDo(G, yy_2_ListContinuationBlock, G->begin, G->end);
  l1285:;	
  {  int yypos1286= G->pos, yythunkpos1286= G->thunkpos;  if (!yy_Indent(G)) { goto l1286; }  if (!yy_ListBlock(G)) { goto l1286; }  yyDo(G, yy_2_ListContinuationBlock, G->begin, G->end);  goto l1285;
  l1286:;	  G->pos= yypos1286; G->thunkpos= yythunkpos1286;
  }  yyDo(G, yy_3_ListContinuationBlock, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "ListContinuationBlock", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1282:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ListContinuationBlock", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ListBlock(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "ListBlock"));  if (!yy_StartList(G)) { goto l1287; }  yyDo(G, yySet, -1, 0);
  {  int yypos1288= G->pos, yythunkpos1288= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1288; }  goto l1287;
  l1288:;	  G->pos= yypos1288; G->thunkpos= yythunkpos1288;
  }  if (!yy_Line(G)) { goto l1287; }  yyDo(G, yy_1_ListBlock, G->begin, G->end);
  l1289:;	
  {  int yypos1290= G->pos, yythunkpos1290= G->thunkpos;  if (!yy_ListBlockLine(G)) { goto l1290; }  yyDo(G, yy_2_ListBlock, G->begin, G->end);  goto l1289;
  l1290:;	  G->pos= yypos1290; G->thunkpos= yythunkpos1290;
  }  yyDo(G, yy_3_ListBlock, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "ListBlock", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1287:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ListBlock", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ListItem(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "ListItem"));
  {  int yypos1292= G->pos, yythunkpos1292= G->thunkpos;  if (!yy_Bullet(G)) { goto l1293; }  goto l1292;
  l1293:;	  G->pos= yypos1292; G->thunkpos= yythunkpos1292;  if (!yy_Enumerator(G)) { goto l1291; }
  }
  l1292:;	  if (!yy_StartList(G)) { goto l1291; }  yyDo(G, yySet, -1, 0);  if (!yy_ListBlock(G)) { goto l1291; }  yyDo(G, yy_1_ListItem, G->begin, G->end);
  l1294:;	
  {  int yypos1295= G->pos, yythunkpos1295= G->thunkpos;  if (!yy_ListContinuationBlock(G)) { goto l1295; }  yyDo(G, yy_2_ListItem, G->begin, G->end);  goto l1294;
  l1295:;	  G->pos= yypos1295; G->thunkpos= yythunkpos1295;
  }  yyDo(G, yy_3_ListItem, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "ListItem", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1291:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ListItem", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Enumerator(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Enumerator"));  if (!yy_NonindentSpace(G)) { goto l1296; }  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1296;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l1296;
  l1297:;	
  {  int yypos1298= G->pos, yythunkpos1298= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l1298;  goto l1297;
  l1298:;	  G->pos= yypos1298; G->thunkpos= yythunkpos1298;
  }  if (!yymatchChar(G, '.')) goto l1296;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1296;  if (!yy_Spacechar(G)) { goto l1296; }
  l1299:;	
  {  int yypos1300= G->pos, yythunkpos1300= G->thunkpos;  if (!yy_Spacechar(G)) { goto l1300; }  goto l1299;
  l1300:;	  G->pos= yypos1300; G->thunkpos= yythunkpos1300;
  }  yyDo(G, yy_1_Enumerator, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Enumerator", G->buf+G->pos));
  return 1;
  l1296:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Enumerator", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ListItemTight(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "ListItemTight"));
  {  int yypos1302= G->pos, yythunkpos1302= G->thunkpos;  if (!yy_Bullet(G)) { goto l1303; }  goto l1302;
  l1303:;	  G->pos= yypos1302; G->thunkpos= yythunkpos1302;  if (!yy_Enumerator(G)) { goto l1301; }
  }
  l1302:;	  if (!yy_StartList(G)) { goto l1301; }  yyDo(G, yySet, -1, 0);  if (!yy_ListBlock(G)) { goto l1301; }  yyDo(G, yy_1_ListItemTight, G->begin, G->end);
  l1304:;	
  {  int yypos1305= G->pos, yythunkpos1305= G->thunkpos;
  {  int yypos1306= G->pos, yythunkpos1306= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1306; }  goto l1305;
  l1306:;	  G->pos= yypos1306; G->thunkpos= yythunkpos1306;
  }  if (!yy_ListContinuationBlock(G)) { goto l1305; }  yyDo(G, yy_2_ListItemTight, G->begin, G->end);  goto l1304;
  l1305:;	  G->pos= yypos1305; G->thunkpos= yythunkpos1305;
  }
  {  int yypos1307= G->pos, yythunkpos1307= G->thunkpos;  if (!yy_ListContinuationBlock(G)) { goto l1307; }  goto l1301;
  l1307:;	  G->pos= yypos1307; G->thunkpos= yythunkpos1307;
  }  yyDo(G, yy_3_ListItemTight, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "ListItemTight", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1301:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ListItemTight", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ListLoose(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 2, 0);
  yyprintf((stderr, "%s\n", "ListLoose"));  if (!yy_StartList(G)) { goto l1308; }  yyDo(G, yySet, -2, 0);  if (!yy_ListItem(G)) { goto l1308; }  yyDo(G, yySet, -1, 0);
  l1311:;	
  {  int yypos1312= G->pos, yythunkpos1312= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1312; }  goto l1311;
  l1312:;	  G->pos= yypos1312; G->thunkpos= yythunkpos1312;
  }  yyDo(G, yy_1_ListLoose, G->begin, G->end);
  l1309:;	
  {  int yypos1310= G->pos, yythunkpos1310= G->thunkpos;  if (!yy_ListItem(G)) { goto l1310; }  yyDo(G, yySet, -1, 0);
  l1313:;	
  {  int yypos1314= G->pos, yythunkpos1314= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1314; }  goto l1313;
  l1314:;	  G->pos= yypos1314; G->thunkpos= yythunkpos1314;
  }  yyDo(G, yy_1_ListLoose, G->begin, G->end);  goto l1309;
  l1310:;	  G->pos= yypos1310; G->thunkpos= yythunkpos1310;
  }  yyDo(G, yy_2_ListLoose, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "ListLoose", G->buf+G->pos));  yyDo(G, yyPop, 2, 0);
  return 1;
  l1308:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ListLoose", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_ListTight(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "ListTight"));  if (!yy_StartList(G)) { goto l1315; }  yyDo(G, yySet, -1, 0);  if (!yy_ListItemTight(G)) { goto l1315; }  yyDo(G, yy_1_ListTight, G->begin, G->end);
  l1316:;	
  {  int yypos1317= G->pos, yythunkpos1317= G->thunkpos;  if (!yy_ListItemTight(G)) { goto l1317; }  yyDo(G, yy_1_ListTight, G->begin, G->end);  goto l1316;
  l1317:;	  G->pos= yypos1317; G->thunkpos= yythunkpos1317;
  }
  l1318:;	
  {  int yypos1319= G->pos, yythunkpos1319= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1319; }  goto l1318;
  l1319:;	  G->pos= yypos1319; G->thunkpos= yythunkpos1319;
  }
  {  int yypos1320= G->pos, yythunkpos1320= G->thunkpos;
  {  int yypos1321= G->pos, yythunkpos1321= G->thunkpos;  if (!yy_Bullet(G)) { goto l1322; }  goto l1321;
  l1322:;	  G->pos= yypos1321; G->thunkpos= yythunkpos1321;  if (!yy_Enumerator(G)) { goto l1320; }
  }
  l1321:;	  goto l1315;
  l1320:;	  G->pos= yypos1320; G->thunkpos= yythunkpos1320;
  }  yyDo(G, yy_2_ListTight, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "ListTight", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1315:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "ListTight", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Bullet(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Bullet"));
  {  int yypos1324= G->pos, yythunkpos1324= G->thunkpos;  if (!yy_HorizontalRule(G)) { goto l1324; }  goto l1323;
  l1324:;	  G->pos= yypos1324; G->thunkpos= yythunkpos1324;
  }  if (!yy_NonindentSpace(G)) { goto l1323; }  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1323;
  {  int yypos1325= G->pos, yythunkpos1325= G->thunkpos;  if (!yymatchChar(G, '+')) goto l1326;  goto l1325;
  l1326:;	  G->pos= yypos1325; G->thunkpos= yythunkpos1325;  if (!yymatchChar(G, '*')) goto l1327;  goto l1325;
  l1327:;	  G->pos= yypos1325; G->thunkpos= yythunkpos1325;  if (!yymatchChar(G, '-')) goto l1323;
  }
  l1325:;	  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1323;  if (!yy_Spacechar(G)) { goto l1323; }
  l1328:;	
  {  int yypos1329= G->pos, yythunkpos1329= G->thunkpos;  if (!yy_Spacechar(G)) { goto l1329; }  goto l1328;
  l1329:;	  G->pos= yypos1329; G->thunkpos= yythunkpos1329;
  }  yyDo(G, yy_1_Bullet, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Bullet", G->buf+G->pos));
  return 1;
  l1323:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Bullet", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_TableCell(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "TableCell"));  if (!yy_Sp(G)) { goto l1330; }
  l1331:;	
  {  int yypos1332= G->pos, yythunkpos1332= G->thunkpos;
  {  int yypos1333= G->pos, yythunkpos1333= G->thunkpos;  if (!yymatchChar(G, '|')) goto l1333;  goto l1332;
  l1333:;	  G->pos= yypos1333; G->thunkpos= yythunkpos1333;
  }
  {  int yypos1334= G->pos, yythunkpos1334= G->thunkpos;  if (!yy_Newline(G)) { goto l1334; }  goto l1332;
  l1334:;	  G->pos= yypos1334; G->thunkpos= yythunkpos1334;
  }  if (!yy_Inline(G)) { goto l1332; }  goto l1331;
  l1332:;	  G->pos= yypos1332; G->thunkpos= yythunkpos1332;
  }  if (!yy_TableBorder(G)) { goto l1330; }
  yyprintf((stderr, "  ok   %s @ %s\n", "TableCell", G->buf+G->pos));
  return 1;
  l1330:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "TableCell", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_TableBorder(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "TableBorder"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1335;  if (!yymatchChar(G, '|')) goto l1335;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1335;  yyDo(G, yy_1_TableBorder, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "TableBorder", G->buf+G->pos));
  return 1;
  l1335:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "TableBorder", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_TableLine(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "TableLine"));  if (!yy_TableBorder(G)) { goto l1336; }  if (!yy_TableCell(G)) { goto l1336; }
  l1337:;	
  {  int yypos1338= G->pos, yythunkpos1338= G->thunkpos;  if (!yy_TableCell(G)) { goto l1338; }  goto l1337;
  l1338:;	  G->pos= yypos1338; G->thunkpos= yythunkpos1338;
  }  if (!yy_Sp(G)) { goto l1336; }  if (!yy_Newline(G)) { goto l1336; }
  yyprintf((stderr, "  ok   %s @ %s\n", "TableLine", G->buf+G->pos));
  return 1;
  l1336:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "TableLine", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_TableDelimiter(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "TableDelimiter"));  if (!yy_TableBorder(G)) { goto l1339; }  if (!yy_Sp(G)) { goto l1339; }
  {  int yypos1342= G->pos, yythunkpos1342= G->thunkpos;  if (!yymatchChar(G, ':')) goto l1342;  goto l1343;
  l1342:;	  G->pos= yypos1342; G->thunkpos= yythunkpos1342;
  }
  l1343:;	  if (!yymatchChar(G, '-')) goto l1339;
  l1344:;	
  {  int yypos1345= G->pos, yythunkpos1345= G->thunkpos;  if (!yymatchChar(G, '-')) goto l1345;  goto l1344;
  l1345:;	  G->pos= yypos1345; G->thunkpos= yythunkpos1345;
  }
  {  int yypos1346= G->pos, yythunkpos1346= G->thunkpos;  if (!yymatchChar(G, ':')) goto l1346;  goto l1347;
  l1346:;	  G->pos= yypos1346; G->thunkpos= yythunkpos1346;
  }
  l1347:;	  if (!yy_Sp(G)) { goto l1339; }  if (!yy_TableBorder(G)) { goto l1339; }
  l1340:;	
  {  int yypos1341= G->pos, yythunkpos1341= G->thunkpos;  if (!yy_Sp(G)) { goto l1341; }
  {  int yypos1348= G->pos, yythunkpos1348= G->thunkpos;  if (!yymatchChar(G, ':')) goto l1348;  goto l1349;
  l1348:;	  G->pos= yypos1348; G->thunkpos= yythunkpos1348;
  }
  l1349:;	  if (!yymatchChar(G, '-')) goto l1341;
  l1350:;	
  {  int yypos1351= G->pos, yythunkpos1351= G->thunkpos;  if (!yymatchChar(G, '-')) goto l1351;  goto l1350;
  l1351:;	  G->pos= yypos1351; G->thunkpos= yythunkpos1351;
  }
  {  int yypos1352= G->pos, yythunkpos1352= G->thunkpos;  if (!yymatchChar(G, ':')) goto l1352;  goto l1353;
  l1352:;	  G->pos= yypos1352; G->thunkpos= yythunkpos1352;
  }
  l1353:;	  if (!yy_Sp(G)) { goto l1341; }  if (!yy_TableBorder(G)) { goto l1341; }  goto l1340;
  l1341:;	  G->pos= yypos1341; G->thunkpos= yythunkpos1341;
  }  if (!yy_Sp(G)) { goto l1339; }  if (!yy_Newline(G)) { goto l1339; }
  yyprintf((stderr, "  ok   %s @ %s\n", "TableDelimiter", G->buf+G->pos));
  return 1;
  l1339:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "TableDelimiter", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_TableHeader(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "TableHeader"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1354;  if (!yy_LocMarker(G)) { goto l1354; }  yyDo(G, yySet, -1, 0);  if (!yy_TableLine(G)) { goto l1354; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1354;  yyDo(G, yy_1_TableHeader, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "TableHeader", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1354:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "TableHeader", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_InlineEquationMultiple(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "InlineEquationMultiple"));
  {  int yypos1356= G->pos, yythunkpos1356= G->thunkpos;  if (!yymatchChar(G, '$')) goto l1356;  goto l1355;
  l1356:;	  G->pos= yypos1356; G->thunkpos= yythunkpos1356;
  }  if (!yy_Nonspacechar(G)) { goto l1355; }
  {  int yypos1359= G->pos, yythunkpos1359= G->thunkpos;  if (!yymatchChar(G, '$')) goto l1359;  goto l1355;
  l1359:;	  G->pos= yypos1359; G->thunkpos= yythunkpos1359;
  }
  {  int yypos1360= G->pos, yythunkpos1360= G->thunkpos;  if (!yy_Newline(G)) { goto l1360; }  goto l1355;
  l1360:;	  G->pos= yypos1360; G->thunkpos= yythunkpos1360;
  }  if (!yymatchDot(G)) goto l1355;
  l1357:;	
  {  int yypos1358= G->pos, yythunkpos1358= G->thunkpos;
  {  int yypos1361= G->pos, yythunkpos1361= G->thunkpos;  if (!yymatchChar(G, '$')) goto l1361;  goto l1358;
  l1361:;	  G->pos= yypos1361; G->thunkpos= yythunkpos1361;
  }
  {  int yypos1362= G->pos, yythunkpos1362= G->thunkpos;  if (!yy_Newline(G)) { goto l1362; }  goto l1358;
  l1362:;	  G->pos= yypos1362; G->thunkpos= yythunkpos1362;
  }  if (!yymatchDot(G)) goto l1358;  goto l1357;
  l1358:;	  G->pos= yypos1358; G->thunkpos= yythunkpos1358;
  }  if (!yymatchChar(G, '$')) goto l1355;  yyText(G, G->begin, G->end);  if (!( IEP_POST )) goto l1355;
  yyprintf((stderr, "  ok   %s @ %s\n", "InlineEquationMultiple", G->buf+G->pos));
  return 1;
  l1355:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "InlineEquationMultiple", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_InlineEquationSingle(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "InlineEquationSingle"));
  {  int yypos1364= G->pos, yythunkpos1364= G->thunkpos;  if (!yymatchChar(G, '$')) goto l1364;  goto l1363;
  l1364:;	  G->pos= yypos1364; G->thunkpos= yythunkpos1364;
  }
  {  int yypos1365= G->pos, yythunkpos1365= G->thunkpos;  if (!yymatchChar(G, '\\')) goto l1365;  goto l1363;
  l1365:;	  G->pos= yypos1365; G->thunkpos= yythunkpos1365;
  }  if (!yy_Nonspacechar(G)) { goto l1363; }  if (!yymatchChar(G, '$')) goto l1363;
  yyprintf((stderr, "  ok   %s @ %s\n", "InlineEquationSingle", G->buf+G->pos));
  return 1;
  l1363:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "InlineEquationSingle", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_InlineEquation(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "InlineEquation"));  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_MATH) )) goto l1366;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1366;  if (!yymatchChar(G, '$')) goto l1366;  yyText(G, G->begin, G->end);  if (!( IEP_PRE )) goto l1366;
  {  int yypos1367= G->pos, yythunkpos1367= G->thunkpos;  if (!yy_InlineEquationSingle(G)) { goto l1368; }  goto l1367;
  l1368:;	  G->pos= yypos1367; G->thunkpos= yythunkpos1367;  if (!yy_InlineEquationMultiple(G)) { goto l1366; }
  }
  l1367:;	
  {  int yypos1369= G->pos, yythunkpos1369= G->thunkpos;  if (!yymatchClass(G, (unsigned char *)"\000\000\000\000\000\000\377\003\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000")) goto l1369;  goto l1366;
  l1369:;	  G->pos= yypos1369; G->thunkpos= yythunkpos1369;
  }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1366;  yyDo(G, yy_1_InlineEquation, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "InlineEquation", G->buf+G->pos));
  return 1;
  l1366:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "InlineEquation", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Nonspacechar(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Nonspacechar"));
  {  int yypos1371= G->pos, yythunkpos1371= G->thunkpos;  if (!yy_Spacechar(G)) { goto l1371; }  goto l1370;
  l1371:;	  G->pos= yypos1371; G->thunkpos= yythunkpos1371;
  }
  {  int yypos1372= G->pos, yythunkpos1372= G->thunkpos;  if (!yy_Newline(G)) { goto l1372; }  goto l1370;
  l1372:;	  G->pos= yypos1372; G->thunkpos= yythunkpos1372;
  }  if (!yymatchDot(G)) goto l1370;
  yyprintf((stderr, "  ok   %s @ %s\n", "Nonspacechar", G->buf+G->pos));
  return 1;
  l1370:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Nonspacechar", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_DisplayFormulaRawMark(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "DisplayFormulaRawMark"));
  {  int yypos1376= G->pos, yythunkpos1376= G->thunkpos;  if (!yymatchChar(G, '{')) goto l1376;  goto l1373;
  l1376:;	  G->pos= yypos1376; G->thunkpos= yythunkpos1376;
  }
  {  int yypos1377= G->pos, yythunkpos1377= G->thunkpos;  if (!yymatchChar(G, '}')) goto l1377;  goto l1373;
  l1377:;	  G->pos= yypos1377; G->thunkpos= yythunkpos1377;
  }  if (!yy_Nonspacechar(G)) { goto l1373; }
  l1374:;	
  {  int yypos1375= G->pos, yythunkpos1375= G->thunkpos;
  {  int yypos1378= G->pos, yythunkpos1378= G->thunkpos;  if (!yymatchChar(G, '{')) goto l1378;  goto l1375;
  l1378:;	  G->pos= yypos1378; G->thunkpos= yythunkpos1378;
  }
  {  int yypos1379= G->pos, yythunkpos1379= G->thunkpos;  if (!yymatchChar(G, '}')) goto l1379;  goto l1375;
  l1379:;	  G->pos= yypos1379; G->thunkpos= yythunkpos1379;
  }  if (!yy_Nonspacechar(G)) { goto l1375; }  goto l1374;
  l1375:;	  G->pos= yypos1375; G->thunkpos= yythunkpos1375;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "DisplayFormulaRawMark", G->buf+G->pos));
  return 1;
  l1373:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "DisplayFormulaRawMark", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_DisplayFormulaRawEnd(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "DisplayFormulaRawEnd"));  if (!yymatchString(G, "\\end")) goto l1380;  if (!yymatchChar(G, '{')) goto l1380;  if (!yy_DisplayFormulaRawMark(G)) { goto l1380; }  if (!yymatchChar(G, '}')) goto l1380;
  yyprintf((stderr, "  ok   %s @ %s\n", "DisplayFormulaRawEnd", G->buf+G->pos));
  return 1;
  l1380:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "DisplayFormulaRawEnd", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Spnl(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Spnl"));  if (!yy_Sp(G)) { goto l1381; }
  {  int yypos1382= G->pos, yythunkpos1382= G->thunkpos;  if (!yy_Newline(G)) { goto l1382; }  if (!yy_Sp(G)) { goto l1382; }  goto l1383;
  l1382:;	  G->pos= yypos1382; G->thunkpos= yythunkpos1382;
  }
  l1383:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Spnl", G->buf+G->pos));
  return 1;
  l1381:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Spnl", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_DisplayFormulaRawStart(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "DisplayFormulaRawStart"));  if (!yymatchString(G, "\\begin")) goto l1384;  if (!yymatchChar(G, '{')) goto l1384;  if (!yy_DisplayFormulaRawMark(G)) { goto l1384; }  if (!yymatchChar(G, '}')) goto l1384;
  yyprintf((stderr, "  ok   %s @ %s\n", "DisplayFormulaRawStart", G->buf+G->pos));
  return 1;
  l1384:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "DisplayFormulaRawStart", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_FormulaNumber(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "FormulaNumber"));  if (!yymatchChar(G, '(')) goto l1385;
  {  int yypos1388= G->pos, yythunkpos1388= G->thunkpos;  if (!yymatchChar(G, ')')) goto l1388;  goto l1385;
  l1388:;	  G->pos= yypos1388; G->thunkpos= yythunkpos1388;
  }
  {  int yypos1389= G->pos, yythunkpos1389= G->thunkpos;  if (!yymatchChar(G, '$')) goto l1389;  goto l1385;
  l1389:;	  G->pos= yypos1389; G->thunkpos= yythunkpos1389;
  }
  {  int yypos1390= G->pos, yythunkpos1390= G->thunkpos;  if (!yy_Newline(G)) { goto l1390; }  goto l1385;
  l1390:;	  G->pos= yypos1390; G->thunkpos= yythunkpos1390;
  }  if (!yymatchDot(G)) goto l1385;
  l1386:;	
  {  int yypos1387= G->pos, yythunkpos1387= G->thunkpos;
  {  int yypos1391= G->pos, yythunkpos1391= G->thunkpos;  if (!yymatchChar(G, ')')) goto l1391;  goto l1387;
  l1391:;	  G->pos= yypos1391; G->thunkpos= yythunkpos1391;
  }
  {  int yypos1392= G->pos, yythunkpos1392= G->thunkpos;  if (!yymatchChar(G, '$')) goto l1392;  goto l1387;
  l1392:;	  G->pos= yypos1392; G->thunkpos= yythunkpos1392;
  }
  {  int yypos1393= G->pos, yythunkpos1393= G->thunkpos;  if (!yy_Newline(G)) { goto l1393; }  goto l1387;
  l1393:;	  G->pos= yypos1393; G->thunkpos= yythunkpos1393;
  }  if (!yymatchDot(G)) goto l1387;  goto l1386;
  l1387:;	  G->pos= yypos1387; G->thunkpos= yythunkpos1387;
  }  if (!yymatchChar(G, ')')) goto l1385;
  yyprintf((stderr, "  ok   %s @ %s\n", "FormulaNumber", G->buf+G->pos));
  return 1;
  l1385:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "FormulaNumber", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_DisplayFormulaRaw(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "DisplayFormulaRaw"));  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_MATH_RAW) )) goto l1394;  if (!yy_NonindentSpace(G)) { goto l1394; }  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1394;  if (!yy_DisplayFormulaRawStart(G)) { goto l1394; }  yyText(G, G->begin, G->end);  if (!( start_dfr() )) goto l1394;  if (!yy_Spnl(G)) { goto l1394; }
  l1395:;	
  {  int yypos1396= G->pos, yythunkpos1396= G->thunkpos;
  {  int yypos1397= G->pos, yythunkpos1397= G->thunkpos;  if (!yy_DisplayFormulaRawStart(G)) { goto l1398; }  yyText(G, G->begin, G->end);  if (!( inc_dfr() )) goto l1398;  goto l1397;
  l1398:;	  G->pos= yypos1397; G->thunkpos= yythunkpos1397;  yyText(G, G->begin, G->end);  if (!( nested_dfr() )) goto l1399;  if (!yy_DisplayFormulaRawEnd(G)) { goto l1399; }  yyText(G, G->begin, G->end);  if (!( dec_dfr() )) goto l1399;  goto l1397;
  l1399:;	  G->pos= yypos1397; G->thunkpos= yythunkpos1397;
  {  int yypos1400= G->pos, yythunkpos1400= G->thunkpos;  if (!yy_DisplayFormulaRawEnd(G)) { goto l1400; }  goto l1396;
  l1400:;	  G->pos= yypos1400; G->thunkpos= yythunkpos1400;
  }
  {  int yypos1401= G->pos, yythunkpos1401= G->thunkpos;  if (!yy_DisplayFormulaRawStart(G)) { goto l1401; }  goto l1396;
  l1401:;	  G->pos= yypos1401; G->thunkpos= yythunkpos1401;
  }
  {  int yypos1402= G->pos, yythunkpos1402= G->thunkpos;  if (!yy_Newline(G)) { goto l1402; }  if (!yy_Newline(G)) { goto l1402; }
  l1403:;	
  {  int yypos1404= G->pos, yythunkpos1404= G->thunkpos;  if (!yy_Newline(G)) { goto l1404; }  goto l1403;
  l1404:;	  G->pos= yypos1404; G->thunkpos= yythunkpos1404;
  }  goto l1396;
  l1402:;	  G->pos= yypos1402; G->thunkpos= yythunkpos1402;
  }  if (!yymatchDot(G)) goto l1396;
  }
  l1397:;	  goto l1395;
  l1396:;	  G->pos= yypos1396; G->thunkpos= yythunkpos1396;
  }  yyText(G, G->begin, G->end);  if (!( !nested_dfr() )) goto l1394;  if (!yy_DisplayFormulaRawEnd(G)) { goto l1394; }  yyText(G, G->begin, G->end);  if (!( dec_dfr() )) goto l1394;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1394;  if (!yy_Sp(G)) { goto l1394; }  if (!yy_Newline(G)) { goto l1394; }  yyDo(G, yy_1_DisplayFormulaRaw, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "DisplayFormulaRaw", G->buf+G->pos));
  return 1;
  l1394:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "DisplayFormulaRaw", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_DisplayFormulaDollar(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "DisplayFormulaDollar"));  if (!yy_NonindentSpace(G)) { goto l1405; }  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1405;  if (!yymatchString(G, "$$")) goto l1405;
  l1406:;	
  {  int yypos1407= G->pos, yythunkpos1407= G->thunkpos;
  {  int yypos1408= G->pos, yythunkpos1408= G->thunkpos;  if (!yymatchString(G, "$$")) goto l1408;  goto l1407;
  l1408:;	  G->pos= yypos1408; G->thunkpos= yythunkpos1408;
  }  if (!yymatchDot(G)) goto l1407;  goto l1406;
  l1407:;	  G->pos= yypos1407; G->thunkpos= yythunkpos1407;
  }  if (!yymatchString(G, "$$")) goto l1405;  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1405;  if (!yy_Sp(G)) { goto l1405; }
  {  int yypos1409= G->pos, yythunkpos1409= G->thunkpos;  if (!yy_FormulaNumber(G)) { goto l1409; }  goto l1410;
  l1409:;	  G->pos= yypos1409; G->thunkpos= yythunkpos1409;
  }
  l1410:;	  if (!yy_Sp(G)) { goto l1405; }  if (!yy_Newline(G)) { goto l1405; }  yyDo(G, yy_1_DisplayFormulaDollar, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "DisplayFormulaDollar", G->buf+G->pos));
  return 1;
  l1405:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "DisplayFormulaDollar", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_FencedCodeBlockTidle(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "FencedCodeBlockTidle"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1411;  if (!yy_FencedCodeBlockStartTidleLine(G)) { goto l1411; }
  l1412:;	
  {  int yypos1413= G->pos, yythunkpos1413= G->thunkpos;  if (!yy_FencedCodeBlockChunkTidle(G)) { goto l1413; }  goto l1412;
  l1413:;	  G->pos= yypos1413; G->thunkpos= yythunkpos1413;
  }  if (!yy_FencedCodeBlockEndTidle(G)) { goto l1411; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1411;  yyDo(G, yy_1_FencedCodeBlockTidle, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "FencedCodeBlockTidle", G->buf+G->pos));
  return 1;
  l1411:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "FencedCodeBlockTidle", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_FencedCodeBlockTick(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "FencedCodeBlockTick"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1414;  if (!yy_FencedCodeBlockStartTickLine(G)) { goto l1414; }
  l1415:;	
  {  int yypos1416= G->pos, yythunkpos1416= G->thunkpos;  if (!yy_FencedCodeBlockChunkTick(G)) { goto l1416; }  goto l1415;
  l1416:;	  G->pos= yypos1416; G->thunkpos= yythunkpos1416;
  }  if (!yy_FencedCodeBlockEndTick(G)) { goto l1414; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1414;  yyDo(G, yy_1_FencedCodeBlockTick, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "FencedCodeBlockTick", G->buf+G->pos));
  return 1;
  l1414:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "FencedCodeBlockTick", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_FencedCodeBlockEndTidle(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "FencedCodeBlockEndTidle"));  if (!yymatchString(G, "~~~")) goto l1417;
  l1418:;	
  {  int yypos1419= G->pos, yythunkpos1419= G->thunkpos;  if (!yy_Spacechar(G)) { goto l1419; }  goto l1418;
  l1419:;	  G->pos= yypos1419; G->thunkpos= yythunkpos1419;
  }  if (!yy_Newline(G)) { goto l1417; }
  yyprintf((stderr, "  ok   %s @ %s\n", "FencedCodeBlockEndTidle", G->buf+G->pos));
  return 1;
  l1417:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "FencedCodeBlockEndTidle", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_FencedCodeBlockChunkTidle(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "FencedCodeBlockChunkTidle"));
  {  int yypos1421= G->pos, yythunkpos1421= G->thunkpos;  if (!yy_FencedCodeBlockEndTidle(G)) { goto l1421; }  goto l1420;
  l1421:;	  G->pos= yypos1421; G->thunkpos= yythunkpos1421;
  }
  l1422:;	
  {  int yypos1423= G->pos, yythunkpos1423= G->thunkpos;
  {  int yypos1424= G->pos, yythunkpos1424= G->thunkpos;  if (!yy_Newline(G)) { goto l1424; }  goto l1423;
  l1424:;	  G->pos= yypos1424; G->thunkpos= yythunkpos1424;
  }  if (!yymatchDot(G)) goto l1423;  goto l1422;
  l1423:;	  G->pos= yypos1423; G->thunkpos= yythunkpos1423;
  }  if (!yy_Newline(G)) { goto l1420; }
  yyprintf((stderr, "  ok   %s @ %s\n", "FencedCodeBlockChunkTidle", G->buf+G->pos));
  return 1;
  l1420:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "FencedCodeBlockChunkTidle", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_FencedCodeBlockStartTidleLine(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "FencedCodeBlockStartTidleLine"));  if (!yy_FencedCodeBlockStartTidle(G)) { goto l1425; }  if (!yy_Newline(G)) { goto l1425; }
  yyprintf((stderr, "  ok   %s @ %s\n", "FencedCodeBlockStartTidleLine", G->buf+G->pos));
  return 1;
  l1425:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "FencedCodeBlockStartTidleLine", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_FencedCodeBlockStartTidle(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "FencedCodeBlockStartTidle"));  if (!yymatchString(G, "~~~")) goto l1426;
  l1427:;	
  {  int yypos1428= G->pos, yythunkpos1428= G->thunkpos;
  {  int yypos1429= G->pos, yythunkpos1429= G->thunkpos;  if (!yy_Newline(G)) { goto l1429; }  goto l1428;
  l1429:;	  G->pos= yypos1429; G->thunkpos= yythunkpos1429;
  }
  {  int yypos1430= G->pos, yythunkpos1430= G->thunkpos;  if (!yymatchChar(G, '~')) goto l1430;  goto l1428;
  l1430:;	  G->pos= yypos1430; G->thunkpos= yythunkpos1430;
  }  if (!yymatchDot(G)) goto l1428;  goto l1427;
  l1428:;	  G->pos= yypos1428; G->thunkpos= yythunkpos1428;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "FencedCodeBlockStartTidle", G->buf+G->pos));
  return 1;
  l1426:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "FencedCodeBlockStartTidle", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Spacechar(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Spacechar"));
  {  int yypos1432= G->pos, yythunkpos1432= G->thunkpos;  if (!yymatchChar(G, ' ')) goto l1433;  goto l1432;
  l1433:;	  G->pos= yypos1432; G->thunkpos= yythunkpos1432;  if (!yymatchChar(G, '\t')) goto l1431;
  }
  l1432:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Spacechar", G->buf+G->pos));
  return 1;
  l1431:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Spacechar", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_FencedCodeBlockEndTick(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "FencedCodeBlockEndTick"));  if (!yymatchString(G, "```")) goto l1434;
  l1435:;	
  {  int yypos1436= G->pos, yythunkpos1436= G->thunkpos;  if (!yy_Spacechar(G)) { goto l1436; }  goto l1435;
  l1436:;	  G->pos= yypos1436; G->thunkpos= yythunkpos1436;
  }  if (!yy_Newline(G)) { goto l1434; }
  yyprintf((stderr, "  ok   %s @ %s\n", "FencedCodeBlockEndTick", G->buf+G->pos));
  return 1;
  l1434:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "FencedCodeBlockEndTick", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_FencedCodeBlockChunkTick(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "FencedCodeBlockChunkTick"));
  {  int yypos1438= G->pos, yythunkpos1438= G->thunkpos;  if (!yy_FencedCodeBlockEndTick(G)) { goto l1438; }  goto l1437;
  l1438:;	  G->pos= yypos1438; G->thunkpos= yythunkpos1438;
  }
  l1439:;	
  {  int yypos1440= G->pos, yythunkpos1440= G->thunkpos;
  {  int yypos1441= G->pos, yythunkpos1441= G->thunkpos;  if (!yy_Newline(G)) { goto l1441; }  goto l1440;
  l1441:;	  G->pos= yypos1441; G->thunkpos= yythunkpos1441;
  }  if (!yymatchDot(G)) goto l1440;  goto l1439;
  l1440:;	  G->pos= yypos1440; G->thunkpos= yythunkpos1440;
  }  if (!yy_Newline(G)) { goto l1437; }
  yyprintf((stderr, "  ok   %s @ %s\n", "FencedCodeBlockChunkTick", G->buf+G->pos));
  return 1;
  l1437:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "FencedCodeBlockChunkTick", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_FencedCodeBlockStartTickLine(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "FencedCodeBlockStartTickLine"));  if (!yy_FencedCodeBlockStartTick(G)) { goto l1442; }  if (!yy_Newline(G)) { goto l1442; }
  yyprintf((stderr, "  ok   %s @ %s\n", "FencedCodeBlockStartTickLine", G->buf+G->pos));
  return 1;
  l1442:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "FencedCodeBlockStartTickLine", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_FencedCodeBlockStartTick(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "FencedCodeBlockStartTick"));  if (!yymatchString(G, "```")) goto l1443;
  l1444:;	
  {  int yypos1445= G->pos, yythunkpos1445= G->thunkpos;
  {  int yypos1446= G->pos, yythunkpos1446= G->thunkpos;  if (!yy_Newline(G)) { goto l1446; }  goto l1445;
  l1446:;	  G->pos= yypos1446; G->thunkpos= yythunkpos1446;
  }
  {  int yypos1447= G->pos, yythunkpos1447= G->thunkpos;  if (!yymatchChar(G, '`')) goto l1447;  goto l1445;
  l1447:;	  G->pos= yypos1447; G->thunkpos= yythunkpos1447;
  }  if (!yymatchDot(G)) goto l1445;  goto l1444;
  l1445:;	  G->pos= yypos1445; G->thunkpos= yythunkpos1445;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "FencedCodeBlockStartTick", G->buf+G->pos));
  return 1;
  l1443:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "FencedCodeBlockStartTick", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_VerbatimChunk(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "VerbatimChunk"));
  l1449:;	
  {  int yypos1450= G->pos, yythunkpos1450= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1450; }  goto l1449;
  l1450:;	  G->pos= yypos1450; G->thunkpos= yythunkpos1450;
  }  if (!yy_NonblankIndentedLine(G)) { goto l1448; }
  l1451:;	
  {  int yypos1452= G->pos, yythunkpos1452= G->thunkpos;  if (!yy_NonblankIndentedLine(G)) { goto l1452; }  goto l1451;
  l1452:;	  G->pos= yypos1452; G->thunkpos= yythunkpos1452;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "VerbatimChunk", G->buf+G->pos));
  return 1;
  l1448:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "VerbatimChunk", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_IndentedLine(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "IndentedLine"));  if (!yy_Indent(G)) { goto l1453; }  if (!yy_Line(G)) { goto l1453; }
  yyprintf((stderr, "  ok   %s @ %s\n", "IndentedLine", G->buf+G->pos));
  return 1;
  l1453:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "IndentedLine", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_NonblankIndentedLine(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "NonblankIndentedLine"));
  {  int yypos1455= G->pos, yythunkpos1455= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1455; }  goto l1454;
  l1455:;	  G->pos= yypos1455; G->thunkpos= yythunkpos1455;
  }  if (!yy_IndentedLine(G)) { goto l1454; }
  yyprintf((stderr, "  ok   %s @ %s\n", "NonblankIndentedLine", G->buf+G->pos));
  return 1;
  l1454:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "NonblankIndentedLine", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Line(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Line"));  if (!yy_RawLine(G)) { goto l1456; }  yyDo(G, yy_1_Line, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Line", G->buf+G->pos));
  return 1;
  l1456:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Line", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_StartList(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "StartList"));
  {  int yypos1458= G->pos, yythunkpos1458= G->thunkpos;  if (!yymatchDot(G)) goto l1457;  G->pos= yypos1458; G->thunkpos= yythunkpos1458;
  }  yyDo(G, yy_1_StartList, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "StartList", G->buf+G->pos));
  return 1;
  l1457:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "StartList", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_BlockQuoteRaw(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "BlockQuoteRaw"));  if (!yy_StartList(G)) { goto l1459; }  yyDo(G, yySet, -1, 0);  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1459;  if (!yymatchChar(G, '>')) goto l1459;
  {  int yypos1462= G->pos, yythunkpos1462= G->thunkpos;  if (!yymatchChar(G, ' ')) goto l1462;  goto l1463;
  l1462:;	  G->pos= yypos1462; G->thunkpos= yythunkpos1462;
  }
  l1463:;	  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1459;  yyDo(G, yy_1_BlockQuoteRaw, G->begin, G->end);  if (!yy_Line(G)) { goto l1459; }  yyDo(G, yy_2_BlockQuoteRaw, G->begin, G->end);
  l1464:;	
  {  int yypos1465= G->pos, yythunkpos1465= G->thunkpos;
  {  int yypos1466= G->pos, yythunkpos1466= G->thunkpos;  if (!yymatchChar(G, '>')) goto l1466;  goto l1465;
  l1466:;	  G->pos= yypos1466; G->thunkpos= yythunkpos1466;
  }
  {  int yypos1467= G->pos, yythunkpos1467= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1467; }  goto l1465;
  l1467:;	  G->pos= yypos1467; G->thunkpos= yythunkpos1467;
  }  if (!yy_Line(G)) { goto l1465; }  yyDo(G, yy_3_BlockQuoteRaw, G->begin, G->end);  goto l1464;
  l1465:;	  G->pos= yypos1465; G->thunkpos= yythunkpos1465;
  }
  l1468:;	
  {  int yypos1469= G->pos, yythunkpos1469= G->thunkpos;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1469;  if (!yy_BlankLine(G)) { goto l1469; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1469;  yyDo(G, yy_4_BlockQuoteRaw, G->begin, G->end);  goto l1468;
  l1469:;	  G->pos= yypos1469; G->thunkpos= yythunkpos1469;
  }
  l1460:;	
  {  int yypos1461= G->pos, yythunkpos1461= G->thunkpos;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1461;  if (!yymatchChar(G, '>')) goto l1461;
  {  int yypos1470= G->pos, yythunkpos1470= G->thunkpos;  if (!yymatchChar(G, ' ')) goto l1470;  goto l1471;
  l1470:;	  G->pos= yypos1470; G->thunkpos= yythunkpos1470;
  }
  l1471:;	  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1461;  yyDo(G, yy_1_BlockQuoteRaw, G->begin, G->end);  if (!yy_Line(G)) { goto l1461; }  yyDo(G, yy_2_BlockQuoteRaw, G->begin, G->end);
  l1472:;	
  {  int yypos1473= G->pos, yythunkpos1473= G->thunkpos;
  {  int yypos1474= G->pos, yythunkpos1474= G->thunkpos;  if (!yymatchChar(G, '>')) goto l1474;  goto l1473;
  l1474:;	  G->pos= yypos1474; G->thunkpos= yythunkpos1474;
  }
  {  int yypos1475= G->pos, yythunkpos1475= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1475; }  goto l1473;
  l1475:;	  G->pos= yypos1475; G->thunkpos= yythunkpos1475;
  }  if (!yy_Line(G)) { goto l1473; }  yyDo(G, yy_3_BlockQuoteRaw, G->begin, G->end);  goto l1472;
  l1473:;	  G->pos= yypos1473; G->thunkpos= yythunkpos1473;
  }
  l1476:;	
  {  int yypos1477= G->pos, yythunkpos1477= G->thunkpos;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1477;  if (!yy_BlankLine(G)) { goto l1477; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1477;  yyDo(G, yy_4_BlockQuoteRaw, G->begin, G->end);  goto l1476;
  l1477:;	  G->pos= yypos1477; G->thunkpos= yythunkpos1477;
  }  goto l1460;
  l1461:;	  G->pos= yypos1461; G->thunkpos= yythunkpos1461;
  }  yyDo(G, yy_5_BlockQuoteRaw, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "BlockQuoteRaw", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1459:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "BlockQuoteRaw", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Endline(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Endline"));
  {  int yypos1479= G->pos, yythunkpos1479= G->thunkpos;  if (!yy_LineBreak(G)) { goto l1480; }  goto l1479;
  l1480:;	  G->pos= yypos1479; G->thunkpos= yythunkpos1479;  if (!yy_TerminalEndline(G)) { goto l1481; }  goto l1479;
  l1481:;	  G->pos= yypos1479; G->thunkpos= yythunkpos1479;  if (!yy_NormalEndline(G)) { goto l1478; }
  }
  l1479:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Endline", G->buf+G->pos));
  return 1;
  l1478:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Endline", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_RawLine(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "RawLine"));
  {  int yypos1483= G->pos, yythunkpos1483= G->thunkpos;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1484;
  l1485:;	
  {  int yypos1486= G->pos, yythunkpos1486= G->thunkpos;
  {  int yypos1487= G->pos, yythunkpos1487= G->thunkpos;  if (!yymatchChar(G, '\r')) goto l1487;  goto l1486;
  l1487:;	  G->pos= yypos1487; G->thunkpos= yythunkpos1487;
  }
  {  int yypos1488= G->pos, yythunkpos1488= G->thunkpos;  if (!yymatchChar(G, '\n')) goto l1488;  goto l1486;
  l1488:;	  G->pos= yypos1488; G->thunkpos= yythunkpos1488;
  }  if (!yymatchDot(G)) goto l1486;  goto l1485;
  l1486:;	  G->pos= yypos1486; G->thunkpos= yythunkpos1486;
  }  if (!yy_Newline(G)) { goto l1484; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1484;  goto l1483;
  l1484:;	  G->pos= yypos1483; G->thunkpos= yythunkpos1483;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1482;  if (!yymatchDot(G)) goto l1482;
  l1489:;	
  {  int yypos1490= G->pos, yythunkpos1490= G->thunkpos;  if (!yymatchDot(G)) goto l1490;  goto l1489;
  l1490:;	  G->pos= yypos1490; G->thunkpos= yythunkpos1490;
  }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1482;  if (!yy_Eof(G)) { goto l1482; }
  }
  l1483:;	  yyDo(G, yy_1_RawLine, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "RawLine", G->buf+G->pos));
  return 1;
  l1482:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "RawLine", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_SetextBottom2(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "SetextBottom2"));  if (!yymatchChar(G, '-')) goto l1491;
  l1492:;	
  {  int yypos1493= G->pos, yythunkpos1493= G->thunkpos;  if (!yymatchChar(G, '-')) goto l1493;  goto l1492;
  l1493:;	  G->pos= yypos1493; G->thunkpos= yythunkpos1493;
  }  if (!yy_Newline(G)) { goto l1491; }
  yyprintf((stderr, "  ok   %s @ %s\n", "SetextBottom2", G->buf+G->pos));
  return 1;
  l1491:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "SetextBottom2", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_SetextBottom1(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "SetextBottom1"));  if (!yymatchChar(G, '=')) goto l1494;
  l1495:;	
  {  int yypos1496= G->pos, yythunkpos1496= G->thunkpos;  if (!yymatchChar(G, '=')) goto l1496;  goto l1495;
  l1496:;	  G->pos= yypos1496; G->thunkpos= yythunkpos1496;
  }  if (!yy_Newline(G)) { goto l1494; }
  yyprintf((stderr, "  ok   %s @ %s\n", "SetextBottom1", G->buf+G->pos));
  return 1;
  l1494:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "SetextBottom1", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_SetextHeading2(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "SetextHeading2"));
  {  int yypos1498= G->pos, yythunkpos1498= G->thunkpos;  if (!yy_RawLine(G)) { goto l1497; }  if (!yy_SetextBottom2(G)) { goto l1497; }  G->pos= yypos1498; G->thunkpos= yythunkpos1498;
  }  if (!yy_LocMarker(G)) { goto l1497; }  yyDo(G, yySet, -1, 0);  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1497;
  {  int yypos1501= G->pos, yythunkpos1501= G->thunkpos;  if (!yy_Endline(G)) { goto l1501; }  goto l1497;
  l1501:;	  G->pos= yypos1501; G->thunkpos= yythunkpos1501;
  }  if (!yy_Inline(G)) { goto l1497; }
  l1499:;	
  {  int yypos1500= G->pos, yythunkpos1500= G->thunkpos;
  {  int yypos1502= G->pos, yythunkpos1502= G->thunkpos;  if (!yy_Endline(G)) { goto l1502; }  goto l1500;
  l1502:;	  G->pos= yypos1502; G->thunkpos= yythunkpos1502;
  }  if (!yy_Inline(G)) { goto l1500; }  goto l1499;
  l1500:;	  G->pos= yypos1500; G->thunkpos= yythunkpos1500;
  }  if (!yy_Sp(G)) { goto l1497; }  if (!yy_Newline(G)) { goto l1497; }  if (!yy_SetextBottom2(G)) { goto l1497; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1497;  yyDo(G, yy_1_SetextHeading2, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "SetextHeading2", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1497:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "SetextHeading2", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_SetextHeading1(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "SetextHeading1"));
  {  int yypos1504= G->pos, yythunkpos1504= G->thunkpos;  if (!yy_RawLine(G)) { goto l1503; }  if (!yy_SetextBottom1(G)) { goto l1503; }  G->pos= yypos1504; G->thunkpos= yythunkpos1504;
  }  if (!yy_LocMarker(G)) { goto l1503; }  yyDo(G, yySet, -1, 0);  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1503;
  {  int yypos1507= G->pos, yythunkpos1507= G->thunkpos;  if (!yy_Endline(G)) { goto l1507; }  goto l1503;
  l1507:;	  G->pos= yypos1507; G->thunkpos= yythunkpos1507;
  }  if (!yy_Inline(G)) { goto l1503; }
  l1505:;	
  {  int yypos1506= G->pos, yythunkpos1506= G->thunkpos;
  {  int yypos1508= G->pos, yythunkpos1508= G->thunkpos;  if (!yy_Endline(G)) { goto l1508; }  goto l1506;
  l1508:;	  G->pos= yypos1508; G->thunkpos= yythunkpos1508;
  }  if (!yy_Inline(G)) { goto l1506; }  goto l1505;
  l1506:;	  G->pos= yypos1506; G->thunkpos= yythunkpos1506;
  }  if (!yy_Sp(G)) { goto l1503; }  if (!yy_Newline(G)) { goto l1503; }  if (!yy_SetextBottom1(G)) { goto l1503; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1503;  yyDo(G, yy_1_SetextHeading1, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "SetextHeading1", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1503:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "SetextHeading1", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_SetextHeading(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "SetextHeading"));
  {  int yypos1510= G->pos, yythunkpos1510= G->thunkpos;  if (!yy_SetextHeading1(G)) { goto l1511; }  goto l1510;
  l1511:;	  G->pos= yypos1510; G->thunkpos= yythunkpos1510;  if (!yy_SetextHeading2(G)) { goto l1509; }
  }
  l1510:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "SetextHeading", G->buf+G->pos));
  return 1;
  l1509:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "SetextHeading", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Space(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Space"));  if (!yy_Spacechar(G)) { goto l1512; }
  l1513:;	
  {  int yypos1514= G->pos, yythunkpos1514= G->thunkpos;  if (!yy_Spacechar(G)) { goto l1514; }  goto l1513;
  l1514:;	  G->pos= yypos1514; G->thunkpos= yythunkpos1514;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "Space", G->buf+G->pos));
  return 1;
  l1512:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Space", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_AtxHeading(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "AtxHeading"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1515;  if (!yy_AtxStart(G)) { goto l1515; }  yyDo(G, yySet, -1, 0);  if (!yy_Space(G)) { goto l1515; }  if (!yy_AtxInline(G)) { goto l1515; }
  l1516:;	
  {  int yypos1517= G->pos, yythunkpos1517= G->thunkpos;  if (!yy_AtxInline(G)) { goto l1517; }  goto l1516;
  l1517:;	  G->pos= yypos1517; G->thunkpos= yythunkpos1517;
  }
  {  int yypos1518= G->pos, yythunkpos1518= G->thunkpos;  if (!yy_Sp(G)) { goto l1518; }
  l1520:;	
  {  int yypos1521= G->pos, yythunkpos1521= G->thunkpos;  if (!yymatchChar(G, '#')) goto l1521;  goto l1520;
  l1521:;	  G->pos= yypos1521; G->thunkpos= yythunkpos1521;
  }  if (!yy_Sp(G)) { goto l1518; }  goto l1519;
  l1518:;	  G->pos= yypos1518; G->thunkpos= yythunkpos1518;
  }
  l1519:;	  if (!yy_Newline(G)) { goto l1515; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1515;  yyDo(G, yy_1_AtxHeading, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "AtxHeading", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1515:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "AtxHeading", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_AtxStart(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "AtxStart"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1522;
  {  int yypos1523= G->pos, yythunkpos1523= G->thunkpos;  if (!yymatchString(G, "######")) goto l1524;  goto l1523;
  l1524:;	  G->pos= yypos1523; G->thunkpos= yythunkpos1523;  if (!yymatchString(G, "#####")) goto l1525;  goto l1523;
  l1525:;	  G->pos= yypos1523; G->thunkpos= yythunkpos1523;  if (!yymatchString(G, "####")) goto l1526;  goto l1523;
  l1526:;	  G->pos= yypos1523; G->thunkpos= yythunkpos1523;  if (!yymatchString(G, "###")) goto l1527;  goto l1523;
  l1527:;	  G->pos= yypos1523; G->thunkpos= yythunkpos1523;  if (!yymatchString(G, "##")) goto l1528;  goto l1523;
  l1528:;	  G->pos= yypos1523; G->thunkpos= yythunkpos1523;  if (!yymatchChar(G, '#')) goto l1522;
  }
  l1523:;	  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1522;  yyDo(G, yy_1_AtxStart, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "AtxStart", G->buf+G->pos));
  return 1;
  l1522:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "AtxStart", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Inline(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Inline"));
  {  int yypos1530= G->pos, yythunkpos1530= G->thunkpos;  if (!yy_Str(G)) { goto l1531; }  goto l1530;
  l1531:;	  G->pos= yypos1530; G->thunkpos= yythunkpos1530;  if (!yy_Endline(G)) { goto l1532; }  goto l1530;
  l1532:;	  G->pos= yypos1530; G->thunkpos= yythunkpos1530;  if (!yy_UlOrStarLine(G)) { goto l1533; }  goto l1530;
  l1533:;	  G->pos= yypos1530; G->thunkpos= yythunkpos1530;  if (!yy_Space(G)) { goto l1534; }  goto l1530;
  l1534:;	  G->pos= yypos1530; G->thunkpos= yythunkpos1530;  if (!yy_Strong(G)) { goto l1535; }  goto l1530;
  l1535:;	  G->pos= yypos1530; G->thunkpos= yythunkpos1530;  if (!yy_Emph(G)) { goto l1536; }  goto l1530;
  l1536:;	  G->pos= yypos1530; G->thunkpos= yythunkpos1530;  if (!yy_Strike(G)) { goto l1537; }  goto l1530;
  l1537:;	  G->pos= yypos1530; G->thunkpos= yythunkpos1530;  if (!yy_Image(G)) { goto l1538; }  goto l1530;
  l1538:;	  G->pos= yypos1530; G->thunkpos= yythunkpos1530;  if (!yy_Link(G)) { goto l1539; }  goto l1530;
  l1539:;	  G->pos= yypos1530; G->thunkpos= yythunkpos1530;  if (!yy_NoteReference(G)) { goto l1540; }  goto l1530;
  l1540:;	  G->pos= yypos1530; G->thunkpos= yythunkpos1530;  if (!yy_InlineNote(G)) { goto l1541; }  goto l1530;
  l1541:;	  G->pos= yypos1530; G->thunkpos= yythunkpos1530;  if (!yy_Code(G)) { goto l1542; }  goto l1530;
  l1542:;	  G->pos= yypos1530; G->thunkpos= yythunkpos1530;  if (!yy_InlineEquation(G)) { goto l1543; }  goto l1530;
  l1543:;	  G->pos= yypos1530; G->thunkpos= yythunkpos1530;  if (!yy_Mark(G)) { goto l1544; }  goto l1530;
  l1544:;	  G->pos= yypos1530; G->thunkpos= yythunkpos1530;  if (!yy_RawHtml(G)) { goto l1545; }  goto l1530;
  l1545:;	  G->pos= yypos1530; G->thunkpos= yythunkpos1530;  if (!yy_Entity(G)) { goto l1546; }  goto l1530;
  l1546:;	  G->pos= yypos1530; G->thunkpos= yythunkpos1530;  if (!yy_EscapedChar(G)) { goto l1547; }  goto l1530;
  l1547:;	  G->pos= yypos1530; G->thunkpos= yythunkpos1530;  if (!yy_Symbol(G)) { goto l1529; }
  }
  l1530:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Inline", G->buf+G->pos));
  return 1;
  l1529:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Inline", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Sp(GREG *G)
{
  yyprintf((stderr, "%s\n", "Sp"));
  l1549:;	
  {  int yypos1550= G->pos, yythunkpos1550= G->thunkpos;  if (!yy_Spacechar(G)) { goto l1550; }  goto l1549;
  l1550:;	  G->pos= yypos1550; G->thunkpos= yythunkpos1550;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "Sp", G->buf+G->pos));
  return 1;
}
YY_RULE(int) yy_AtxInline(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "AtxInline"));
  {  int yypos1552= G->pos, yythunkpos1552= G->thunkpos;  if (!yy_Newline(G)) { goto l1552; }  goto l1551;
  l1552:;	  G->pos= yypos1552; G->thunkpos= yythunkpos1552;
  }
  {  int yypos1553= G->pos, yythunkpos1553= G->thunkpos;  if (!yy_Sp(G)) { goto l1553; }
  l1554:;	
  {  int yypos1555= G->pos, yythunkpos1555= G->thunkpos;  if (!yymatchChar(G, '#')) goto l1555;  goto l1554;
  l1555:;	  G->pos= yypos1555; G->thunkpos= yythunkpos1555;
  }  if (!yy_Sp(G)) { goto l1553; }  if (!yy_Newline(G)) { goto l1553; }  goto l1551;
  l1553:;	  G->pos= yypos1553; G->thunkpos= yythunkpos1553;
  }  if (!yy_Inline(G)) { goto l1551; }
  yyprintf((stderr, "  ok   %s @ %s\n", "AtxInline", G->buf+G->pos));
  return 1;
  l1551:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "AtxInline", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Inlines(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Inlines"));
  {  int yypos1559= G->pos, yythunkpos1559= G->thunkpos;
  {  int yypos1561= G->pos, yythunkpos1561= G->thunkpos;  if (!yy_Endline(G)) { goto l1561; }  goto l1560;
  l1561:;	  G->pos= yypos1561; G->thunkpos= yythunkpos1561;
  }  if (!yy_Inline(G)) { goto l1560; }  goto l1559;
  l1560:;	  G->pos= yypos1559; G->thunkpos= yythunkpos1559;  if (!yy_Endline(G)) { goto l1556; }
  {  int yypos1562= G->pos, yythunkpos1562= G->thunkpos;  if (!yy_Inline(G)) { goto l1556; }  G->pos= yypos1562; G->thunkpos= yythunkpos1562;
  }
  }
  l1559:;	
  l1557:;	
  {  int yypos1558= G->pos, yythunkpos1558= G->thunkpos;
  {  int yypos1563= G->pos, yythunkpos1563= G->thunkpos;
  {  int yypos1565= G->pos, yythunkpos1565= G->thunkpos;  if (!yy_Endline(G)) { goto l1565; }  goto l1564;
  l1565:;	  G->pos= yypos1565; G->thunkpos= yythunkpos1565;
  }  if (!yy_Inline(G)) { goto l1564; }  goto l1563;
  l1564:;	  G->pos= yypos1563; G->thunkpos= yythunkpos1563;  if (!yy_Endline(G)) { goto l1558; }
  {  int yypos1566= G->pos, yythunkpos1566= G->thunkpos;  if (!yy_Inline(G)) { goto l1558; }  G->pos= yypos1566; G->thunkpos= yythunkpos1566;
  }
  }
  l1563:;	  goto l1557;
  l1558:;	  G->pos= yypos1558; G->thunkpos= yythunkpos1558;
  }
  {  int yypos1567= G->pos, yythunkpos1567= G->thunkpos;  if (!yy_Endline(G)) { goto l1567; }  goto l1568;
  l1567:;	  G->pos= yypos1567; G->thunkpos= yythunkpos1567;
  }
  l1568:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Inlines", G->buf+G->pos));
  return 1;
  l1556:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Inlines", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_NonindentSpace(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "NonindentSpace"));
  {  int yypos1570= G->pos, yythunkpos1570= G->thunkpos;  if (!yymatchString(G, "   ")) goto l1571;  goto l1570;
  l1571:;	  G->pos= yypos1570; G->thunkpos= yythunkpos1570;  if (!yymatchString(G, "  ")) goto l1572;  goto l1570;
  l1572:;	  G->pos= yypos1570; G->thunkpos= yythunkpos1570;  if (!yymatchChar(G, ' ')) goto l1573;  goto l1570;
  l1573:;	  G->pos= yypos1570; G->thunkpos= yythunkpos1570;  if (!yymatchString(G, "")) goto l1569;
  }
  l1570:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "NonindentSpace", G->buf+G->pos));
  return 1;
  l1569:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "NonindentSpace", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Plain(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Plain"));  if (!yy_Inlines(G)) { goto l1574; }
  yyprintf((stderr, "  ok   %s @ %s\n", "Plain", G->buf+G->pos));
  return 1;
  l1574:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Plain", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Para(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Para"));  if (!yy_NonindentSpace(G)) { goto l1575; }  if (!yy_Inlines(G)) { goto l1575; }  if (!yy_BlankLine(G)) { goto l1575; }
  l1576:;	
  {  int yypos1577= G->pos, yythunkpos1577= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1577; }  goto l1576;
  l1577:;	  G->pos= yypos1577; G->thunkpos= yythunkpos1577;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "Para", G->buf+G->pos));
  return 1;
  l1575:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Para", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_StyleBlock(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "StyleBlock"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1578;  if (!yy_LocMarker(G)) { goto l1578; }  yyDo(G, yySet, -1, 0);  if (!yy_InStyleTags(G)) { goto l1578; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1578;
  l1579:;	
  {  int yypos1580= G->pos, yythunkpos1580= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1580; }  goto l1579;
  l1580:;	  G->pos= yypos1580; G->thunkpos= yythunkpos1580;
  }  yyDo(G, yy_1_StyleBlock, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "StyleBlock", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1578:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "StyleBlock", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HtmlBlock(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "HtmlBlock"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1581;  if (!yy_LocMarker(G)) { goto l1581; }  yyDo(G, yySet, -1, 0);
  {  int yypos1582= G->pos, yythunkpos1582= G->thunkpos;  if (!yy_HtmlBlockInTags(G)) { goto l1583; }  goto l1582;
  l1583:;	  G->pos= yypos1582; G->thunkpos= yythunkpos1582;  if (!yy_HtmlComment(G)) { goto l1584; }  goto l1582;
  l1584:;	  G->pos= yypos1582; G->thunkpos= yythunkpos1582;  if (!yy_HtmlBlockSelfClosing(G)) { goto l1581; }
  }
  l1582:;	  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1581;  if (!yy_BlankLine(G)) { goto l1581; }
  l1585:;	
  {  int yypos1586= G->pos, yythunkpos1586= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1586; }  goto l1585;
  l1586:;	  G->pos= yypos1586; G->thunkpos= yythunkpos1586;
  }  yyDo(G, yy_1_HtmlBlock, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "HtmlBlock", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1581:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HtmlBlock", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Table(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "Table"));  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_TABLE) )) goto l1587;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1587;  if (!yy_LocMarker(G)) { goto l1587; }  yyDo(G, yySet, -1, 0);  if (!yy_TableHeader(G)) { goto l1587; }  if (!yy_TableDelimiter(G)) { goto l1587; }
  l1588:;	
  {  int yypos1589= G->pos, yythunkpos1589= G->thunkpos;  if (!yy_TableLine(G)) { goto l1589; }  goto l1588;
  l1589:;	  G->pos= yypos1589; G->thunkpos= yythunkpos1589;
  }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1587;  yyDo(G, yy_1_Table, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Table", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1587:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Table", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_BulletList(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "BulletList"));
  {  int yypos1591= G->pos, yythunkpos1591= G->thunkpos;  if (!yy_Bullet(G)) { goto l1590; }  G->pos= yypos1591; G->thunkpos= yythunkpos1591;
  }
  {  int yypos1592= G->pos, yythunkpos1592= G->thunkpos;  if (!yy_ListTight(G)) { goto l1593; }  goto l1592;
  l1593:;	  G->pos= yypos1592; G->thunkpos= yythunkpos1592;  if (!yy_ListLoose(G)) { goto l1590; }
  }
  l1592:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "BulletList", G->buf+G->pos));
  return 1;
  l1590:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "BulletList", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_OrderedList(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "OrderedList"));
  {  int yypos1595= G->pos, yythunkpos1595= G->thunkpos;  if (!yy_Enumerator(G)) { goto l1594; }  G->pos= yypos1595; G->thunkpos= yythunkpos1595;
  }
  {  int yypos1596= G->pos, yythunkpos1596= G->thunkpos;  if (!yy_ListTight(G)) { goto l1597; }  goto l1596;
  l1597:;	  G->pos= yypos1596; G->thunkpos= yythunkpos1596;  if (!yy_ListLoose(G)) { goto l1594; }
  }
  l1596:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "OrderedList", G->buf+G->pos));
  return 1;
  l1594:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "OrderedList", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Heading(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Heading"));
  {  int yypos1599= G->pos, yythunkpos1599= G->thunkpos;  if (!yy_SetextHeading(G)) { goto l1600; }  goto l1599;
  l1600:;	  G->pos= yypos1599; G->thunkpos= yythunkpos1599;  if (!yy_AtxHeading(G)) { goto l1598; }
  }
  l1599:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Heading", G->buf+G->pos));
  return 1;
  l1598:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Heading", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_HorizontalRule(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "HorizontalRule"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1601;  if (!yy_NonindentSpace(G)) { goto l1601; }
  {  int yypos1602= G->pos, yythunkpos1602= G->thunkpos;  if (!yymatchChar(G, '*')) goto l1603;  if (!yy_Sp(G)) { goto l1603; }  if (!yymatchChar(G, '*')) goto l1603;  if (!yy_Sp(G)) { goto l1603; }  if (!yymatchChar(G, '*')) goto l1603;
  l1604:;	
  {  int yypos1605= G->pos, yythunkpos1605= G->thunkpos;  if (!yy_Sp(G)) { goto l1605; }  if (!yymatchChar(G, '*')) goto l1605;  goto l1604;
  l1605:;	  G->pos= yypos1605; G->thunkpos= yythunkpos1605;
  }  goto l1602;
  l1603:;	  G->pos= yypos1602; G->thunkpos= yythunkpos1602;  if (!yymatchChar(G, '-')) goto l1606;  if (!yy_Sp(G)) { goto l1606; }  if (!yymatchChar(G, '-')) goto l1606;  if (!yy_Sp(G)) { goto l1606; }  if (!yymatchChar(G, '-')) goto l1606;
  l1607:;	
  {  int yypos1608= G->pos, yythunkpos1608= G->thunkpos;  if (!yy_Sp(G)) { goto l1608; }  if (!yymatchChar(G, '-')) goto l1608;  goto l1607;
  l1608:;	  G->pos= yypos1608; G->thunkpos= yythunkpos1608;
  }  goto l1602;
  l1606:;	  G->pos= yypos1602; G->thunkpos= yythunkpos1602;  if (!yymatchChar(G, '_')) goto l1601;  if (!yy_Sp(G)) { goto l1601; }  if (!yymatchChar(G, '_')) goto l1601;  if (!yy_Sp(G)) { goto l1601; }  if (!yymatchChar(G, '_')) goto l1601;
  l1609:;	
  {  int yypos1610= G->pos, yythunkpos1610= G->thunkpos;  if (!yy_Sp(G)) { goto l1610; }  if (!yymatchChar(G, '_')) goto l1610;  goto l1609;
  l1610:;	  G->pos= yypos1610; G->thunkpos= yythunkpos1610;
  }
  }
  l1602:;	  if (!yy_Sp(G)) { goto l1601; }  if (!yy_Newline(G)) { goto l1601; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1601;  yyDo(G, yy_1_HorizontalRule, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "HorizontalRule", G->buf+G->pos));
  return 1;
  l1601:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "HorizontalRule", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Reference(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 3, 0);
  yyprintf((stderr, "%s\n", "Reference"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1611;  if (!yy_LocMarker(G)) { goto l1611; }  yyDo(G, yySet, -3, 0);  if (!yy_NonindentSpace(G)) { goto l1611; }
  {  int yypos1612= G->pos, yythunkpos1612= G->thunkpos;  if (!yymatchString(G, "[]")) goto l1612;  goto l1611;
  l1612:;	  G->pos= yypos1612; G->thunkpos= yythunkpos1612;
  }  if (!yy_Label(G)) { goto l1611; }  yyDo(G, yySet, -2, 0);  if (!yymatchChar(G, ':')) goto l1611;  if (!yy_Spnl(G)) { goto l1611; }  if (!yy_RefSrc(G)) { goto l1611; }  yyDo(G, yySet, -1, 0);  if (!yy_RefTitle(G)) { goto l1611; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1611;  if (!yy_BlankLine(G)) { goto l1611; }
  l1613:;	
  {  int yypos1614= G->pos, yythunkpos1614= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1614; }  goto l1613;
  l1614:;	  G->pos= yypos1614; G->thunkpos= yythunkpos1614;
  }  yyDo(G, yy_1_Reference, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Reference", G->buf+G->pos));  yyDo(G, yyPop, 3, 0);
  return 1;
  l1611:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Reference", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Note(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "Note"));  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_NOTES) )) goto l1615;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1615;  if (!yy_LocMarker(G)) { goto l1615; }  yyDo(G, yySet, -1, 0);  if (!yy_NonindentSpace(G)) { goto l1615; }  if (!yy_RawNoteReference(G)) { goto l1615; }  if (!yymatchChar(G, ':')) goto l1615;  if (!yy_Sp(G)) { goto l1615; }  if (!yy_RawNoteBlock(G)) { goto l1615; }
  l1616:;	
  {  int yypos1617= G->pos, yythunkpos1617= G->thunkpos;
  {  int yypos1618= G->pos, yythunkpos1618= G->thunkpos;  if (!yy_Indent(G)) { goto l1617; }  G->pos= yypos1618; G->thunkpos= yythunkpos1618;
  }  if (!yy_RawNoteBlock(G)) { goto l1617; }  goto l1616;
  l1617:;	  G->pos= yypos1617; G->thunkpos= yythunkpos1617;
  }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1615;  yyDo(G, yy_1_Note, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Note", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1615:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Note", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_DisplayFormula(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "DisplayFormula"));  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_MATH) )) goto l1619;
  {  int yypos1620= G->pos, yythunkpos1620= G->thunkpos;  if (!yy_DisplayFormulaDollar(G)) { goto l1621; }  goto l1620;
  l1621:;	  G->pos= yypos1620; G->thunkpos= yythunkpos1620;  if (!yy_DisplayFormulaRaw(G)) { goto l1619; }
  }
  l1620:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "DisplayFormula", G->buf+G->pos));
  return 1;
  l1619:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "DisplayFormula", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_FencedCodeBlock(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "FencedCodeBlock"));
  {  int yypos1623= G->pos, yythunkpos1623= G->thunkpos;  if (!yy_FencedCodeBlockTick(G)) { goto l1624; }  goto l1623;
  l1624:;	  G->pos= yypos1623; G->thunkpos= yythunkpos1623;  if (!yy_FencedCodeBlockTidle(G)) { goto l1622; }
  }
  l1623:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "FencedCodeBlock", G->buf+G->pos));
  return 1;
  l1622:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "FencedCodeBlock", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Verbatim(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "Verbatim"));  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1625;  if (!yy_LocMarker(G)) { goto l1625; }  yyDo(G, yySet, -1, 0);  if (!yy_VerbatimChunk(G)) { goto l1625; }
  l1626:;	
  {  int yypos1627= G->pos, yythunkpos1627= G->thunkpos;  if (!yy_VerbatimChunk(G)) { goto l1627; }  goto l1626;
  l1627:;	  G->pos= yypos1627; G->thunkpos= yythunkpos1627;
  }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1625;  yyDo(G, yy_1_Verbatim, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "Verbatim", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1625:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Verbatim", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_BlockQuote(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "BlockQuote"));  if (!yy_BlockQuoteRaw(G)) { goto l1628; }  yyDo(G, yySet, -1, 0);  yyDo(G, yy_1_BlockQuote, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "BlockQuote", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1628:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "BlockQuote", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_BlankLine(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "BlankLine"));  if (!yy_Sp(G)) { goto l1629; }  if (!yy_Newline(G)) { goto l1629; }
  yyprintf((stderr, "  ok   %s @ %s\n", "BlankLine", G->buf+G->pos));
  return 1;
  l1629:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "BlankLine", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_FrontMatterEndMark(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "FrontMatterEndMark"));
  {  int yypos1631= G->pos, yythunkpos1631= G->thunkpos;  if (!yymatchString(G, "---")) goto l1632;  goto l1631;
  l1632:;	  G->pos= yypos1631; G->thunkpos= yythunkpos1631;  if (!yymatchString(G, "...")) goto l1630;
  }
  l1631:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "FrontMatterEndMark", G->buf+G->pos));
  return 1;
  l1630:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "FrontMatterEndMark", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_FrontMatterBlock(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "FrontMatterBlock"));
  {  int yypos1634= G->pos, yythunkpos1634= G->thunkpos;  if (!yy_FrontMatterEndMark(G)) { goto l1634; }  goto l1633;
  l1634:;	  G->pos= yypos1634; G->thunkpos= yythunkpos1634;
  }
  l1635:;	
  {  int yypos1636= G->pos, yythunkpos1636= G->thunkpos;
  {  int yypos1637= G->pos, yythunkpos1637= G->thunkpos;  if (!yy_Newline(G)) { goto l1637; }  goto l1636;
  l1637:;	  G->pos= yypos1637; G->thunkpos= yythunkpos1637;
  }  if (!yymatchDot(G)) goto l1636;  goto l1635;
  l1636:;	  G->pos= yypos1636; G->thunkpos= yythunkpos1636;
  }  if (!yy_Newline(G)) { goto l1633; }
  yyprintf((stderr, "  ok   %s @ %s\n", "FrontMatterBlock", G->buf+G->pos));
  return 1;
  l1633:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "FrontMatterBlock", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Newline(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Newline"));
  {  int yypos1639= G->pos, yythunkpos1639= G->thunkpos;  if (!yymatchChar(G, '\n')) goto l1640;  goto l1639;
  l1640:;	  G->pos= yypos1639; G->thunkpos= yythunkpos1639;  if (!yymatchChar(G, '\r')) goto l1638;
  {  int yypos1641= G->pos, yythunkpos1641= G->thunkpos;  if (!yymatchChar(G, '\n')) goto l1641;  goto l1642;
  l1641:;	  G->pos= yypos1641; G->thunkpos= yythunkpos1641;
  }
  l1642:;	
  }
  l1639:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Newline", G->buf+G->pos));
  return 1;
  l1638:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Newline", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_LocMarker(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "LocMarker"));
  {  int yypos1644= G->pos, yythunkpos1644= G->thunkpos;  if (!yymatchDot(G)) goto l1643;  G->pos= yypos1644; G->thunkpos= yythunkpos1644;
  }  yyDo(G, yy_1_LocMarker, G->begin, G->end);
  yyprintf((stderr, "  ok   %s @ %s\n", "LocMarker", G->buf+G->pos));
  return 1;
  l1643:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "LocMarker", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Block(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Block"));
  l1646:;	
  {  int yypos1647= G->pos, yythunkpos1647= G->thunkpos;  if (!yy_BlankLine(G)) { goto l1647; }  goto l1646;
  l1647:;	  G->pos= yypos1647; G->thunkpos= yythunkpos1647;
  }
  {  int yypos1648= G->pos, yythunkpos1648= G->thunkpos;  if (!yy_BlockQuote(G)) { goto l1649; }  goto l1648;
  l1649:;	  G->pos= yypos1648; G->thunkpos= yythunkpos1648;  if (!yy_Verbatim(G)) { goto l1650; }  goto l1648;
  l1650:;	  G->pos= yypos1648; G->thunkpos= yythunkpos1648;  if (!yy_FencedCodeBlock(G)) { goto l1651; }  goto l1648;
  l1651:;	  G->pos= yypos1648; G->thunkpos= yythunkpos1648;  if (!yy_DisplayFormula(G)) { goto l1652; }  goto l1648;
  l1652:;	  G->pos= yypos1648; G->thunkpos= yythunkpos1648;  if (!yy_Note(G)) { goto l1653; }  goto l1648;
  l1653:;	  G->pos= yypos1648; G->thunkpos= yythunkpos1648;  if (!yy_Reference(G)) { goto l1654; }  goto l1648;
  l1654:;	  G->pos= yypos1648; G->thunkpos= yythunkpos1648;  if (!yy_HorizontalRule(G)) { goto l1655; }  goto l1648;
  l1655:;	  G->pos= yypos1648; G->thunkpos= yythunkpos1648;  if (!yy_Heading(G)) { goto l1656; }  goto l1648;
  l1656:;	  G->pos= yypos1648; G->thunkpos= yythunkpos1648;  if (!yy_OrderedList(G)) { goto l1657; }  goto l1648;
  l1657:;	  G->pos= yypos1648; G->thunkpos= yythunkpos1648;  if (!yy_BulletList(G)) { goto l1658; }  goto l1648;
  l1658:;	  G->pos= yypos1648; G->thunkpos= yythunkpos1648;  if (!yy_Table(G)) { goto l1659; }  goto l1648;
  l1659:;	  G->pos= yypos1648; G->thunkpos= yythunkpos1648;  if (!yy_HtmlBlock(G)) { goto l1660; }  goto l1648;
  l1660:;	  G->pos= yypos1648; G->thunkpos= yythunkpos1648;  if (!yy_StyleBlock(G)) { goto l1661; }  goto l1648;
  l1661:;	  G->pos= yypos1648; G->thunkpos= yythunkpos1648;  if (!yy_Para(G)) { goto l1662; }  goto l1648;
  l1662:;	  G->pos= yypos1648; G->thunkpos= yythunkpos1648;  if (!yy_Plain(G)) { goto l1645; }
  }
  l1648:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "Block", G->buf+G->pos));
  return 1;
  l1645:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Block", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_FrontMatter(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;  yyDo(G, yyPush, 1, 0);
  yyprintf((stderr, "%s\n", "FrontMatter"));  if (!yy_LocMarker(G)) { goto l1663; }  yyDo(G, yySet, -1, 0);
  {  int yypos1664= G->pos, yythunkpos1664= G->thunkpos;  yyText(G, G->begin, G->end);  if (!( EXT(pmh_EXT_FRONTMATTER) )) goto l1664;  yyText(G, G->begin, G->end);  if (!(YY_BEGIN)) goto l1664;  if (!yymatchString(G, "---")) goto l1664;  if (!yy_Newline(G)) { goto l1664; }
  l1666:;	
  {  int yypos1667= G->pos, yythunkpos1667= G->thunkpos;  if (!yy_FrontMatterBlock(G)) { goto l1667; }  goto l1666;
  l1667:;	  G->pos= yypos1667; G->thunkpos= yythunkpos1667;
  }  if (!yy_FrontMatterEndMark(G)) { goto l1664; }  if (!yy_Newline(G)) { goto l1664; }  yyText(G, G->begin, G->end);  if (!(YY_END)) goto l1664;  yyDo(G, yy_1_FrontMatter, G->begin, G->end);  goto l1665;
  l1664:;	  G->pos= yypos1664; G->thunkpos= yythunkpos1664;
  }
  l1665:;	
  yyprintf((stderr, "  ok   %s @ %s\n", "FrontMatter", G->buf+G->pos));  yyDo(G, yyPop, 1, 0);
  return 1;
  l1663:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "FrontMatter", G->buf+G->pos));
  return 0;
}
YY_RULE(int) yy_Doc(GREG *G)
{  int yypos0= G->pos, yythunkpos0= G->thunkpos;
  yyprintf((stderr, "%s\n", "Doc"));  if (!yy_FrontMatter(G)) { goto l1668; }
  l1669:;	
  {  int yypos1670= G->pos, yythunkpos1670= G->thunkpos;  if (!yy_Block(G)) { goto l1670; }  goto l1669;
  l1670:;	  G->pos= yypos1670; G->thunkpos= yythunkpos1670;
  }
  yyprintf((stderr, "  ok   %s @ %s\n", "Doc", G->buf+G->pos));
  return 1;
  l1668:;	  G->pos= yypos0; G->thunkpos= yythunkpos0;
  yyprintf((stderr, "  fail %s @ %s\n", "Doc", G->buf+G->pos));
  return 0;
}

#ifndef YY_PART

typedef int (*yyrule)(GREG *G);

YY_PARSE(int) YY_NAME(parse_from)(GREG *G, yyrule yystart)
{
  int yyok;
  if (!G->buflen)
    {
      G->buflen= YY_BUFFER_START_SIZE;
      G->buf= (char *)YY_ALLOC(G->buflen, G->data);
      G->textlen= YY_BUFFER_START_SIZE;
      G->text= (char *)YY_ALLOC(G->textlen, G->data);
      G->thunkslen= YY_STACK_SIZE;
      G->thunks= (yythunk *)YY_ALLOC(sizeof(yythunk) * G->thunkslen, G->data);
      G->valslen= YY_STACK_SIZE;
      G->vals= (YYSTYPE*)YY_ALLOC(sizeof(YYSTYPE) * G->valslen, G->data);
      G->begin= G->end= G->pos= G->limit= G->thunkpos= 0;
    }
  G->pos = 0;
  G->begin= G->end= G->pos;
  G->thunkpos= 0;
  G->val= G->vals;
  yyok= yystart(G);
  if (yyok) yyDone(G);
  yyCommit(G);
  return yyok;
  (void)yyrefill(NULL);
  (void)yymatchDot(NULL);
  (void)yymatchChar(NULL, 0);
  (void)yymatchString(NULL, NULL);
  (void)yymatchClass(NULL, NULL);
  (void)yyDo(NULL, NULL, 0, 0);
  (void)yyText(NULL, 0, 0);
  (void)yyDone(NULL);
  (void)yyCommit(NULL);
  (void)yyAccept(NULL, 0);
  (void)yyPush(NULL, NULL, 0, NULL, NULL);
  (void)yyPop(NULL, NULL, 0, NULL, NULL);
  (void)yySet(NULL, NULL, 0, NULL, NULL);
}

YY_PARSE(int) YY_NAME(parse)(GREG *G)
{
  return YY_NAME(parse_from)(G, yy_Doc);
}

YY_PARSE(GREG *) YY_NAME(parse_new)(YY_XTYPE data)
{
  GREG *G = (GREG *)YY_CALLOC(1, sizeof(GREG), G->data);
  G->data = data;
  return G;
}

YY_PARSE(void) YY_NAME(parse_free)(GREG *G)
{
  YY_FREE(G->buf);
  YY_FREE(G->text);
  YY_FREE(G->thunks);
  YY_FREE(G->vals);
  YY_FREE(G);
}

#endif




/* PEG Markdown Highlight
 * Copyright 2011-2016 Ali Rantakari -- http://hasseg.org
 * Licensed under the GPL2+ and MIT licenses (see LICENSE for more info).
 * 
 * pmh_parser_foot.c
 * 
 * Code to be appended to the end of the parser code generated from the
 * PEG grammar.
 */


static void _parse(parser_data *p_data, yyrule start_rule)
{
    GREG *g = YY_NAME(parse_new)(p_data);
    if (start_rule == NULL)
        YY_NAME(parse)(g);
    else
        YY_NAME(parse_from)(g, start_rule);
    YY_NAME(parse_free)(g);
    
    pmh_PRINTF("\n\n");
}

static void parse_markdown(parser_data *p_data)
{
    pmh_PRINTF("\nPARSING DOCUMENT: ");
    
    _parse(p_data, NULL);
}

static void parse_references(parser_data *p_data)
{
    pmh_PRINTF("\nPARSING REFERENCES: ");
    
    p_data->parsing_only_references = true;
    _parse(p_data, yy_References);
    p_data->parsing_only_references = false;
    
    p_data->references = p_data->head_elems[pmh_REFERENCE];
    p_data->head_elems[pmh_REFERENCE] = NULL;
}

